@@ -1,18 +1,18 @@
-/* pocl.h - global pocl declarations.
+/* pocl.h - global pocl declarations for the host side runtime.
 
    Copyright (c) 2011 Universidad Rey Juan Carlos
-                 2011-2013 Pekka Jääskeläinen / Tampere University of Technology
-  
+                 2011-2019 Pekka Jääskeläinen
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -24,20 +24,24 @@
 
 /**
  * @file pocl.h
- * 
+ *
  * The declarations in this file are such that are used both in the
  * libpocl implementation CL and the kernel compiler. Others should be
- * moved to pocl_cl.h of lib/CL or under the kernel compiler dir. 
+ * moved to pocl_cl.h of lib/CL or under the kernel compiler dir.
  * @todo Check if there are extra declarations here that could be moved.
  */
 #ifndef POCL_H
 #define POCL_H
 
+#ifndef CL_TARGET_OPENCL_VERSION
+#define CL_TARGET_OPENCL_VERSION 220
+#endif
 #include <CL/opencl.h>
 
-#include "pocl_device.h"
 #include "config.h"
 
+#include "pocl_context.h"
+
 /* detects restrict, variadic macros etc */
 #include "pocl_compiler_features.h"
 
@@ -92,19 +96,25 @@ struct _build_program_callback
 
 // Command Queue datatypes
 
+#define POCL_KERNEL_DIGEST_SIZE 16
+typedef uint8_t pocl_kernel_hash_t[POCL_KERNEL_DIGEST_SIZE];
+
 // clEnqueueNDRangeKernel
 typedef struct
 {
-  char *tmp_dir; 
-  pocl_workgroup wg;
+  void *hash;
+  void *wg; /* The work group function ptr. Device specific. */
   cl_kernel kernel;
-  size_t local_x;
-  size_t local_y;
-  size_t local_z;
+  /* The launch data that can be passed to the kernel execution environment. */
   struct pocl_context pc;
   struct pocl_argument *arguments;
-  /* Can be used to store/cache device-specific data. */
-  void **device_data;
+  /* Can be used to store/cache arbitrary device-specific data. */
+  void *device_data;
+  /* If set to 1, disallow any work-group function specialization. */
+  int force_generic_wg_func;
+  /* If set to 1, disallow "small grid" WG function specialization. */
+  int force_large_grid_wg_func;
+  unsigned device_i;
 } _cl_command_run;
 
 // clEnqueueNativeKernel
@@ -349,50 +359,31 @@ struct _cl_command_node
 {
   _cl_command_t command;
   cl_command_type type;
-  _cl_command_node * volatile next; // for linked-list storage
-  _cl_command_node * volatile prev;
+  _cl_command_node *next; // for linked-list storage
+  _cl_command_node *prev;
   cl_event event;
-  const cl_event *volatile event_wait_list;
+  const cl_event *event_wait_list;
   cl_device_id device;
-  volatile cl_int ready;
+  /* The index of the targeted device in the platform's device list. */
+  unsigned device_i;
+  cl_int ready;
 };
 
+#ifndef LLVM_9_0
+#define LLVM_OLDER_THAN_9_0 1
+
+#ifndef LLVM_8_0
+#define LLVM_OLDER_THAN_8_0 1
+
 #ifndef LLVM_7_0
 #define LLVM_OLDER_THAN_7_0 1
 
 #ifndef LLVM_6_0
 #define LLVM_OLDER_THAN_6_0 1
 
-#ifndef LLVM_5_0
-#define LLVM_OLDER_THAN_5_0 1
-
-#ifndef LLVM_4_0
-#define LLVM_OLDER_THAN_4_0 1
-
-#ifndef LLVM_3_9
-#define LLVM_OLDER_THAN_3_9 1
-
-#ifndef LLVM_3_8
-#define LLVM_OLDER_THAN_3_8 1
-
-#ifndef LLVM_3_7
-#define LLVM_OLDER_THAN_3_7 1
-
-#ifndef LLVM_3_6
-#define LLVM_OLDER_THAN_3_6 1
-
 #endif
 #endif
 #endif
 #endif
-#endif
-#endif
-#endif
-#endif
-
-
-#if (defined LLVM_4_0)
-# define LLVM_OLDER_THAN_5_0 1
-#endif
 
 #endif /* POCL_H */
