@@ -24,6 +24,8 @@
 #ifndef POCL_CACHE_H
 #define POCL_CACHE_H
 
+#include "pocl_cl.h"
+
 /* The filename in which the work group (parallelizable) kernel LLVM bc is stored in
    the kernel's temp dir. */
 #define POCL_PARALLEL_BC_FILENAME   "/parallel.bc"
@@ -32,7 +34,6 @@
 extern "C" {
 #endif
 
-#define CL_USE_DEPRECATED_OPENCL_1_1_APIS
 #include <CL/cl.h>
 
 #ifdef __GNUC__
@@ -87,33 +88,25 @@ int pocl_cache_write_descriptor(cl_program   program,
                                 const char*  content,
                                 size_t       size);
 
-void pocl_cache_kernel_cachedir_path (char* kernel_cachedir_path,
-                                      cl_program program,
-                                      unsigned device_i,
-                                      cl_kernel kernel,
-                                      const char* append_str,
-                                      size_t local_x,
-                                      size_t local_y,
-                                      size_t local_z);
-
-
-int pocl_cache_write_kernel_parallel_bc(void*        bc,
-                                        cl_program   program,
-                                        unsigned     device_i,
-                                        cl_kernel    kernel,
-                                        size_t       local_x,
-                                        size_t       local_y,
-                                        size_t       local_z);
+void pocl_cache_kernel_cachedir_path (char *kernel_cachedir_path,
+                                      cl_program program, unsigned device_i,
+                                      cl_kernel kernel, const char *append_str,
+                                      _cl_command_node *command,
+                                      int specialize);
 
+int pocl_cache_write_kernel_parallel_bc (void *bc, cl_program program,
+                                         int device_i, cl_kernel kernel,
+                                         _cl_command_node *command,
+                                         int specialize);
 
 // required by pocl_binary.c
 
-void pocl_cache_program_path(char*        path,
-                             cl_program   program,
-                             unsigned     device_i);
+void pocl_cache_program_path (char *path, cl_program program,
+                              unsigned device_i);
 
-void pocl_cache_kernel_cachedir(char* kernel_cachedir_path, cl_program   program,
-                                unsigned device_i, const char* kernel_name);
+void pocl_cache_kernel_cachedir (char *kernel_cachedir_path,
+                                 cl_program program, unsigned device_i,
+                                 const char *kernel_name);
 
 // these two required by llvm API
 
@@ -121,14 +114,15 @@ void pocl_cache_program_bc_path(char*       program_bc_path,
                                cl_program   program,
                                unsigned     device_i);
 
-void pocl_cache_work_group_function_path(char* parallel_bc_path, cl_program program,
-                              unsigned device_i, cl_kernel kernel,
-                              size_t local_x, size_t local_y, size_t local_z);
+void pocl_cache_work_group_function_path (char *parallel_bc_path,
+                                          cl_program program,
+                                          unsigned device_i, cl_kernel kernel,
+                                          _cl_command_node *command,
+                                          int specialize);
 
-void pocl_cache_final_binary_path(char* final_binary_path, cl_program program,
-                               unsigned device_i, cl_kernel kernel,
-                               size_t local_x, size_t local_y,
-                               size_t local_z);
+void pocl_cache_final_binary_path (char *final_binary_path, cl_program program,
+                                   unsigned device_i, cl_kernel kernel,
+                                   _cl_command_node *command, int specialize);
 
 #ifdef __GNUC__
 #pragma GCC visibility pop
