@@ -1,13 +1,13 @@
 /* OpenCL runtime library: clEnqueueNDRangeKernel()
 
    Copyright (c) 2011 Universidad Rey Juan Carlos and
-                 2012-2013 Pekka Jääskeläinen / Tampere University of Technology
+                 2012-2019 Pekka Jääskeläinen
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
+   of this software and associated documentation files (the "Software"), to
+   deal in the Software without restriction, including without limitation the
+   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+   sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
 
    The above copyright notice and this permission notice shall be included in
@@ -17,9 +17,9 @@
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+   IN THE SOFTWARE.
 */
 
 #include "config.h"
@@ -29,6 +29,8 @@
 #include "pocl_cache.h"
 #include "utlist.h"
 #include "pocl_binary.h"
+#include "pocl_context.h"
+
 #ifndef _MSC_VER
 #  include <unistd.h>
 #else
@@ -104,12 +106,11 @@ POname(clEnqueueNDRangeKernel)(cl_command_queue command_queue,
   unsigned i;
   int errcode = 0;
   cl_device_id realdev = NULL;
-  struct pocl_context pc;
   _cl_command_node *command_node;
   /* alloc from stack to avoid malloc. num_args is the absolute max needed */
-  cl_mem mem_list[kernel->num_args + 1];
+  cl_mem mem_list[kernel->meta->num_args + 1];
   /* reserve space for potential buffer migrate events */
-  cl_event new_event_wait_list[num_events_in_wait_list + kernel->num_args + 1];
+  cl_event new_event_wait_list[num_events_in_wait_list + kernel->meta->num_args + 1];
 
   POCL_RETURN_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
 
@@ -154,15 +155,11 @@ POname(clEnqueueNDRangeKernel)(cl_command_queue command_queue,
   POCL_RETURN_ERROR_COND((global_x == 0 || global_y == 0 || global_z == 0),
     CL_INVALID_GLOBAL_WORK_SIZE);
 
-  for (i = 0; i < kernel->num_args; i++)
-    {
-      POCL_RETURN_ERROR_ON((!kernel->arg_info[i].is_set),
-        CL_INVALID_KERNEL_ARGS, "The %i-th kernel argument is not set!\n", i);
-    }
-
   max_local_x = command_queue->device->max_work_item_sizes[0];
-  max_local_y = command_queue->device->max_work_item_sizes[1];
-  max_local_z = command_queue->device->max_work_item_sizes[2];
+  max_local_y
+      = work_dim > 1 ? command_queue->device->max_work_item_sizes[1] : 1;
+  max_local_z
+      = work_dim > 2 ? command_queue->device->max_work_item_sizes[2] : 1;
   max_group_size = command_queue->device->max_work_group_size;
 
   if (local_work_size != NULL)
@@ -206,15 +203,15 @@ POname(clEnqueueNDRangeKernel)(cl_command_queue command_queue,
    * work size _must_ be specified, and it _must_ match the attribute
    * specification
    */
-  if (kernel->reqd_wg_size != NULL &&
-      kernel->reqd_wg_size[0] > 0 &&
-      kernel->reqd_wg_size[1] > 0 &&
-      kernel->reqd_wg_size[2] > 0)
+  if (kernel->meta->reqd_wg_size[0] > 0 &&
+      kernel->meta->reqd_wg_size[1] > 0 &&
+      kernel->meta->reqd_wg_size[2] > 0)
     {
-      POCL_RETURN_ERROR_COND((local_work_size == NULL ||
-          local_x != kernel->reqd_wg_size[0] ||
-          local_y != kernel->reqd_wg_size[1] ||
-          local_z != kernel->reqd_wg_size[2]), CL_INVALID_WORK_GROUP_SIZE);
+      POCL_RETURN_ERROR_COND ((local_work_size == NULL
+                               || local_x != kernel->meta->reqd_wg_size[0]
+                               || local_y != kernel->meta->reqd_wg_size[1]
+                               || local_z != kernel->meta->reqd_wg_size[2]),
+                              CL_INVALID_WORK_GROUP_SIZE);
     }
   /* otherwise, if the local work size was not specified find the optimal one.
    * Note that at some point we also checked for local > global. This doesn't
@@ -279,12 +276,23 @@ POname(clEnqueueNDRangeKernel)(cl_command_queue command_queue,
 
       /*                      /------- K ------\  /-------- k' -------\  */
       local_x = upper_divisor (global_x / grain_x, max_local_x / grain_x);
-      local_y = upper_divisor (global_y / grain_y, max_local_y / grain_y);
-      local_z = upper_divisor (global_z / grain_z, max_local_z / grain_z);
-
       local_x *= grain_x;
-      local_y *= grain_y;
-      local_z *= grain_z;
+
+      if (work_dim > 1)
+        {
+          local_y = upper_divisor (global_y / grain_y, max_local_y / grain_y);
+          local_y *= grain_y;
+        }
+      else
+        local_y = 1;
+
+      if (work_dim > 2)
+        {
+          local_z = upper_divisor (global_z / grain_z, max_local_z / grain_z);
+          local_z *= grain_z;
+        }
+      else
+        local_z = 1;
 
       /* So we now have the largest possible local sizes that divide the global
        * sizes while being multiples of the grain size.
@@ -455,26 +463,49 @@ if (local_##c1 > 1 && local_##c1 <= local_##c2 && local_##c1 <= local_##c3 && \
   assert (global_y % local_y == 0);
   assert (global_z % local_z == 0);
 
-  char cachedir[POCL_FILENAME_LENGTH];
-  int realdev_i = pocl_cl_device_to_index (kernel->program, realdev);
-  assert (realdev_i >= 0);
-  pocl_cache_kernel_cachedir_path (cachedir, kernel->program,
-                                   realdev_i, kernel, "",
-                                   local_x, local_y, local_z);
-
   b_migrate_count = 0;
   buffer_count = 0;
 
   /* count mem objects and enqueue needed mem migrations */
-  for (i = 0; i < kernel->num_args; ++i)
+  for (i = 0; i < kernel->meta->num_args; ++i)
     {
+      struct pocl_argument_info *a = &kernel->meta->arg_info[i];
       struct pocl_argument *al = &(kernel->dyn_arguments[i]);
-      if (kernel->arg_info[i].type == POCL_ARG_TYPE_IMAGE ||
-          (!kernel->arg_info[i].is_local
-           && kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER
-           && al->value != NULL))
+
+      POCL_GOTO_ERROR_ON ((!al->is_set),
+                            CL_INVALID_KERNEL_ARGS,
+                            "The %i-th kernel argument is not set!\n", i);
+
+
+      if (a->type == POCL_ARG_TYPE_IMAGE
+          || (!ARGP_IS_LOCAL (a) && a->type == POCL_ARG_TYPE_POINTER
+              && al->value != NULL))
         {
           cl_mem buf = *(cl_mem *) (al->value);
+
+          if (a->type == POCL_ARG_TYPE_IMAGE)
+            {
+              POCL_GOTO_ON_UNSUPPORTED_IMAGE (buf, command_queue->device);
+            }
+          else
+            {
+              POCL_GOTO_ON_SUB_MISALIGN (buf, command_queue);
+
+              if (buf->parent != NULL)
+              {
+                  *(cl_mem *)(al->value) = buf->parent;
+                  al->offset = buf->origin;
+                  buf = buf->parent;
+              }
+              else
+                al->offset = 0;
+
+              POCL_GOTO_ERROR_ON ((buf->size > command_queue->device->max_mem_alloc_size),
+                                    CL_OUT_OF_RESOURCES,
+                                    "ARG %u: buffer is larger than "
+                                    "device's MAX_MEM_ALLOC_SIZE\n", i);
+            }
+
           mem_list[buffer_count++] = buf;
           POname(clRetainMemObject) (buf);
           /* if buffer has no owner,
@@ -516,33 +547,37 @@ if (local_##c1 > 1 && local_##c1 <= local_##c2 && local_##c1 <= local_##c3 && \
   if (errcode != CL_SUCCESS)
     goto ERROR;
 
-  pc.work_dim = work_dim;
-  pc.num_groups[0] = global_x / local_x;
-  pc.num_groups[1] = global_y / local_y;
-  pc.num_groups[2] = global_z / local_z;
-  pc.global_offset[0] = offset_x;
-  pc.global_offset[1] = offset_y;
-  pc.global_offset[2] = offset_z;
 
   command_node->type = CL_COMMAND_NDRANGE_KERNEL;
-  command_node->command.run.tmp_dir = strdup (cachedir);
   command_node->command.run.kernel = kernel;
-  command_node->command.run.pc = pc;
-  command_node->command.run.local_x = local_x;
-  command_node->command.run.local_y = local_y;
-  command_node->command.run.local_z = local_z;
+  command_node->command.run.pc.local_size[0] = local_x;
+  command_node->command.run.pc.local_size[1] = local_y;
+  command_node->command.run.pc.local_size[2] = local_z;
+  command_node->command.run.pc.work_dim = work_dim;
+  command_node->command.run.pc.num_groups[0] = global_x / local_x;
+  command_node->command.run.pc.num_groups[1] = global_y / local_y;
+  command_node->command.run.pc.num_groups[2] = global_z / local_z;
+  command_node->command.run.pc.global_offset[0] = offset_x;
+  command_node->command.run.pc.global_offset[1] = offset_y;
+  command_node->command.run.pc.global_offset[2] = offset_z;
+
+  int realdev_i = pocl_cl_device_to_index (kernel->program, realdev);
+  assert (realdev_i >= 0);
+  command_node->device_i = (unsigned)realdev_i;
+  command_node->command.run.hash = kernel->meta->build_hash[realdev_i];
 
   /* Copy the currently set kernel arguments because the same kernel
      object can be reused for new launches with different arguments. */
   command_node->command.run.arguments =
-    (struct pocl_argument *) malloc ((kernel->num_args + kernel->num_locals) *
+    (struct pocl_argument *) malloc ((kernel->meta->num_args) *
                                      sizeof (struct pocl_argument));
 
-  for (i = 0; i < kernel->num_args + kernel->num_locals; ++i)
+  for (i = 0; i < kernel->meta->num_args; ++i)
     {
       struct pocl_argument *arg = &command_node->command.run.arguments[i];
       size_t arg_alloc_size = kernel->dyn_arguments[i].size;
       arg->size = arg_alloc_size;
+      arg->offset = kernel->dyn_arguments[i].offset;
 
       if (kernel->dyn_arguments[i].value == NULL)
         {
@@ -553,7 +588,7 @@ if (local_##c1 > 1 && local_##c1 <= local_##c2 && local_##c1 <= local_##c3 && \
           /* FIXME: this is a cludge to determine an acceptable alignment,
            * we should probably extract the argument alignment from the
            * LLVM bytecode during kernel header generation. */
-          size_t arg_alignment = pocl_size_ceil2(arg_alloc_size);
+          size_t arg_alignment = pocl_size_ceil2 (arg_alloc_size);
           if (arg_alignment >= MAX_EXTENDED_ALIGNMENT)
             arg_alignment = MAX_EXTENDED_ALIGNMENT;
           if (arg_alloc_size < arg_alignment)
