@@ -46,6 +46,8 @@ POname(clEnqueueWriteBuffer)(cl_command_queue command_queue,
 
   POCL_RETURN_ERROR_COND((buffer == NULL), CL_INVALID_MEM_OBJECT);
 
+  POCL_RETURN_ON_SUB_MISALIGN (buffer, command_queue);
+
   POCL_RETURN_ERROR_ON((command_queue->context != buffer->context),
     CL_INVALID_CONTEXT, "buffer and command_queue are not from the same context\n");
 
@@ -60,6 +62,12 @@ POname(clEnqueueWriteBuffer)(cl_command_queue command_queue,
   if (pocl_buffer_boundcheck (buffer, offset, size) != CL_SUCCESS)
     return CL_INVALID_VALUE;
 
+  POCL_CONVERT_SUBBUFFER_OFFSET (buffer, offset);
+
+  POCL_RETURN_ERROR_ON((buffer->size > command_queue->device->max_mem_alloc_size),
+                        CL_OUT_OF_RESOURCES,
+                        "buffer is larger than device's MAX_MEM_ALLOC_SIZE\n");
+
   errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
                                         event_wait_list);
   if (errcode != CL_SUCCESS)
@@ -82,7 +90,7 @@ POname(clEnqueueWriteBuffer)(cl_command_queue command_queue,
   POname(clRetainMemObject) (buffer);
   buffer->owning_device = command_queue->device;
 
-  pocl_command_enqueue(command_queue, cmd);
+  pocl_command_enqueue (command_queue, cmd);
 
   if (blocking_write)
     POname(clFinish) (command_queue);
