@@ -59,6 +59,8 @@ POname(clGetDeviceInfo)(cl_device_id   device,
                 void *         param_value,
                 size_t *       param_value_size_ret) CL_API_SUFFIX__VERSION_1_0
 {
+  POCL_RETURN_ERROR_COND ((device == NULL), CL_INVALID_DEVICE);
+
   switch (param_name)
   {
   case CL_DEVICE_IMAGE_SUPPORT:
@@ -94,7 +96,7 @@ POname(clGetDeviceInfo)(cl_device_id   device,
     }
   case CL_DEVICE_MAX_WORK_ITEM_SIZES:
     {
-      /* We allocate a 3-elementa array for this in pthread.c */
+      /* We allocate a 3-element array for this in pthread.c */
       typedef struct { size_t size[3]; } size_t_3;
       POCL_RETURN_GETINFO(size_t_3, *(size_t_3 const *)device->max_work_item_sizes);
     }
@@ -150,8 +152,9 @@ POname(clGetDeviceInfo)(cl_device_id   device,
 
   case CL_DEVICE_MAX_PARAMETER_SIZE:
     POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(size_t, device->max_parameter_size);
-  case CL_DEVICE_MEM_BASE_ADDR_ALIGN               : 
-    POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_uint, device->mem_base_addr_align);
+  case CL_DEVICE_MEM_BASE_ADDR_ALIGN               :
+    POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK (
+        cl_uint, (device->mem_base_addr_align * 8));
   case CL_DEVICE_MIN_DATA_TYPE_ALIGN_SIZE          : 
     POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_uint, device->min_data_type_align_size);
   case CL_DEVICE_SINGLE_FP_CONFIG                  :
@@ -181,11 +184,7 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_AVAILABLE                         :
     POCL_RETURN_GETINFO(cl_bool, device->available);
   case CL_DEVICE_COMPILER_AVAILABLE                :
-#ifdef OCS_AVAILABLE
     POCL_RETURN_GETINFO(cl_bool, device->compiler_available);
-#else
-    POCL_RETURN_GETINFO(cl_bool, CL_FALSE);
-#endif
   case CL_DEVICE_LINKER_AVAILABLE                  :
     /* TODO currently we return the same availability as the compiler,
      * since if the compiler is available the linker MUST be available
@@ -322,6 +321,8 @@ POname(clGetDeviceInfo)(cl_device_id   device,
     POCL_RETURN_GETINFO(size_t, device->global_var_pref_size);
   case CL_DEVICE_MAX_GLOBAL_VARIABLE_SIZE:
     POCL_RETURN_GETINFO(size_t, device->global_var_max_size);
+  case CL_DEVICE_IL_VERSION:
+    POCL_RETURN_GETINFO_STR (device->spirv_version);
   }
   return CL_INVALID_VALUE;
 }
