@@ -51,7 +51,7 @@ POname(clReleaseEvent)(cl_event event) CL_API_SUFFIX__VERSION_1_0
           POCL_MEM_FREE (p);
         }
 
-      POCL_MSG_PRINT_REFCOUNTS ("Free event %d\n", event->id);
+      POCL_MSG_PRINT_REFCOUNTS ("Free event %d | %p\n", event->id, event);
       if (event->command_type != CL_COMMAND_USER &&
           event->queue->device->ops->free_event_data)
         event->queue->device->ops->free_event_data(event);
