@@ -1,28 +1,29 @@
 /* basic.c - a minimalistic pocl device driver layer implementation
 
    Copyright (c) 2011-2013 Universidad Rey Juan Carlos and
-                 2011-2014 Pekka Jääskeläinen / Tampere University of Technology
-   
+                 2011-2019 Pekka Jääskeläinen
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
+   of this software and associated documentation files (the "Software"), to
+   deal in the Software without restriction, including without limitation the
+   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+   sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+   IN THE SOFTWARE.
 */
 
 #include "config.h"
+#include "config2.h"
 #include "basic.h"
 #include "cpuinfo.h"
 #include "topology/pocl_topology.h"
@@ -32,6 +33,7 @@
 #include "pocl_util.h"
 
 #include <assert.h>
+#include <limits.h>
 #include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,11 +43,16 @@
 #include "pocl_cache.h"
 #include "pocl_timing.h"
 #include "pocl_file_util.h"
+#include "pocl_workgroup_func.h"
 
 #ifdef OCS_AVAILABLE
 #include "pocl_llvm.h"
 #endif
 
+#ifndef HAVE_LIBDL
+#error Basic driver requires DL library
+#endif
+
 /* default WG size in each dimension & total WG size.
  * this should be reasonable for CPU */
 #define DEFAULT_WG_SIZE 4096
@@ -53,9 +60,7 @@
 struct data {
   /* Currently loaded kernel. */
   cl_kernel current_kernel;
-  /* Loaded kernel dynamic library handle. */
-  lt_dlhandle current_dlhandle;
-  
+
   /* List of commands ready to be executed */
   _cl_command_node * volatile ready_list;
   /* List of commands not yet ready to be executed */
@@ -124,10 +129,9 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->unmap_mem = pocl_basic_unmap_mem;
   ops->run = pocl_basic_run;
   ops->run_native = pocl_basic_run_native;
-  ops->get_timer_value = pocl_basic_get_timer_value;
   ops->join = pocl_basic_join;
   ops->submit = pocl_basic_submit;
-  ops->broadcast = pocl_basic_broadcast;
+  ops->broadcast = pocl_broadcast;
   ops->notify = pocl_basic_notify;
   ops->flush = pocl_basic_flush;
   ops->build_hash = pocl_basic_build_hash;
@@ -141,7 +145,6 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->svm_copy = pocl_basic_svm_copy;
   ops->svm_fill = pocl_basic_svm_fill;
 
-  ops->get_supported_image_formats = pocl_basic_get_supported_image_formats;
   ops->create_image = NULL;
   ops->free_image = NULL;
   ops->create_sampler = NULL;
@@ -170,11 +173,17 @@ pocl_basic_build_hash (cl_device_id device)
 
 static cl_device_partition_property basic_partition_properties[1] = { 0 };
 
+
+static const char *final_ld_flags[] =
+  {"-nostartfiles", HOST_LD_FLAGS_ARRAY, NULL};
+
 void
 pocl_init_cpu_device_infos (cl_device_id dev)
 {
+  size_t i;
   dev->type = CL_DEVICE_TYPE_CPU;
   dev->max_work_item_dimensions = 3;
+  dev->final_linkage_flags = final_ld_flags;
 
   SETUP_DEVICE_CL_VERSION(HOST_DEVICE_CL_VERSION_MAJOR, HOST_DEVICE_CL_VERSION_MINOR)
   /*
@@ -227,22 +236,31 @@ pocl_init_cpu_device_infos (cl_device_id dev)
 
 #endif
 
-  dev->address_bits = POCL_DEVICE_ADDRESS_BITS;
+  dev->grid_width_specialization_limit = USHRT_MAX;
+  dev->address_bits = HOST_DEVICE_ADDRESS_BITS;
   dev->image_support = CL_TRUE;
   /* Use the minimum values until we get a more sensible upper limit from
      somewhere. */
-  dev->max_read_image_args = dev->max_write_image_args = dev->max_read_write_image_args = 128;
+  dev->max_read_image_args = dev->max_write_image_args
+      = dev->max_read_write_image_args = 128;
   dev->image2d_max_width = dev->image2d_max_height = 8192;
   dev->image3d_max_width = dev->image3d_max_height = dev->image3d_max_depth = 2048;
+  dev->max_samplers = 16;
+
+  for (i = 0; i < NUM_OPENCL_IMAGE_TYPES; ++i)
+    {
+      dev->num_image_formats[i]
+          = sizeof (supported_image_formats) / sizeof (cl_image_format);
+      dev->image_formats[i] = supported_image_formats;
+    }
+
   dev->image_max_buffer_size = 65536;
   dev->image_max_array_size = 2048;
-  dev->max_samplers = 16;
   dev->max_constant_args = 8;
-
   dev->max_mem_alloc_size = 0;
   dev->max_parameter_size = 1024;
-  dev->min_data_type_align_size = MAX_EXTENDED_ALIGNMENT; // this is in bytes
-  dev->mem_base_addr_align = MAX_EXTENDED_ALIGNMENT*8; // this is in bits
+  dev->min_data_type_align_size = MAX_EXTENDED_ALIGNMENT;
+  dev->mem_base_addr_align = MAX_EXTENDED_ALIGNMENT;
   dev->half_fp_config = 0;
   dev->single_fp_config = CL_FP_ROUND_TO_NEAREST | CL_FP_INF_NAN;
 #ifdef __x86_64__
@@ -285,6 +303,7 @@ pocl_init_cpu_device_infos (cl_device_id dev)
   dev->available = CL_TRUE;
   dev->compiler_available = CL_TRUE;
   dev->spmd = CL_FALSE;
+  dev->arg_buffer_launcher = CL_FALSE;
   dev->workgroup_pass = CL_TRUE;
   dev->execution_capabilities = CL_EXEC_KERNEL | CL_EXEC_NATIVE_KERNEL;
   dev->platform = 0;
@@ -337,6 +356,7 @@ pocl_init_cpu_device_infos (cl_device_id dev)
   dev->extensions = HOST_DEVICE_EXTENSIONS;
   dev->has_64bit_long = 1;
   dev->autolocals_to_args = 1;
+  dev->device_alloca_locals = 0;
 
 #ifdef OCS_AVAILABLE
 
@@ -347,9 +367,7 @@ pocl_init_cpu_device_infos (cl_device_id dev)
   dev->llvm_cpu = get_llvm_cpu_name ();
 #endif
 
-  if(dev->llvm_cpu && (!strcmp(dev->llvm_cpu, "(unknown)")))
-    dev->llvm_cpu = OCL_KERNEL_TARGET_CPU;
-
+  dev->spirv_version = "SPIR-V_1.2";
 #else /* No compiler, no CPU info */
   dev->llvm_cpu = NULL;
   dev->llvm_target_triplet = "";
@@ -390,7 +408,6 @@ pocl_basic_init (unsigned j, cl_device_id device, const char* parameters)
     return CL_OUT_OF_HOST_MEMORY;
 
   d->current_kernel = NULL;
-  d->current_dlhandle = 0;
   device->data = d;
 
   pocl_init_cpu_device_infos (device);
@@ -541,17 +558,15 @@ pocl_basic_write (void *data,
   memcpy ((char *)device_ptr + offset, host_ptr, size);
 }
 
-
 void
-pocl_basic_run 
-(void *data, 
- _cl_command_node* cmd)
+pocl_basic_run (void *data, _cl_command_node *cmd)
 {
   struct data *d;
   struct pocl_argument *al;
   size_t x, y, z;
   unsigned i;
   cl_kernel kernel = cmd->command.run.kernel;
+  pocl_kernel_metadata_t *meta = kernel->meta;
   struct pocl_context *pc = &cmd->command.run.pc;
 
   assert (data != NULL);
@@ -559,46 +574,62 @@ pocl_basic_run
 
   d->current_kernel = kernel;
 
-  void **arguments = (void**)malloc(
-      sizeof(void*) * (kernel->num_args + kernel->num_locals)
-    );
+  void **arguments = (void **)malloc (sizeof (void *)
+                                      * (meta->num_args + meta->num_locals));
 
   /* Process the kernel arguments. Convert the opaque buffer
-     pointers to real device pointers, allocate dynamic local 
+     pointers to real device pointers, allocate dynamic local
      memory buffers, etc. */
-  for (i = 0; i < kernel->num_args; ++i)
+  for (i = 0; i < meta->num_args; ++i)
     {
       al = &(cmd->command.run.arguments[i]);
-      if (kernel->arg_info[i].is_local)
+      if (ARG_IS_LOCAL (meta->arg_info[i]))
         {
-          arguments[i] = malloc (sizeof (void *));
-          *(void **)(arguments[i]) = pocl_aligned_malloc(MAX_EXTENDED_ALIGNMENT, al->size);
+          if (cmd->device->device_alloca_locals)
+            {
+              /* Local buffers are allocated in the device side work-group
+                 launcher. Let's pass only the sizes of the local args in
+                 the arg buffer. */
+              assert (sizeof (size_t) == sizeof (void *));
+              arguments[i] = (void *)al->size;
+            }
+          else
+            {
+              arguments[i] = malloc (sizeof (void *));
+              *(void **)(arguments[i]) =
+                pocl_aligned_malloc(MAX_EXTENDED_ALIGNMENT, al->size);
+            }
         }
-      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER)
+      else if (meta->arg_info[i].type == POCL_ARG_TYPE_POINTER)
         {
-          /* It's legal to pass a NULL pointer to clSetKernelArguments. In 
+          /* It's legal to pass a NULL pointer to clSetKernelArguments. In
              that case we must pass the same NULL forward to the kernel.
              Otherwise, the user must have created a buffer with per device
              pointers stored in the cl_mem. */
+          arguments[i] = malloc (sizeof (void *));
           if (al->value == NULL)
             {
-              arguments[i] = malloc (sizeof (void *));
               *(void **)arguments[i] = NULL;
             }
           else
-            arguments[i] = &((*(cl_mem *) (al->value))->device_ptrs[cmd->device->dev_id].mem_ptr);
+            {
+              cl_mem m = (*(cl_mem *)(al->value));
+              void *ptr = m->device_ptrs[cmd->device->dev_id].mem_ptr;
+              *(void **)arguments[i] = (char *)ptr + al->offset;
+            }
         }
-      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_IMAGE)
+      else if (meta->arg_info[i].type == POCL_ARG_TYPE_IMAGE)
         {
           dev_image_t di;
           fill_dev_image_t (&di, al, cmd->device);
 
-          void* devptr = pocl_aligned_malloc(MAX_EXTENDED_ALIGNMENT,  sizeof(dev_image_t));
+          void *devptr = pocl_aligned_malloc (MAX_EXTENDED_ALIGNMENT,
+                                              sizeof (dev_image_t));
           arguments[i] = malloc (sizeof (void *));
           *(void **)(arguments[i]) = devptr;
           memcpy (devptr, &di, sizeof (dev_image_t));
         }
-      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_SAMPLER)
+      else if (meta->arg_info[i].type == POCL_ARG_TYPE_SAMPLER)
         {
           dev_sampler_t ds;
           fill_dev_sampler_t(&ds, al);
@@ -610,24 +641,22 @@ pocl_basic_run
           arguments[i] = al->value;
         }
     }
-  for (i = kernel->num_args;
-       i < kernel->num_args + kernel->num_locals;
-       ++i)
-    {
-      al = &(cmd->command.run.arguments[i]);
-      arguments[i] = malloc (sizeof (void *));
-      *(void **)(arguments[i]) = pocl_aligned_malloc(MAX_EXTENDED_ALIGNMENT, al->size);
-    }
 
-  pc->local_size[0] = cmd->command.run.local_x;
-  pc->local_size[1] = cmd->command.run.local_y;
-  pc->local_size[2] = cmd->command.run.local_z;
+  if (!cmd->device->device_alloca_locals)
+    for (i = 0; i < meta->num_locals; ++i)
+      {
+        size_t s = meta->local_sizes[i];
+        size_t j = meta->num_args + i;
+        arguments[j] = malloc (sizeof (void *));
+        void *pp = pocl_aligned_malloc (MAX_EXTENDED_ALIGNMENT, s);
+        *(void **)(arguments[j]) = pp;
+      }
 
   pc->printf_buffer = d->printf_buffer;
   assert (pc->printf_buffer != NULL);
   pc->printf_buffer_capacity = cmd->device->printf_buffer_size;
   assert (pc->printf_buffer_capacity > 0);
-  size_t position = 0;
+  uint32_t position = 0;
   pc->printf_buffer_position = &position;
 
   unsigned rm = pocl_save_rm ();
@@ -636,20 +665,10 @@ pocl_basic_run
   pocl_set_ftz (kernel->program->flush_denorms);
 
   for (z = 0; z < pc->num_groups[2]; ++z)
-    {
-      for (y = 0; y < pc->num_groups[1]; ++y)
-        {
-          for (x = 0; x < pc->num_groups[0]; ++x)
-            {
-              pc->group_id[0] = x;
-              pc->group_id[1] = y;
-              pc->group_id[2] = z;
-
-              cmd->command.run.wg (arguments, pc);
-
-            }
-        }
-    }
+    for (y = 0; y < pc->num_groups[1]; ++y)
+      for (x = 0; x < pc->num_groups[0]; ++x)
+        ((pocl_workgroup_func) cmd->command.run.wg)
+	  ((uint8_t *)arguments, (uint8_t *)pc, x, y, z);
 
   pocl_restore_rm (rm);
   pocl_restore_ftz (ftz);
@@ -660,41 +679,47 @@ pocl_basic_run
       position = 0;
     }
 
-  for (i = 0; i < kernel->num_args; ++i)
+  for (i = 0; i < meta->num_args; ++i)
     {
-      if (kernel->arg_info[i].is_local)
+      if (ARG_IS_LOCAL (meta->arg_info[i]))
         {
-          POCL_MEM_FREE(*(void **)(arguments[i]));
-          POCL_MEM_FREE(arguments[i]);
+          if (!cmd->device->device_alloca_locals)
+            {
+              POCL_MEM_FREE(*(void **)(arguments[i]));
+              POCL_MEM_FREE(arguments[i]);
+            }
+          else
+            {
+              /* Device side local space allocation has deallocation via stack
+                 unwind. */
+            }
         }
-      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_IMAGE ||
-                kernel->arg_info[i].type == POCL_ARG_TYPE_SAMPLER)
+      else if (meta->arg_info[i].type == POCL_ARG_TYPE_IMAGE
+               || meta->arg_info[i].type == POCL_ARG_TYPE_SAMPLER)
         {
-          if (kernel->arg_info[i].type != POCL_ARG_TYPE_SAMPLER)
+          if (meta->arg_info[i].type != POCL_ARG_TYPE_SAMPLER)
             POCL_MEM_FREE (*(void **)(arguments[i]));
           POCL_MEM_FREE(arguments[i]);
         }
-      else if (kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER && *(void**)arguments[i] == NULL)
+      else if (meta->arg_info[i].type == POCL_ARG_TYPE_POINTER)
         {
           POCL_MEM_FREE(arguments[i]);
         }
     }
-  for (i = kernel->num_args;
-       i < kernel->num_args + kernel->num_locals;
-       ++i)
-    {
-      POCL_MEM_FREE(*(void **)(arguments[i]));
-      POCL_MEM_FREE(arguments[i]);
-    }
+
+  if (!cmd->device->device_alloca_locals)
+    for (i = 0; i < meta->num_locals; ++i)
+      {
+        POCL_MEM_FREE (*(void **)(arguments[meta->num_args + i]));
+        POCL_MEM_FREE (arguments[meta->num_args + i]);
+      }
   free(arguments);
 
   pocl_release_dlhandle_cache (cmd);
 }
 
 void
-pocl_basic_run_native 
-(void *data, 
- _cl_command_node* cmd)
+pocl_basic_run_native (void *data, _cl_command_node *cmd)
 {
   cmd->command.native.user_func(cmd->command.native.args);
 }
@@ -709,12 +734,12 @@ pocl_basic_copy (void *data,
                  size_t src_offset,
                  size_t size)
 {
-  void *__restrict__ src_ptr = src_mem_id->mem_ptr;
-  void *__restrict__ dst_ptr = dst_mem_id->mem_ptr;
-  if (src_ptr == dst_ptr)
+  char *__restrict__ src_ptr = src_mem_id->mem_ptr;
+  char *__restrict__ dst_ptr = dst_mem_id->mem_ptr;
+  if ((src_ptr + src_offset) == (dst_ptr + dst_offset))
     return;
 
-  memcpy ((char *)dst_ptr + dst_offset, (char *)src_ptr + src_offset, size);
+  memcpy (dst_ptr + dst_offset, src_ptr + src_offset, size);
 }
 
 void
@@ -755,7 +780,7 @@ pocl_basic_copy_rect (void *data,
       (unsigned)dst_origin[1], (unsigned)dst_origin[2],
       (unsigned long)src_row_pitch, (unsigned long)src_slice_pitch,
       (unsigned long)dst_row_pitch, (unsigned long)dst_slice_pitch,
-      region[0], region[1], region[2]);
+      (unsigned long)region[0], (unsigned long)region[1], (unsigned long)region[2]);
 
   size_t j, k;
 
@@ -813,7 +838,7 @@ pocl_basic_write_rect (void *data,
       (unsigned)host_origin[1], (unsigned)host_origin[2],
       (unsigned long)buffer_row_pitch, (unsigned long)buffer_slice_pitch,
       (unsigned long)host_row_pitch, (unsigned long)host_slice_pitch,
-      region[0], region[1], region[2]);
+      (unsigned long)region[0], (unsigned long)region[1], (unsigned long)region[2]);
 
   size_t j, k;
 
@@ -876,7 +901,7 @@ pocl_basic_read_rect (void *data,
       (unsigned)host_origin[1], (unsigned)host_origin[2],
       (unsigned long)buffer_row_pitch, (unsigned long)buffer_slice_pitch,
       (unsigned long)host_row_pitch, (unsigned long)host_slice_pitch,
-      region[0], region[1], region[2]);
+      (unsigned long)region[0], (unsigned long)region[1], (unsigned long)region[2]);
 
   size_t j, k;
 
@@ -1052,7 +1077,7 @@ pocl_basic_unmap_mem(void *data,
 }
 
 cl_int
-pocl_basic_uninit (cl_device_id device)
+pocl_basic_uninit (unsigned j, cl_device_id device)
 {
   struct data *d = (struct data*)device->data;
   POCL_DESTROY_LOCK (d->cq_lock);
@@ -1063,14 +1088,13 @@ pocl_basic_uninit (cl_device_id device)
 }
 
 cl_int
-pocl_basic_reinit (cl_device_id device)
+pocl_basic_reinit (unsigned j, cl_device_id device)
 {
   struct data *d = (struct data *)calloc (1, sizeof (struct data));
   if (d == NULL)
     return CL_OUT_OF_HOST_MEMORY;
 
   d->current_kernel = NULL;
-  d->current_dlhandle = 0;
 
   assert (device->printf_buffer_size > 0);
   d->printf_buffer = pocl_aligned_malloc (MAX_EXTENDED_ALIGNMENT,
@@ -1082,25 +1106,6 @@ pocl_basic_reinit (cl_device_id device)
   return CL_SUCCESS;
 }
 
-cl_ulong
-pocl_basic_get_timer_value (void *data) 
-{
-  return pocl_gettimemono_ns();
-}
-
-cl_int 
-pocl_basic_get_supported_image_formats (cl_mem_flags flags,
-                                        const cl_image_format **image_formats,
-                                        cl_uint *num_img_formats)
-{
-    if (num_img_formats == NULL || image_formats == NULL)
-      return CL_INVALID_VALUE;
-
-    *num_img_formats = sizeof(supported_image_formats)/sizeof(cl_image_format);
-    *image_formats = supported_image_formats;
-    
-    return CL_SUCCESS; 
-}
 
 static void basic_command_scheduler (struct data *d) 
 {
@@ -1126,7 +1131,7 @@ pocl_basic_submit (_cl_command_node *node, cl_command_queue cq)
   struct data *d = node->device->data;
 
   if (node != NULL && node->type == CL_COMMAND_NDRANGE_KERNEL)
-    pocl_check_dlhandle_cache (node, 1);
+    pocl_check_kernel_dlhandle_cache (node, 1, 1);
 
   node->ready = 1;
   POCL_LOCK (d->cq_lock);
@@ -1149,7 +1154,7 @@ void pocl_basic_flush (cl_device_id device, cl_command_queue cq)
 }
 
 void
-pocl_basic_join(cl_device_id device, cl_command_queue cq)
+pocl_basic_join (cl_device_id device, cl_command_queue cq)
 {
   struct data *d = (struct data*)device->data;
 
@@ -1168,7 +1173,7 @@ pocl_basic_notify (cl_device_id device, cl_event event, cl_event finished)
 
   if (finished->status < CL_COMPLETE)
     {
-      POCL_UPDATE_EVENT_FAILED (event);
+      pocl_update_event_failed (event);
       return;
     }
 
@@ -1179,7 +1184,7 @@ pocl_basic_notify (cl_device_id device, cl_event event, cl_event finished)
     {
       if (event->status == CL_QUEUED)
         {
-          POCL_UPDATE_EVENT_SUBMITTED (event);
+          pocl_update_event_submitted (event);
           POCL_LOCK (d->cq_lock);
           CDL_DELETE (d->command_list, node);
           CDL_PREPEND (d->ready_list, node);
@@ -1191,16 +1196,11 @@ pocl_basic_notify (cl_device_id device, cl_event event, cl_event finished)
 }
 
 void
-pocl_basic_broadcast (cl_event event)
-{
-  pocl_broadcast (event);
-}
-
-void
-pocl_basic_compile_kernel (_cl_command_node *cmd, cl_kernel kernel, cl_device_id device)
+pocl_basic_compile_kernel (_cl_command_node *cmd, cl_kernel kernel,
+                           cl_device_id device, int specialize)
 {
   if (cmd != NULL && cmd->type == CL_COMMAND_NDRANGE_KERNEL)
-    pocl_check_dlhandle_cache (cmd, 0);
+    pocl_check_kernel_dlhandle_cache (cmd, 0, specialize);
 }
 
 /*********************** IMAGES ********************************/
