@@ -1,18 +1,18 @@
 /* basic.h - a minimalistic pocl device driver layer implementation
 
    Copyright (c) 2011 Universidad Rey Juan Carlos and
-                 2012 Pekka Jääskeläinen / Tampere University of Technology
-   
+                 2012-2016 Pekka Jääskeläinen / Tampere University of Technology
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -28,7 +28,9 @@
  * a minimalistic (but still working) device driver for pocl.
  *
  * It is a "native device" without multithreading and uses the malloc
- * directly for buffer allocation etc.
+ * directly for buffer allocation etc. It also executes work groups in
+ * their sequential (increasing) order, thus makes it useful as a test
+ * device.
  */
 
 #ifndef POCL_BASIC_H
