@@ -35,6 +35,7 @@
 #include "pocl_llvm.h"
 #include "pocl_mem_management.h"
 #include "pocl_runtime_config.h"
+#include "pocl_timing.h"
 #include "pocl_util.h"
 
 #include <string.h>
@@ -128,8 +129,6 @@ pocl_cuda_error (CUresult result, unsigned line, const char *func,
 void
 pocl_cuda_init_device_ops (struct pocl_device_ops *ops)
 {
-  pocl_basic_init_device_ops (ops);
-
   ops->device_name = "CUDA";
   ops->build_hash = pocl_cuda_build_hash;
   ops->probe = pocl_cuda_probe;
@@ -143,6 +142,7 @@ pocl_cuda_init_device_ops (struct pocl_device_ops *ops)
   ops->compile_kernel = pocl_cuda_compile_kernel;
   ops->submit = pocl_cuda_submit;
   ops->notify = pocl_cuda_notify;
+  ops->broadcast = pocl_broadcast;
   ops->wait_event = pocl_cuda_wait_event;
   ops->update_event = pocl_cuda_update_event;
   ops->free_event_data = pocl_cuda_free_event_data;
@@ -159,10 +159,6 @@ pocl_cuda_init_device_ops (struct pocl_device_ops *ops)
   ops->copy_rect = NULL;
   ops->unmap_mem = NULL;
   ops->run = NULL;
-
-  /* TODO: implement remaining ops functions if needed: */
-  /* broadcast */
-  /* get_timer_value */
 }
 
 cl_int
@@ -241,7 +237,6 @@ pocl_cuda_init (unsigned j, cl_device_id dev, const char *parameters)
       GET_CU_PROP (CLOCK_RATE, dev->max_clock_frequency);
       dev->max_clock_frequency /= 1000;
       GET_CU_PROP (TEXTURE_ALIGNMENT, dev->mem_base_addr_align);
-      dev->mem_base_addr_align *= 8;
       GET_CU_PROP (INTEGRATED, dev->host_unified_memory);
     }
   if (CUDA_CHECK_ERROR (result, "cuDeviceGetAttribute"))
@@ -312,7 +307,7 @@ pocl_cuda_init (unsigned j, cl_device_id dev, const char *parameters)
       result = cuEventCreate (&data->epoch_event, CU_EVENT_DEFAULT);
       CUDA_CHECK_ERROR (result, "cuEventCreate");
 
-      data->epoch = dev->ops->get_timer_value (dev->data);
+      data->epoch = pocl_gettimemono_ns ();
 
       result = cuEventRecord (data->epoch_event, 0);
       result = cuEventSynchronize (data->epoch_event);
@@ -438,7 +433,7 @@ pocl_cuda_probe (struct pocl_device_ops *ops)
 }
 
 cl_int
-pocl_cuda_uninit (cl_device_id device)
+pocl_cuda_uninit (unsigned j, cl_device_id device)
 {
   pocl_cuda_device_data_t *data = device->data;
 
@@ -762,15 +757,16 @@ pocl_cuda_submit_unmap_mem (CUstream stream, pocl_mem_identifier *dst_mem_id,
 
 static pocl_cuda_kernel_data_t *
 load_or_generate_kernel (cl_kernel kernel, cl_device_id device,
-                         int has_offsets)
+                         int has_offsets, unsigned device_i,
+                         _cl_command_node *command, int specialized)
 {
   CUresult result;
-
+  pocl_kernel_metadata_t *meta = kernel->meta;
   /* Check if we already have a compiled kernel function */
-  pocl_cuda_kernel_data_t *kdata = (pocl_cuda_kernel_data_t *)kernel->data;
+  pocl_cuda_kernel_data_t *kdata
+      = (pocl_cuda_kernel_data_t *)meta->data[device_i];
   if (kdata)
     {
-      kdata += device->dev_id;
       if ((has_offsets && kdata->kernel_offsets)
           || (!has_offsets && kdata->kernel))
         return kdata;
@@ -778,9 +774,8 @@ load_or_generate_kernel (cl_kernel kernel, cl_device_id device,
   else
     {
       /* TODO: when can we release this? */
-      kernel->data
-          = calloc (pocl_num_devices, sizeof (pocl_cuda_kernel_data_t));
-      kdata = kernel->data + device->dev_id;
+      kdata = meta->data[device_i]
+          = (void *)calloc (1, sizeof (pocl_cuda_kernel_data_t));
     }
 
   pocl_cuda_device_data_t *ddata = (pocl_cuda_device_data_t *)device->data;
@@ -789,7 +784,8 @@ load_or_generate_kernel (cl_kernel kernel, cl_device_id device,
   POCL_LOCK(ddata->compile_lock);
 
   /* Generate the parallel bitcode file linked with the kernel library */
-  int error = pocl_llvm_generate_workgroup_function (device, kernel, 0, 0, 0);
+  int error = pocl_llvm_generate_workgroup_function (device_i, device, kernel,
+                                                     command, specialized);
   if (error)
     {
       POCL_MSG_PRINT_GENERAL ("pocl_llvm_generate_workgroup_function() failed"
@@ -798,14 +794,11 @@ load_or_generate_kernel (cl_kernel kernel, cl_device_id device,
     }
 
   char bc_filename[POCL_FILENAME_LENGTH];
-  unsigned device_i = pocl_cl_device_to_index (kernel->program, device);
   pocl_cache_work_group_function_path (bc_filename, kernel->program, device_i,
-                                       kernel, 0, 0, 0);
+                                       kernel, command, specialized);
 
   char ptx_filename[POCL_FILENAME_LENGTH];
   strcpy (ptx_filename, bc_filename);
-  if (has_offsets)
-    strncat (ptx_filename, ".offsets", POCL_FILENAME_LENGTH - 1);
   strncat (ptx_filename, ".ptx", POCL_FILENAME_LENGTH - 1);
 
   if (!pocl_exists (ptx_filename))
@@ -832,8 +825,8 @@ load_or_generate_kernel (cl_kernel kernel, cl_device_id device,
   /* Get pointer aligment */
   if (!kdata->alignments)
     {
-      kdata->alignments = calloc (kernel->num_args + kernel->num_locals + 4,
-                                  sizeof (size_t));
+      kdata->alignments
+          = calloc (meta->num_args + meta->num_locals + 4, sizeof (size_t));
       pocl_cuda_get_ptr_arg_alignment (bc_filename, kernel->name,
                                        kdata->alignments);
     }
@@ -856,37 +849,39 @@ load_or_generate_kernel (cl_kernel kernel, cl_device_id device,
 
 void
 pocl_cuda_compile_kernel (_cl_command_node *cmd, cl_kernel kernel,
-                          cl_device_id device)
+                          cl_device_id device, int specialize)
 {
-  load_or_generate_kernel (kernel, device, 0);
+  load_or_generate_kernel (kernel, device, 0, cmd->command.run.device_i, cmd,
+                           specialize);
 }
 
 void
-pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
+pocl_cuda_submit_kernel (CUstream stream, _cl_command_node *cmd,
                          cl_device_id device, cl_event event)
 {
+  _cl_command_run run = cmd->command.run;
   cl_kernel kernel = run.kernel;
   pocl_argument *arguments = run.arguments;
   struct pocl_context pc = run.pc;
+  pocl_kernel_metadata_t *meta = kernel->meta;
 
   /* Check if we need to handle global work offsets */
-  int has_offsets = 0;
-  if (pc.global_offset[0] || pc.global_offset[1] || pc.global_offset[2])
-    has_offsets = 1;
+  int has_offsets =
+    (pc.global_offset[0] || pc.global_offset[1] || pc.global_offset[2]);
 
   /* Get kernel function */
-  pocl_cuda_kernel_data_t *kdata
-      = load_or_generate_kernel (kernel, device, has_offsets);
+  pocl_cuda_kernel_data_t *kdata = load_or_generate_kernel (
+      kernel, device, has_offsets, run.device_i, cmd, 1);
   CUmodule module = has_offsets ? kdata->module_offsets : kdata->module;
   CUfunction function = has_offsets ? kdata->kernel_offsets : kdata->kernel;
 
   /* Prepare kernel arguments */
   void *null = NULL;
   unsigned sharedMemBytes = 0;
-  void *params[kernel->num_args + kernel->num_locals + 4];
-  unsigned sharedMemOffsets[kernel->num_args + kernel->num_locals];
+  void *params[meta->num_args + meta->num_locals + 4];
+  unsigned sharedMemOffsets[meta->num_args + meta->num_locals];
   unsigned constantMemBytes = 0;
-  unsigned constantMemOffsets[kernel->num_args];
+  unsigned constantMemOffsets[meta->num_args];
   unsigned globalOffsets[3];
 
   /* Get handle to constant memory buffer */
@@ -897,9 +892,9 @@ pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
 
   CUresult result;
   unsigned i;
-  for (i = 0; i < kernel->num_args; i++)
+  for (i = 0; i < meta->num_args; i++)
     {
-      pocl_argument_type type = kernel->arg_info[i].type;
+      pocl_argument_type type = meta->arg_info[i].type;
       switch (type)
         {
         case POCL_ARG_TYPE_NONE:
@@ -907,7 +902,7 @@ pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
           break;
         case POCL_ARG_TYPE_POINTER:
           {
-            if (kernel->arg_info[i].is_local)
+            if (ARG_IS_LOCAL (meta->arg_info[i]))
               {
                 size_t size = arguments[i].size;
                 size_t align = kdata->alignments[i];
@@ -921,7 +916,7 @@ pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
 
                 sharedMemBytes += size;
               }
-            else if (kernel->arg_info[i].address_qualifier
+            else if (meta->arg_info[i].address_qualifier
                      == CL_KERNEL_ARG_ADDRESS_CONSTANT)
               {
                 assert (constant_mem_base);
@@ -929,7 +924,8 @@ pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
                 /* Get device pointer */
                 cl_mem mem = *(void **)arguments[i].value;
                 CUdeviceptr src
-                    = (CUdeviceptr)mem->device_ptrs[device->dev_id].mem_ptr;
+                    = (CUdeviceptr)mem->device_ptrs[device->dev_id].mem_ptr
+                      + arguments[i].offset;
 
                 size_t align = kdata->alignments[i];
                 if (constantMemBytes % align)
@@ -953,7 +949,8 @@ pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
                 if (arguments[i].value)
                   {
                     cl_mem mem = *(void **)arguments[i].value;
-                    params[i] = &mem->device_ptrs[device->dev_id].mem_ptr;
+                    params[i] = &mem->device_ptrs[device->dev_id].mem_ptr
+                                + arguments[i].offset;
 
 #if defined __arm__
                     /* On ARM with USE_HOST_PTR, perform explicit copy to
@@ -984,13 +981,13 @@ pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
     POCL_ABORT ("[CUDA] Total constant buffer size %u exceeds %lu allocated\n",
                 constantMemBytes, constant_mem_size);
 
-  unsigned arg_index = kernel->num_args;
+  unsigned arg_index = meta->num_args;
 
   /* Deal with automatic local allocations */
   /* TODO: Would be better to remove arguments and make these static GEPs */
-  for (i = 0; i < kernel->num_locals; ++i, ++arg_index)
+  for (i = 0; i < meta->num_locals; ++i, ++arg_index)
     {
-      size_t size = arguments[arg_index].size;
+      size_t size = meta->local_sizes[i];
       size_t align = kdata->alignments[arg_index];
 
       /* Pad offset to align memory */
@@ -1018,8 +1015,9 @@ pocl_cuda_submit_kernel (CUstream stream, _cl_command_run run,
 
   /* Launch kernel */
   result = cuLaunchKernel (function, pc.num_groups[0], pc.num_groups[1],
-                           pc.num_groups[2], run.local_x, run.local_y,
-                           run.local_z, sharedMemBytes, stream, params, NULL);
+                           pc.num_groups[2], pc.local_size[0],
+                           pc.local_size[1], pc.local_size[2], sharedMemBytes,
+                           stream, params, NULL);
   CUDA_CHECK (result, "cuLaunchKernel");
 }
 
@@ -1098,7 +1096,7 @@ pocl_cuda_submit_node (_cl_command_node *node, cl_command_queue cq, int locked)
       CUDA_CHECK (result, "cuEventRecord");
     }
 
-  POCL_UPDATE_EVENT_SUBMITTED (node->event);
+  pocl_update_event_submitted (node->event);
 
   POCL_UNLOCK_OBJ (node->event);
 
@@ -1210,7 +1208,7 @@ pocl_cuda_submit_node (_cl_command_node *node, cl_command_queue cq, int locked)
         break;
       }
     case CL_COMMAND_NDRANGE_KERNEL:
-      pocl_cuda_submit_kernel (stream, cmd->run, node->device, node->event);
+      pocl_cuda_submit_kernel (stream, node, node->device, node->event);
       break;
 
     case CL_COMMAND_MARKER:
@@ -1341,11 +1339,11 @@ pocl_cuda_finalize_command (cl_device_id device, cl_event event)
       cl_kernel kernel = event->command.run.kernel;
       pocl_argument *arguments = event->command.run.arguments;
       unsigned i;
-      for (i = 0; i < kernel->num_args; i++)
+      for (i = 0; i < meta->num_args; i++)
         {
-          if (kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER)
+          if (meta->arg_info[i].type == POCL_ARG_TYPE_POINTER)
             {
-              if (!kernel->arg_info[i].is_local && arguments[i].value)
+              if (!ARG_IS_LOCAL (meta->arg_info[i]) && arguments[i].value)
                 {
                   cl_mem mem = *(void **)arguments[i].value;
                   if (mem->flags & CL_MEM_USE_HOST_PTR)
@@ -1369,85 +1367,51 @@ pocl_cuda_finalize_command (cl_device_id device, cl_event event)
     }
 
   /* Handle failed events */
-  if (event->status < 0)
-    {
-      pocl_broadcast (event);
-      pocl_update_command_queue (event);
-      POname (clReleaseEvent) (event);
-      return;
-    }
 
-  POCL_UPDATE_EVENT_RUNNING (event);
-  POCL_UPDATE_EVENT_COMPLETE (event);
+
+  pocl_update_event_running (event);
+  if (event->status < 0)
+    pocl_update_event_failed (event);
+  else
+    POCL_UPDATE_EVENT_COMPLETE_MSG (event, "CUDA event");
 }
 
 void
-pocl_cuda_update_event (cl_device_id device, cl_event event, cl_int status)
+pocl_cuda_update_event (cl_device_id device, cl_event event)
 {
-  switch (status)
+  if ((event->status == CL_COMPLETE)
+      && (event->queue->properties & CL_QUEUE_PROFILING_ENABLE))
     {
-    case CL_QUEUED:
-      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
-        event->time_queue = device->ops->get_timer_value (device->data);
-      event->status = status;
-      break;
-    case CL_SUBMITTED:
-      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
-        event->time_submit = device->ops->get_timer_value (device->data);
-      event->status = status;
-      break;
-    case CL_RUNNING:
-      /* Wait until complete to get the timing from the CUDA events */
-      event->status = status;
-      break;
-    case CL_COMPLETE:
-      pocl_mem_objs_cleanup (event);
-
       /* Update timing info with CUDA event timers if profiling enabled */
-      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
-        {
-          /* CUDA doesn't provide a way to get event timestamps directly,
-           * only the elapsed time between two events. We use the elapsed
-           * time from the epoch event enqueued on device creation to get
-           * the actual timestamps.
-           *
-           * Since the CUDA timer resolution is lower than the host timer,
-           * this can sometimes result in the start time being before the
-           * submit time, so we use max() to ensure the timestamps are
-           * sane. */
+      /* CUDA doesn't provide a way to get event timestamps directly,
+       * only the elapsed time between two events. We use the elapsed
+       * time from the epoch event enqueued on device creation to get
+       * the actual timestamps.
+       *
+       * Since the CUDA timer resolution is lower than the host timer,
+       * this can sometimes result in the start time being before the
+       * submit time, so we use max() to ensure the timestamps are
+       * sane. */
 
-          float diff;
-          CUresult result;
-          pocl_cuda_event_data_t *event_data
-              = (pocl_cuda_event_data_t *)event->data;
-          cl_ulong epoch = ((pocl_cuda_device_data_t *)device->data)->epoch;
+      float diff;
+      CUresult result;
+      pocl_cuda_event_data_t *event_data
+          = (pocl_cuda_event_data_t *)event->data;
+      cl_ulong epoch = ((pocl_cuda_device_data_t *)device->data)->epoch;
 
-          result = cuEventElapsedTime (
-              &diff, ((pocl_cuda_device_data_t *)device->data)->epoch_event,
-              event_data->start);
-          CUDA_CHECK (result, "cuEventElapsedTime");
-          event->time_start = (cl_ulong) (epoch + diff * 1e6);
-          event->time_start = max (event->time_start, event->time_submit + 1);
+      result = cuEventElapsedTime (
+          &diff, ((pocl_cuda_device_data_t *)device->data)->epoch_event,
+          event_data->start);
+      CUDA_CHECK (result, "cuEventElapsedTime");
+      event->time_start = (cl_ulong) (epoch + diff * 1e6);
+      event->time_start = max (event->time_start, event->time_submit + 1);
 
-          result = cuEventElapsedTime (
-              &diff, ((pocl_cuda_device_data_t *)device->data)->epoch_event,
-              event_data->end);
-          CUDA_CHECK (result, "cuEventElapsedTime");
-          event->time_end = (cl_ulong) (epoch + diff * 1e6);
-          event->time_end = max (event->time_end, event->time_start + 1);
-        }
-
-      event->status = CL_COMPLETE;
-
-      POCL_UNLOCK_OBJ (event);
-      device->ops->broadcast (event);
-      pocl_update_command_queue (event);
-      POCL_LOCK_OBJ (event);
-
-      break;
-    default:
-      assert (0 && "Invalid event status\n");
-      break;
+      result = cuEventElapsedTime (
+          &diff, ((pocl_cuda_device_data_t *)device->data)->epoch_event,
+          event_data->end);
+      CUDA_CHECK (result, "cuEventElapsedTime");
+      event->time_end = (cl_ulong) (epoch + diff * 1e6);
+      event->time_end = max (event->time_end, event->time_start + 1);
     }
 }
 
