@@ -58,8 +58,6 @@ static void fixConstantMemArgs(llvm::Module *Module, const char *KernelName);
 static void fixLocalMemArgs(llvm::Module *Module, const char *KernelName);
 static void fixPrintF(llvm::Module *Module);
 static void handleGetWorkDim(llvm::Module *Module, const char *KernelName);
-static void handleGlobalOffsets(llvm::Module *Module, const char *KernelName,
-                                bool HasOffsets);
 static void linkLibDevice(llvm::Module *Module, const char *KernelName,
                           const char *LibDevicePath);
 static void mapLibDeviceCalls(llvm::Module *Module);
@@ -88,7 +86,6 @@ int pocl_ptx_gen(const char *BitcodeFilename, const char *PTXFilename,
   fixConstantMemArgs(Module->get(), KernelName);
   fixLocalMemArgs(Module->get(), KernelName);
   handleGetWorkDim(Module->get(), KernelName);
-  handleGlobalOffsets(Module->get(), KernelName, HasOffsets);
   addKernelAnnotations(Module->get(), KernelName);
   mapLibDeviceCalls(Module->get());
   linkLibDevice(Module->get(), KernelName, LibDevicePath);
@@ -135,7 +132,7 @@ int pocl_ptx_gen(const char *BitcodeFilename, const char *PTXFilename,
   llvm::SmallVector<char, 4096> Data;
   llvm::raw_svector_ostream PTXStream(Data);
   if (Machine->addPassesToEmitFile(Passes, PTXStream,
-#if ! LLVM_OLDER_THAN_7_0
+#ifndef LLVM_OLDER_THAN_7_0
                                    nullptr,
 #endif
                                    llvm::TargetMachine::CGFT_AssemblyFile)) {
@@ -184,13 +181,13 @@ void addKernelAnnotations(llvm::Module *Module, const char *KernelName) {
 // Essentially, the variadic list of arguments is replaced with a single array
 // instead.
 //
-// This function changes the prototype of __cl_printf to take an array instead
+// This function changes the prototype of printf to take an array instead
 // of a variadic argument list. It updates the function body to read from
 // this array to retrieve each argument instead of using the dummy __cl_va_arg
-// function. We then visit each __cl_printf callsite and generate the argument
+// function. We then visit each printf callsite and generate the argument
 // array to pass instead of the variadic list.
 void fixPrintF(llvm::Module *Module) {
-  llvm::Function *OldPrintF = Module->getFunction("__cl_printf");
+  llvm::Function *OldPrintF = Module->getFunction("printf");
   if (!OldPrintF)
     return;
 
@@ -204,7 +201,7 @@ void fixPrintF(llvm::Module *Module) {
   pocl::eraseFunctionAndCallers(Module->getFunction("llvm.va_start"));
   pocl::eraseFunctionAndCallers(Module->getFunction("llvm.va_end"));
 
-  // Create new non-variadic __cl_printf function.
+  // Create new non-variadic printf function.
   llvm::Type *ReturnType = OldPrintF->getReturnType();
   llvm::FunctionType *NewPrintfType =
       llvm::FunctionType::get(ReturnType, {FormatType, I64Ptr}, false);
@@ -218,11 +215,7 @@ void fixPrintF(llvm::Module *Module) {
 
   // Create i32 to hold current argument index.
   llvm::AllocaInst *ArgIndexPtr =
-#if LLVM_OLDER_THAN_5_0
-      new llvm::AllocaInst(I32, llvm::ConstantInt::get(I32, 1));
-#else
       new llvm::AllocaInst(I32, 0, llvm::ConstantInt::get(I32, 1));
-#endif
   ArgIndexPtr->insertBefore(&*NewPrintF->begin()->begin());
   llvm::StoreInst *ArgIndexInit =
       new llvm::StoreInst(llvm::ConstantInt::get(I32, 0), ArgIndexPtr);
@@ -231,13 +224,9 @@ void fixPrintF(llvm::Module *Module) {
   // Replace calls to _cl_va_arg with reads from new i64 array argument.
   llvm::Function *VaArgFunc = Module->getFunction("__cl_va_arg");
   if (VaArgFunc) {
-#if LLVM_OLDER_THAN_5_0
-    llvm::Argument *ArgsIn = &*++NewPrintF->arg_begin();
-#else
     auto args = NewPrintF->arg_begin();
     args++;
     llvm::Argument *ArgsIn = args;
-#endif
     std::vector<llvm::Value *> VaArgCalls(VaArgFunc->user_begin(),
                                           VaArgFunc->user_end());
     for (auto &U : VaArgCalls) {
@@ -296,11 +285,7 @@ void fixPrintF(llvm::Module *Module) {
     // Allocate array for arguments.
     // TODO: Deal with vector arguments.
     llvm::AllocaInst *Args =
-#if LLVM_OLDER_THAN_5_0
-        new llvm::AllocaInst(I64, llvm::ConstantInt::get(I32, NumArgs));
-#else
         new llvm::AllocaInst(I64, 0, llvm::ConstantInt::get(I32, NumArgs));
-#endif
     Args->insertBefore(Call);
 
     // Loop over arguments (skipping format).
@@ -464,67 +449,6 @@ void handleGetWorkDim(llvm::Module *Module, const char *KernelName) {
   // TODO: What if get_work_dim() is called from a non-kernel function?
 }
 
-// If we don't need to handle offsets, just replaces uses of the offset
-// variables with constant zero. Otherwise, add additional kernel arguments for
-// the offsets and use those instead.
-void handleGlobalOffsets(llvm::Module *Module, const char *KernelName,
-                         bool HasOffsets) {
-  if (!HasOffsets) {
-    llvm::Type *I32 = llvm::Type::getInt32Ty(Module->getContext());
-    llvm::Value *Zero = llvm::ConstantInt::getSigned(I32, 0);
-    replaceScalarGlobalVar(Module, "_global_offset_x", Zero);
-    replaceScalarGlobalVar(Module, "_global_offset_y", Zero);
-    replaceScalarGlobalVar(Module, "_global_offset_z", Zero);
-    return;
-  }
-
-  llvm::Function *Function = Module->getFunction(KernelName);
-  if (!Function)
-    POCL_ABORT("[CUDA] ptx-gen: kernel function not found in module\n");
-
-  // Add additional arguments for the global offsets.
-  llvm::FunctionType *FunctionType = Function->getFunctionType();
-  std::vector<llvm::Type *> ArgumentTypes(FunctionType->param_begin(),
-                                          FunctionType->param_end());
-  llvm::Type *I32 = llvm::Type::getInt32Ty(Module->getContext());
-  ArgumentTypes.push_back(I32);
-  ArgumentTypes.push_back(I32);
-  ArgumentTypes.push_back(I32);
-
-  // Create new function.
-  llvm::FunctionType *NewFunctionType =
-      llvm::FunctionType::get(Function->getReturnType(), ArgumentTypes, false);
-  llvm::Function *NewFunction = llvm::Function::Create(
-      NewFunctionType, Function->getLinkage(), Function->getName(), Module);
-  NewFunction->takeName(Function);
-
-  // Map function arguments.
-  llvm::ValueToValueMapTy VV;
-  llvm::Function::arg_iterator OldArg;
-  llvm::Function::arg_iterator NewArg;
-  for (OldArg = Function->arg_begin(), NewArg = NewFunction->arg_begin();
-       OldArg != Function->arg_end(); NewArg++, OldArg++) {
-    NewArg->takeName(&*OldArg);
-    VV[&*OldArg] = &*NewArg;
-  }
-
-  // Clone function.
-  llvm::SmallVector<llvm::ReturnInst *, 1> RI;
-  llvm::CloneFunctionInto(NewFunction, Function, VV, true, RI);
-
-  // Replace uses of the global offset variables with the new arguments.
-  NewArg->setName("global_offset_x");
-  replaceScalarGlobalVar(Module, "_global_offset_x", (&*NewArg++));
-  NewArg->setName("global_offset_y");
-  replaceScalarGlobalVar(Module, "_global_offset_y", (&*NewArg++));
-  NewArg->setName("global_offset_z");
-  replaceScalarGlobalVar(Module, "_global_offset_z", (&*NewArg++));
-
-  // TODO: What if the offsets are in a function that isn't the kernel?
-
-  Function->eraseFromParent();
-}
-
 int findLibDevice(char LibDevicePath[PATH_MAX], const char *Arch) {
   // Extract numeric portion of SM version.
   char *End;
@@ -857,11 +781,13 @@ void mapLibDeviceCalls(llvm::Module *Module) {
     {"llvm.pow.f32", "__nv_powf"},
     {"llvm.pow.f64", "__nv_pow"},
 
+    {"llvm.powi.f32", "__nv_powif"},
+    {"llvm.powi.f64", "__nv_powi"},
+
     // TODO: frexp
     // TODO: ldexp
     // TODO: lgamma_r
     // TODO: modf
-    // TODO: pown
     // TODO: remquo
     // TODO: rootn
   };
@@ -880,9 +806,14 @@ void mapLibDeviceCalls(llvm::Module *Module) {
       if (Call) {
         // Create function declaration for libdevice version.
         llvm::FunctionType *FunctionType = Function->getFunctionType();
+#ifdef LLVM_OLDER_THAN_9_0
         llvm::Constant *LibDeviceFunction = Module->getOrInsertFunction(
             Entry.LibDeviceFunctionName, FunctionType);
-
+#else
+        llvm::FunctionCallee FC = Module->getOrInsertFunction(
+            Entry.LibDeviceFunctionName, FunctionType);
+        llvm::Function *LibDeviceFunction = llvm::cast<llvm::Function>(FC.getCallee());
+#endif
         // Replace function with libdevice version.
         std::vector<llvm::Value *> Args(Call->arg_begin(), Call->arg_end());
         llvm::CallInst *NewCall =
