@@ -1,18 +1,18 @@
 /* Definition of available OpenCL devices.
 
    Copyright (c) 2011 Universidad Rey Juan Carlos and
-                 2012-2015 Pekka Jääskeläinen / Tampere University of Technology
-   
+                 2012-2018 Pekka Jääskeläinen
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -65,6 +65,10 @@
 #include "cuda/pocl-cuda.h"
 #endif
 
+#if defined(BUILD_ACCEL)
+#include "accel/accel.h"
+#endif
+
 #define MAX_DEV_NAME_LEN 64
 
 /* the enabled devices */
@@ -76,8 +80,12 @@ typedef void (*init_device_ops)(struct pocl_device_ops*);
 
 /* All init function for device operations available to pocl */
 static init_device_ops pocl_devices_init_ops[] = {
-  pocl_pthread_init_device_ops,
+#ifdef BUILD_BASIC
   pocl_basic_init_device_ops,
+#endif
+#ifdef BUILD_PTHREAD
+  pocl_pthread_init_device_ops,
+#endif
 #if defined(TCE_AVAILABLE)
   pocl_ttasim_init_device_ops,
 #endif
@@ -87,6 +95,9 @@ static init_device_ops pocl_devices_init_ops[] = {
 #if defined(BUILD_CUDA)
   pocl_cuda_init_device_ops,
 #endif
+#if defined(BUILD_ACCEL)
+  pocl_accel_init_device_ops,
+#endif
 };
 
 #define POCL_NUM_DEVICE_TYPES (sizeof(pocl_devices_init_ops) / sizeof((pocl_devices_init_ops)[0]))
@@ -111,7 +122,7 @@ int pocl_device_get_env_count(const char *dev_type)
   const char *dev_env = getenv(POCL_DEVICES_ENV);
   char *ptr, *saveptr = NULL, *tofree, *token;
   unsigned int dev_count = 0;
-  if (dev_env == NULL) 
+  if (dev_env == NULL)
     {
       return -1;
     }
@@ -198,20 +209,6 @@ str_toupper(char *out, const char *in)
   out[i] = '\0';
 }
 
-static inline void
-pocl_string_to_dirname(char *str)
-{
-  char *s_ptr;
-  if (!str) return;
-
-  // Replace special characters with '_'
-  for (s_ptr = str; (*s_ptr); s_ptr++)
-    {
-      if (!isalnum(*s_ptr))
-        *s_ptr = '_';
-    }
-}
-
 /* This ugly hack is required because:
  *
  * OpenCL 1.2 specification, 6.3 Operators :
@@ -229,6 +226,7 @@ pocl_string_to_dirname(char *str)
  * it's likely to ruin the performance.
  */
 
+#ifdef ENABLE_HOST_CPU_DEVICES
 #ifdef __linux__
 #ifdef __x86_64__
 
@@ -320,22 +318,19 @@ sigfpe_signal_handler (int signo, siginfo_t *si, void *data)
     }
 }
 
+#endif
 #endif
 #endif
 
 cl_int
 pocl_uninit_devices ()
 {
-  assert (first_init_done);
   cl_int retval = CL_SUCCESS;
 
-  if (!devices_active)
-    return retval;
-
-  if (pocl_num_devices == 0)
-    return CL_DEVICE_NOT_FOUND;
-
   POCL_LOCK (pocl_init_lock);
+  if ((!devices_active) || (pocl_num_devices == 0))
+    goto FINISH;
+
   POCL_MSG_PRINT_GENERAL ("UNINIT all devices\n");
 
   unsigned i, j, dev_index;
@@ -352,7 +347,7 @@ pocl_uninit_devices ()
             continue;
           if (d->ops->reinit == NULL || d->ops->uninit == NULL)
             continue;
-          cl_int ret = d->ops->uninit (d);
+          cl_int ret = d->ops->uninit (j, d);
           if (ret != CL_SUCCESS)
             {
               retval = ret;
@@ -382,7 +377,6 @@ pocl_reinit_devices ()
   if (pocl_num_devices == 0)
     return CL_DEVICE_NOT_FOUND;
 
-  POCL_LOCK (pocl_init_lock);
   POCL_MSG_WARN ("REINIT all devices\n");
 
   unsigned i, j, dev_index;
@@ -400,7 +394,7 @@ pocl_reinit_devices ()
             continue;
           if (d->ops->reinit == NULL || d->ops->uninit == NULL)
             continue;
-          cl_int ret = d->ops->reinit (d);
+          cl_int ret = d->ops->reinit (j, d);
           if (ret != CL_SUCCESS)
             {
               retval = ret;
@@ -414,46 +408,41 @@ pocl_reinit_devices ()
 FINISH:
 
   devices_active = 1;
-  POCL_UNLOCK (pocl_init_lock);
-
   return retval;
 }
 
 cl_int
 pocl_init_devices ()
 {
+  int errcode = CL_SUCCESS;
 
   /* This is a workaround to a nasty problem with libhwloc: When
      initializing basic, it calls libhwloc to query device info.
      In case libhwloc has the OpenCL plugin installed, it initializes
      it and it leads to initializing pocl again which leads to an
-     infinite loop. */
+     infinite loop. This only protects against recursive calls of
+     pocl_init_devices(), so must be done without pocl_init_lock held. */
+  if (init_in_progress)
+    return CL_SUCCESS; /* debatable, but what else can we do ? */
 
-  if (!first_init_done)
-    {
-      if (init_in_progress)
-        return CL_SUCCESS; /* debatable, but what else can we do ? */
-      init_in_progress = 1;
-      POCL_INIT_LOCK (pocl_init_lock);
-    }
-
-  POCL_LOCK(pocl_init_lock);
+  POCL_LOCK (pocl_init_lock);
+  init_in_progress = 1;
 
   if (first_init_done)
     {
-      POCL_UNLOCK(pocl_init_lock);
-      POCL_MSG_PRINT_GENERAL ("FIRST INIT done; REINIT all devices\n");
       if (!devices_active)
-        pocl_reinit_devices (); // TODO err check
-      return pocl_num_devices ? CL_SUCCESS : CL_DEVICE_NOT_FOUND;
+        {
+          POCL_MSG_PRINT_GENERAL ("FIRST INIT done; REINIT all devices\n");
+          pocl_reinit_devices (); // TODO err check
+        }
+      errcode = pocl_num_devices ? CL_SUCCESS : CL_DEVICE_NOT_FOUND;
+      goto ERROR;
     }
 
   /* first time initialization */
-
   unsigned i, j, dev_index;
   char env_name[1024];
   char dev_name[MAX_DEV_NAME_LEN] = { 0 };
-  static unsigned int device_count[POCL_NUM_DEVICE_TYPES];
 
   /* Set a global debug flag, so we don't have to call pocl_get_bool_option
    * everytime we use the debug macros */
@@ -463,15 +452,19 @@ pocl_init_devices ()
   stderr_is_a_tty = isatty(fileno(stderr));
 #endif
 
-  if (pocl_cache_init_topdir ())
-    {
-      first_init_done = 1;
-      pocl_num_devices = 0;
-      POCL_UNLOCK (pocl_init_lock);
-      return CL_DEVICE_NOT_FOUND;
-    }
+  POCL_GOTO_ERROR_ON ((pocl_cache_init_topdir ()), CL_DEVICE_NOT_FOUND,
+                      "Cache directory initialization failed");
+
   pocl_event_tracing_init ();
 
+#ifdef HAVE_SLEEP
+  int delay = pocl_get_int_option ("POCL_STARTUP_DELAY", 0);
+  if (delay > 0)
+    sleep (delay);
+#endif
+
+
+#ifdef ENABLE_HOST_CPU_DEVICES
 #ifdef __linux__
 #ifdef __x86_64__
 
@@ -500,6 +493,7 @@ pocl_init_devices ()
       assert (res == 0);
     }
 
+#endif
 #endif
 #endif
 
@@ -515,16 +509,13 @@ pocl_init_devices ()
       pocl_num_devices += device_count[i];
     }
 
-  if (pocl_num_devices == 0)
-    {
-      const char *dev_env = pocl_get_string_option ("POCL_DEVICES", NULL);
-      POCL_MSG_WARN ("no devices found. POCL_DEVICES=%s\n", dev_env);
-      return CL_DEVICE_NOT_FOUND;
-    }
+  const char *dev_env = pocl_get_string_option ("POCL_DEVICES", NULL);
+  POCL_GOTO_ERROR_ON ((pocl_num_devices == 0), CL_DEVICE_NOT_FOUND,
+                      "no devices found. POCL_DEVICES=%s\n", dev_env);
 
   pocl_devices = (struct _cl_device_id*) calloc(pocl_num_devices, sizeof(struct _cl_device_id));
-  POCL_RETURN_ERROR_ON ((pocl_devices == NULL), CL_OUT_OF_HOST_MEMORY,
-                        "Can not allocate memory for devices\n");
+  POCL_GOTO_ERROR_ON ((pocl_devices == NULL), CL_OUT_OF_HOST_MEMORY,
+                      "Can not allocate memory for devices\n");
 
   dev_index = 0;
   /* Init infos for each probed devices */
@@ -534,7 +525,6 @@ pocl_init_devices ()
       assert(pocl_device_ops[i].init);
       for (j = 0; j < device_count[i]; ++j)
         {
-          cl_int ret = CL_SUCCESS;
           cl_device_id dev = &pocl_devices[dev_index];
           dev->ops = &pocl_device_ops[i];
           dev->dev_id = dev_index;
@@ -551,20 +541,15 @@ pocl_init_devices ()
 
           /* Check if there are device-specific parameters set in the
              POCL_DEVICEn_PARAMETERS env. */
-          POCL_RETURN_ERROR_ON (
+          POCL_GOTO_ERROR_ON (
               (snprintf (env_name, 1024, "POCL_%s%d_PARAMETERS", dev_name, j)
                < 0),
               CL_OUT_OF_HOST_MEMORY, "Unable to generate the env string.");
-          ret = pocl_devices[dev_index].ops->init (j, &pocl_devices[dev_index], getenv(env_name));
-          switch (ret)
-          {
-          case CL_OUT_OF_HOST_MEMORY:
-            return ret;
-          case CL_SUCCESS:
-            break;
-          default:
-            pocl_devices[dev_index].available = 0;
-          }
+          errcode = pocl_devices[dev_index].ops->init (
+              j, &pocl_devices[dev_index], getenv (env_name));
+          POCL_GOTO_ERROR_ON ((errcode != CL_SUCCESS), CL_DEVICE_NOT_AVAILABLE,
+                              "Device %i / %s initialization failed!", j,
+                              dev_name);
 
           ++dev_index;
         }
@@ -572,9 +557,10 @@ pocl_init_devices ()
 
   first_init_done = 1;
   devices_active = 1;
+ERROR:
   init_in_progress = 0;
-  POCL_UNLOCK(pocl_init_lock);
-  return CL_SUCCESS;
+  POCL_UNLOCK (pocl_init_lock);
+  return errcode;
 }
 
 int pocl_get_unique_global_mem_id ()
