@@ -1,17 +1,18 @@
-/* tce_common.cc - common functionality over the different TCE/TTA device drivers.
+/* tce_common.cc - common functionality over the different TCE/TTA device
+   drivers.
+
+   Copyright (c) 2012-2019 Pekka Jääskeläinen / Tampere University of Technology
 
-   Copyright (c) 2012-2014 Pekka Jääskeläinen / Tampere University of Technology
-   
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -24,12 +25,14 @@
 #include "pocl_util.h"
 #include "pocl_cache.h"
 #include "pocl_llvm.h"
+#include "pocl_device.h"
 #include "utlist.h"
 #include "common.h"
 
 #include "config.h"
 #include "pocl_runtime_config.h"
 #include "pocl_hash.h"
+#include "pocl_cache.h"
 
 #ifndef _MSC_VER
 #  include <unistd.h>
@@ -57,6 +60,7 @@
 using namespace TTAMachine;
 
 #include <algorithm>
+#include <sstream>
 
 #define ALIGNMENT MAX_EXTENDED_ALIGNMENT
 
@@ -121,13 +125,9 @@ TCEDevice::readWordFromDevice(uint32_t addr) {
 void
 TCEDevice::findDataMemoryAddresses() {
   /* Figure out the locations of the shared data structures in
-     the device memories from the fully-linked program. 
-  */
+     the device memories from the fully-linked program. */
   const TTAProgram::Program* prog = currentProgram;
   assert (prog != NULL);
-
-  const TTAProgram::GlobalScope& globalScope = prog->globalScopeConst();
-
   commandQueueAddr = global_as->start() + TTA_UNALLOCATED_GLOBAL_SPACE;
 }
 
@@ -203,11 +203,11 @@ TCEDevice::initMemoryManagement(const TTAMachine::Machine& mach) {
 #define SUBST(x) "  -DKERNEL_EXE_CMD_OFFSET=" # x
 #define OFFSET_ARG(c) SUBST(c)
 
-TCEString
-TCEDevice::tceccCommandLine
-(_cl_command_run *run_cmd, const TCEString& inputSrc, 
- const TCEString& outputTpef, const TCEString extraParams) 
-{
+TCEString TCEDevice::tceccCommandLine(_cl_command_run *run_cmd,
+                                      const TCEString &tempDir,
+                                      const TCEString &inputSrc,
+                                      const TCEString &outputTpef,
+                                      const TCEString extraParams) {
 
   TCEString mainC;
   if (isMultiCoreMachine()) 
@@ -235,8 +235,6 @@ TCEDevice::tceccCommandLine
 
   extraFlags += OFFSET_ARG(TTA_UNALLOCATED_GLOBAL_SPACE);
 
-  TCEString tempDir = run_cmd->tmp_dir;
-
   std::string kernelObjSrc = "";
   kernelObjSrc += tempDir;
   kernelObjSrc += "/../descriptor.so.kernel_obj.c";
@@ -266,32 +264,27 @@ TCEDevice::tceccCommandLine
   return cmdLine;
 }
 
-bool 
-TCEDevice::isNewKernel(const _cl_command_run* runCmd) 
-{
-  if (curKernel == NULL || runCmd->kernel != curKernel) 
+bool TCEDevice::isNewKernel(const _cl_command_run *runCmd) {
+  if (curKernel == NULL || runCmd->kernel != curKernel)
     return true;
 
   bool newKernel = true;
-  if (runCmd->local_x != curLocalX ||
-      runCmd->local_y != curLocalY ||
-      runCmd->local_z != curLocalZ)
+  if (runCmd->pc.local_size[0] != curLocalX ||
+      runCmd->pc.local_size[1] != curLocalY ||
+      runCmd->pc.local_size[2] != curLocalZ)
     newKernel = true;
   else
     newKernel = false;
   return newKernel;
 }
 
-
-void 
-TCEDevice::updateCurrentKernel(const _cl_command_run* runCmd, 
-                               uint32_t kernelAddr)
-{
+void TCEDevice::updateCurrentKernel(const _cl_command_run *runCmd,
+                                    uint32_t kernelAddr) {
   curKernelAddr = kernelAddr;
   curKernel = runCmd->kernel;
-  curLocalX = runCmd->local_x;
-  curLocalY = runCmd->local_y;
-  curLocalZ = runCmd->local_z;
+  curLocalX = runCmd->pc.local_size[0];
+  curLocalY = runCmd->pc.local_size[1];
+  curLocalZ = runCmd->pc.local_size[2];
 }
 
 void *
@@ -304,7 +297,7 @@ pocl_tce_malloc (void *device_data, cl_mem_flags flags,
   if (chunk == NULL) return NULL;
 
 #ifdef DEBUG_TTA_DRIVER
-  printf("host: malloc %x (host) %d (device) size: %u\n", host_ptr, chunk->start_address, size);
+  printf("host: malloc %p : %lu / %zu\n", host_ptr, chunk->start_address, size);
 #endif
 
   if ((flags & CL_MEM_COPY_HOST_PTR) ||  
@@ -364,7 +357,7 @@ pocl_tce_write (void *data,
   TCEDevice *d = (TCEDevice*)data;
   chunk_info_t *chunk = (chunk_info_t*)device_ptr;
 #ifdef DEBUG_TTA_DRIVER
-  printf ("host: write %x %x %u\n", host_ptr, chunk->start_address + offset,
+  printf ("host: write %p <- %lx / %zu\n", src_host_ptr, chunk->start_address + offset,
           size);
 #endif
   d->copyHostToDevice (src_host_ptr, chunk->start_address + offset, size);
@@ -382,23 +375,12 @@ pocl_tce_read (void *data,
   TCEDevice* d = (TCEDevice*)data;
   chunk_info_t *chunk = (chunk_info_t*)device_ptr;
 #ifdef DEBUG_TTA_DRIVER
-  printf ("host: read to %x (host) from %d (device) %u\n", host_ptr,
+  printf ("host: read %p -> %lx / %zu\n", dst_host_ptr,
           chunk->start_address + offset, size);
 #endif
   d->copyDeviceToHost (chunk->start_address + offset, dst_host_ptr, size);
 }
 
-void *
-pocl_tce_create_sub_buffer (void */*device_data*/, void* buffer, size_t origin, size_t size)
-{
-#ifdef DEBUG_TTA_DRIVER
-  printf("host: create sub buffer %d (buf start) + %d size: %d\n", 
-         ((chunk_info_t*)buffer)->start_address, origin, size);
-#endif
-
-  return create_sub_chunk ((chunk_info_t*)buffer, origin, size);
-}
-
 chunk_info_t*
 pocl_tce_malloc_local (void *device_data, size_t size) 
 {
@@ -413,60 +395,106 @@ pocl_tce_free (cl_device_id device, cl_mem mem_obj)
   free_chunk ((chunk_info_t*) ptr);
 }
 
-void
-pocl_tce_compile_kernel(_cl_command_node *cmd,
-                        cl_kernel kernel, cl_device_id device)
-{
-  if (cmd->type != CL_COMMAND_NDRANGE_KERNEL)
+static void pocl_tce_write_kernel_descriptor(cl_device_id device,
+                                             unsigned device_i,
+                                             cl_kernel kernel) {
+  // Generate the kernel_obj.c file. This should be optional
+  // and generated only for the heterogeneous standalone devices which
+  // need the definitions to accompany the kernels, for the launcher
+  // code.
+  // TODO: the scripts use a generated kernel.h header file that
+  // gets added to this file. No checks seem to fail if that file
+  // is missing though, so it is left out from there for now
+
+  std::stringstream content;
+  pocl_kernel_metadata_t *meta = kernel->meta;
+
+  content << std::endl
+          << "#include <pocl_device.h>" << std::endl
+          << "void _pocl_kernel_" << meta->name
+          << "_workgroup(uint8_t* args, uint8_t*, "
+          << "uint32_t, uint32_t, uint32_t);" << std::endl
+          << "void _pocl_kernel_" << meta->name
+          << "_workgroup_fast(uint8_t* args, uint8_t*, "
+          << "uint32_t, uint32_t, uint32_t);" << std::endl;
+
+  if (device->global_as_id != 0)
+    content << "__attribute__((address_space(" << device->global_as_id << ")))"
+            << std::endl;
+
+  content << "__kernel_metadata _" << meta->name << "_md = {" << std::endl
+          << "     \"" << meta->name << "\"," << std::endl
+          << "     " << meta->num_args << "," << std::endl
+          << "     " << meta->num_locals << "," << std::endl
+          << "     _pocl_kernel_" << meta->name << "_workgroup_fast"
+          << std::endl
+          << " };" << std::endl;
+
+  pocl_cache_write_descriptor(kernel->program, device_i, meta->name,
+                              content.str().c_str(), content.str().size());
+}
+
+void pocl_tce_compile_kernel(_cl_command_node *Command, cl_kernel Kernel,
+                             cl_device_id Device, int Specialize) {
+  if (Command->type != CL_COMMAND_NDRANGE_KERNEL)
     return;
+  _cl_command_run *RunCommand = &Command->command.run;
 
-  void* data = cmd->device->data;
-  TCEDevice *d = (TCEDevice*)data;
+  void *Data = Command->device->data;
+  TCEDevice *Dev = (TCEDevice *)Data;
 
-  if (!kernel)
-    kernel = cmd->command.run.kernel;
-  if (!device)
-    device = cmd->device;
+  if (!Kernel)
+    Kernel = Command->command.run.kernel;
+  if (!Device)
+    Device = Command->device;
 
-  POCL_LOCK(d->tce_compile_lock);
-  int error = pocl_llvm_generate_workgroup_function(device, kernel,
-      cmd->command.run.local_x, cmd->command.run.local_y,
-      cmd->command.run.local_z);
+  POCL_LOCK(Dev->tce_compile_lock);
+  int Error = pocl_llvm_generate_workgroup_function(
+      Command->device_i, Device, Kernel, Command, Specialize);
 
-  if (error) {
-    POCL_UNLOCK(d->tce_compile_lock);
+  if (Error) {
+    POCL_UNLOCK(Dev->tce_compile_lock);
     POCL_MSG_PRINT_GENERAL("TCE: pocl_llvm_generate_workgroup_function()"
-                           " failed for kernel %s\n", kernel->name);
-    assert(error == 0);
+                           " failed for kernel %s\n",
+                           Kernel->name);
+    assert(Error == 0);
   }
 
-  char bytecode[POCL_FILENAME_LENGTH];
+  // 12 == strlen (POCL_PARALLEL_BC_FILENAME)
+  char ByteCode[POCL_FILENAME_LENGTH + 13];
 
-  assert(d != NULL);
-  assert(cmd->command.run.kernel);
-  assert(cmd->command.run.tmp_dir);
+  assert(Dev != NULL);
+  assert(Command->command.run.kernel);
 
-  if (d->isNewKernel(&(cmd->command.run))) {
-    std::string assemblyFileName(cmd->command.run.tmp_dir);
-    assemblyFileName += "/parallel.tpef";
+  char CacheDir[POCL_FILENAME_LENGTH];
+  pocl_cache_kernel_cachedir_path(CacheDir, Kernel->program, Command->device_i,
+                                  Kernel, "", Command, 1);
+  RunCommand->device_data = strdup(CacheDir);
 
-    if (access (assemblyFileName.c_str(), F_OK) != 0)
-      {
-        error = snprintf (bytecode, POCL_FILENAME_LENGTH,
-                          "%s%s", cmd->command.run.tmp_dir, POCL_PARALLEL_BC_FILENAME);
-        TCEString buildCmd =
-          d->tceccCommandLine(&cmd->command.run, bytecode, assemblyFileName);
+  if (Dev->isNewKernel(RunCommand)) {
+
+    pocl_tce_write_kernel_descriptor(Device, Command->device_i, Kernel);
+
+    std::string AssemblyFileName(CacheDir);
+    TCEString TempDir(CacheDir);
+    AssemblyFileName += "/parallel.tpef";
+
+    if (access(AssemblyFileName.c_str(), F_OK) != 0) {
+      Error = snprintf(ByteCode, POCL_FILENAME_LENGTH, "%s%s", CacheDir,
+                       POCL_PARALLEL_BC_FILENAME);
+      TCEString BuildCmd = Dev->tceccCommandLine(RunCommand, TempDir, ByteCode,
+                                                 AssemblyFileName);
 
 #ifdef DEBUG_TTA_DRIVER
-      std::cerr << "CMD: " << buildCmd << std::endl;
+      std::cerr << "CMD: " << BuildCmd << std::endl;
 #endif
-      error = system(buildCmd.c_str());
-      if (error != 0)
+      Error = system(BuildCmd.c_str());
+      if (Error != 0)
         POCL_ABORT("Error while running tcecc.\n");
-      }
+    }
   }
 
-  POCL_UNLOCK(d->tce_compile_lock);
+  POCL_UNLOCK(Dev->tce_compile_lock);
 }
 
 void
@@ -480,10 +508,10 @@ pocl_tce_run(void *data, _cl_command_node* cmd)
 
   assert(d != NULL);
   assert(cmd->command.run.kernel);
-  assert(cmd->command.run.tmp_dir);
+  assert(cmd->command.run.device_data);
 
   if (d->isNewKernel(&(cmd->command.run))) {
-    std::string assemblyFileName(cmd->command.run.tmp_dir);
+    std::string assemblyFileName((const char*)cmd->command.run.device_data);
     assemblyFileName += "/parallel.tpef";
 
     std::string kernelMdSymbolName = "_";
@@ -524,10 +552,13 @@ pocl_tce_run(void *data, _cl_command_node* cmd)
   /* Chunks to be freed after the kernel finishes. */
   ChunkVector tempChunks;
 
-  for (i = 0; i < cmd->command.run.kernel->num_args; ++i)
+  cl_kernel kernel = cmd->command.run.kernel;
+  pocl_kernel_metadata_t *meta = kernel->meta;
+
+  for (i = 0; i < meta->num_args; ++i)
     {
       al = &(cmd->command.run.arguments[i]);
-      if (cmd->command.run.kernel->arg_info[i].is_local)
+      if (ARG_IS_LOCAL (meta->arg_info[i]))
         {
           chunk_info_t* local_chunk = pocl_tce_malloc_local (d, al->size);
           if (local_chunk == NULL)
@@ -535,12 +566,12 @@ pocl_tce_run(void *data, _cl_command_node* cmd)
 
           dev_cmd.args[i] = byteswap_uint32_t (local_chunk->start_address, d->needsByteSwap);
 #ifdef DEBUG_TTA_DRIVER
-          printf ("host: allocated %d bytes of local memory for arg %d @ %d\n", 
+          printf ("host: allocated %zu bytes of local memory for arg %u @ %lu\n",
                   al->size, i, local_chunk->start_address);
 #endif
           tempChunks.push_back(local_chunk);
         }
-      else if (cmd->command.run.kernel->arg_info[i].type == POCL_ARG_TYPE_POINTER)
+      else if (meta->arg_info[i].type == POCL_ARG_TYPE_POINTER)
         {
           /* It's legal to pass a NULL pointer to clSetKernelArguments. In 
              that case we must pass the same NULL forward to the kernel.
@@ -549,8 +580,13 @@ pocl_tce_run(void *data, _cl_command_node* cmd)
           if (al->value == NULL)
             dev_cmd.args[i] = 0;
           else
-            dev_cmd.args[i] = byteswap_uint32_t 
-              (((chunk_info_t*)((*(cl_mem *) (al->value))->device_ptrs[d->parent->dev_id].mem_ptr))->start_address, d->needsByteSwap);
+            {
+              cl_mem m = (*(cl_mem *)(al->value));
+              void *p = m->device_ptrs[d->parent->dev_id].mem_ptr;
+              dev_cmd.args[i] = byteswap_uint32_t (
+                  ((chunk_info_t *)p)->start_address + al->offset,
+                  d->needsByteSwap);
+            }
         }
       else /* The scalar values should be byteswapped by the user. */
         {
@@ -560,7 +596,7 @@ pocl_tce_run(void *data, _cl_command_node* cmd)
           if (arg_space == NULL)
             POCL_ABORT ("Could not allocate memory from the device argument space. Out of global mem?\n");
 #ifdef DEBUG_TTA_DRIVER
-          printf ("host: copied value from %x to global argument memory\n", al->value);
+          printf ("host: copied value from %p to global argument memory\n", al->value);
 #endif
           dev_cmd.args[i] = byteswap_uint32_t (arg_space->start_address, d->needsByteSwap);
           tempChunks.push_back(arg_space);
@@ -568,19 +604,17 @@ pocl_tce_run(void *data, _cl_command_node* cmd)
     }
 
   /* Allocate the automatic local buffers. */
-  for (std::size_t i = cmd->command.run.kernel->num_args;
-       i < cmd->command.run.kernel->num_args + cmd->command.run.kernel->num_locals;
-       ++i) 
+  for (i = 0; i < meta->num_locals; ++i)
     {
-      al = &(cmd->command.run.arguments[i]);
-      chunk_info_t* local_chunk = pocl_tce_malloc_local (d, al->size);
+      size_t s = meta->local_sizes[i];
+      chunk_info_t* local_chunk = pocl_tce_malloc_local (d, s);
       if (local_chunk == NULL)
         POCL_ABORT ("Could not allocate memory for an automatic local argument. Out of local mem?\n");
 
-      dev_cmd.args[i] = byteswap_uint32_t (local_chunk->start_address, d->needsByteSwap);
+      dev_cmd.args[meta->num_args + i] = byteswap_uint32_t (local_chunk->start_address, d->needsByteSwap);
 #ifdef DEBUG_TTA_DRIVER
-      printf ("host: allocated %d bytes of local memory for automated local arg %d @ %d\n", 
-              al->size, i, local_chunk->start_address);
+      printf ("host: allocated %zu bytes of local memory for automated local arg %u @ %lu\n",
+              s, (meta->num_args + i), local_chunk->start_address);
 #endif      
       tempChunks.push_back(local_chunk);
     }
@@ -647,6 +681,8 @@ pocl_tce_run(void *data, _cl_command_node* cmd)
        i != tempChunks.end(); ++i) 
     free_chunk (*i);
 
+  POCL_MEM_FREE(cmd->command.run.device_data);
+
 #ifdef DEBUG_TTA_DRIVER
   printf("host: local memory allocations:\n");
   print_chunks (d->local_mem.chunks);
@@ -914,7 +950,7 @@ static void tce_command_scheduler (TCEDevice *d)
       POCL_UNLOCK(d->cq_lock);
       assert (node->event->status == CL_SUBMITTED);
       if (node->type == CL_COMMAND_NDRANGE_KERNEL)
-        pocl_tce_compile_kernel(node, NULL, NULL);
+        pocl_tce_compile_kernel(node, NULL, NULL, 1);
       pocl_exec_command(node);
       POCL_LOCK(d->cq_lock);
     }
@@ -967,7 +1003,7 @@ pocl_tce_notify (cl_device_id device, cl_event event, cl_event finished)
   _cl_command_node * volatile node = event->command;
 
   if (finished->status < CL_COMPLETE) {
-    POCL_UPDATE_EVENT_FAILED(event);
+    pocl_update_event_failed(event);
     return;
   }
 
@@ -976,7 +1012,7 @@ pocl_tce_notify (cl_device_id device, cl_event event, cl_event finished)
 
   if (pocl_command_is_ready(event)) {
     if (event->status == CL_QUEUED) {
-      POCL_UPDATE_EVENT_SUBMITTED(event);
+      pocl_update_event_submitted(event);
       POCL_LOCK(d->cq_lock);
       CDL_DELETE(d->command_list, node);
       CDL_PREPEND(d->ready_list, node);
