@@ -1,17 +1,18 @@
-/* ttasim.cc - a pocl device driver for simulating TTA devices using TCE's ttasim
+/* ttasim.cc - a pocl device driver for simulating TTA devices using TCE's
+   ttasim
+
+   Copyright (c) 2012-2019 Pekka Jääskeläinen / Tampere University
 
-   Copyright (c) 2012-2015 Pekka Jääskeläinen / Tampere University of Technology
-   
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -90,7 +91,6 @@ pocl_ttasim_init_device_ops(struct pocl_device_ops *ops)
   ops->map_mem = pocl_tce_map_mem;
   ops->unmap_mem = pocl_tce_unmap_mem;
   ops->run = pocl_tce_run;
-  ops->get_timer_value = pocl_ttasim_get_timer_value;
   ops->init_build = pocl_tce_init_build;
   ops->flush = pocl_tce_flush;
   ops->join = pocl_tce_join;
@@ -98,9 +98,7 @@ pocl_ttasim_init_device_ops(struct pocl_device_ops *ops)
   ops->compile_kernel = pocl_tce_compile_kernel;
   ops->broadcast = pocl_broadcast;
   ops->notify = pocl_tce_notify;
-  ops->update_event = NULL; //pocl_ttasim_update_event;
   ops->build_hash = pocl_tce_build_hash;
-
 }
 
 
@@ -566,6 +564,7 @@ pocl_ttasim_init (unsigned j, cl_device_id dev, const char* parameters)
   dev->global_as_id = TTA_ASID_GLOBAL;
   dev->local_as_id = TTA_ASID_LOCAL;
   dev->constant_as_id = TTA_ASID_CONSTANT;
+  dev->context_as_id = 0;
 
   SETUP_DEVICE_CL_VERSION (TCE_DEVICE_CL_VERSION_MAJOR,
                            TCE_DEVICE_CL_VERSION_MINOR);
@@ -590,49 +589,27 @@ pocl_ttasim_uninit (unsigned j, cl_device_id device)
   return CL_SUCCESS;
 }
 
-
-cl_ulong
-pocl_ttasim_get_timer_value (void *data) 
-{
-  TTASimDevice *d = (TTASimDevice*)data;
-  return d->timeStamp();
-}
-
 void pocl_ttasim_update_event (cl_device_id device, cl_event event, cl_int status)
 {
+  TTASimDevice *d = (TTASimDevice *)device->data;
   switch (status)
     {
     case CL_QUEUED:
-      event->status = status;
       if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
-        event->time_queue = device->ops->get_timer_value(device->data);
+        event->time_queue = d->timeStamp();
       break;
     case CL_SUBMITTED:
-      event->status = status;
       if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
-        event->time_submit = device->ops->get_timer_value(device->data);
+        event->time_submit = d->timeStamp();
       break;
     case CL_RUNNING:
-      event->status = status;
-      if (event->command_type == CL_COMMAND_NDRANGE_KERNEL)
-        break;
       if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
-        event->time_start = device->ops->get_timer_value(device->data);
+        event->time_start = d->timeStamp();
       break;
     case CL_COMPLETE:
       POCL_MSG_PRINT_INFO("TTA: Command complete, event %d\n", event->id);
       POCL_LOCK_OBJ (event);
-      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE &&
-          event->command_type != CL_COMMAND_NDRANGE_KERNEL)
-        event->time_end = device->ops->get_timer_value(device->data);
-      device->ops->broadcast(event);
-      pocl_mem_objs_cleanup (event);
-      event->status = CL_COMPLETE;
-
-      POCL_UNLOCK_OBJ (event);
-      break;
-    default:
-      assert("Invalid event status\n");
-      break;
+      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
+        event->time_end = d->timeStamp();
     }
 }
