@@ -1,7 +1,7 @@
 /* OpenCL runtime library: compile_and_link_program()
 
    Copyright (c) 2011-2013 Universidad Rey Juan Carlos,
-                 2011-2018 Pekka Jääskeläinen
+                 2011-2019 Pekka Jääskeläinen
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
@@ -67,6 +67,9 @@ static const char cl_parameters[] =
   "-cl-std=CL1.1 "
   "-cl-std=CL2.0 "
   "-cl-kernel-arg-info "
+  "-cl-strict-aliasing "
+  "-cl-denorms-are-zero "
+  "-cl-no-signed-zeros "
   "-w "
   "-g "
   "-Werror ";
@@ -84,11 +87,6 @@ static const char cl_program_link_options[] =
   "-cl-finite-math-only "
   "-cl-fast-relaxed-math ";
 
-static const char cl_parameters_supported_after_clang_3_9[] =
-  "-cl-strict-aliasing " /* deprecated after OCL1.0 */
-  "-cl-denorms-are-zero "
-  "-cl-no-signed-zeros ";
-
 static const char cl_parameters_not_yet_supported_by_clang[] =
   "-cl-uniform-work-group-size ";
 
@@ -114,12 +112,8 @@ static const char cl_parameters_not_yet_supported_by_clang[] =
   }
 
 #ifdef OCS_AVAILABLE
-/* if 'only_spmd_devices' is set, prebuild the kernel binaries only for the
- * SPMD devices in the context, otherwise build binaries for all devices.
- * The former is useful for clBuildProgram(), the latter for
- * clGetProgramInfo(CL_PROGRAM_BINARIES). */
 cl_int
-program_compile_dynamic_wg_binaries (cl_program program, int only_spmd_devices)
+program_compile_dynamic_wg_binaries (cl_program program)
 {
   unsigned i, device_i;
   _cl_command_node cmd;
@@ -128,6 +122,11 @@ program_compile_dynamic_wg_binaries (cl_program program, int only_spmd_devices)
   if (program->num_kernels == 0)
     return CL_SUCCESS;
 
+  /* For binaries of other than Executable type (libraries, compiled but
+   * not linked programs, etc), do not attempt to compile the kernels. */
+  if (program->binary_type != CL_PROGRAM_BINARY_TYPE_EXECUTABLE)
+    return CL_SUCCESS;
+
   memset(&cmd, 0, sizeof(_cl_command_node));
   cmd.type = CL_COMMAND_NDRANGE_KERNEL;
 
@@ -143,11 +142,8 @@ program_compile_dynamic_wg_binaries (cl_program program, int only_spmd_devices)
       if (program->pocl_binaries[device_i] || (!program->binaries[device_i]))
         continue;
 
-      if (only_spmd_devices && (device->spmd == CL_FALSE))
-        continue;
-
       cmd.device = device;
-      cmd.command.run.device_i = device_i;
+      cmd.device_i = device_i;
 
       struct _cl_kernel fake_k;
       memset (&fake_k, 0, sizeof (fake_k));
@@ -173,13 +169,23 @@ program_compile_dynamic_wg_binaries (cl_program program, int only_spmd_devices)
               local_z = kernel->meta->reqd_wg_size[2];
             }
 
-          cmd.command.run.local_x = local_x;
-          cmd.command.run.local_y = local_y;
-          cmd.command.run.local_z = local_z;
+          cmd.command.run.pc.local_size[0] = local_x;
+          cmd.command.run.pc.local_size[1] = local_y;
+          cmd.command.run.pc.local_size[2] = local_z;
 
           cmd.command.run.kernel = kernel;
 
-          device->ops->compile_kernel (&cmd, kernel, device);
+          /* Force generate a generic WG function to ensure generality. */
+          device->ops->compile_kernel (&cmd, kernel, device, 0);
+
+          /* Then generate a specialized one with goffset 0 since it's a very
+             common case. */
+
+          cmd.command.run.pc.global_offset[0]
+              = cmd.command.run.pc.global_offset[1]
+              = cmd.command.run.pc.global_offset[2] = 0;
+          cmd.command.run.force_large_grid_wg_func = 1;
+          device->ops->compile_kernel (&cmd, kernel, device, 1);
         }
     }
 
@@ -228,8 +234,8 @@ process_options (const char *options, char *modded_options, char *link_options,
   while (token != NULL)
     {
       /* check if parameter is supported compiler parameter */
-      if (memcmp (token, "-cl", 3) == 0 || memcmp (token, "-w", 2) == 0
-          || memcmp (token, "-Werror", 7) == 0)
+      if (strncmp (token, "-cl", 3) == 0 || strncmp (token, "-w", 2) == 0
+          || strncmp (token, "-Werror", 7) == 0)
         {
           if (strstr (cl_program_link_options, token))
             {
@@ -262,19 +268,6 @@ process_options (const char *options, char *modded_options, char *link_options,
               /* the LLVM API call pushes the parameters directly to the
                  frontend without using -Xclang */
             }
-          else if (strstr (cl_parameters_supported_after_clang_3_9, token))
-            {
-#ifndef LLVM_OLDER_THAN_3_9
-/* the LLVM API call pushes the parameters directly to the
- * frontend without using -Xclang*/
-#else
-              APPEND_TO_MAIN_BUILD_LOG (
-                  "This build option is supported after clang3.9: %s\n",
-                  token);
-              token = strtok_r (NULL, " ", &saveptr);
-              continue;
-#endif
-            }
           else if (strstr (cl_parameters_not_yet_supported_by_clang, token))
             {
               APPEND_TO_MAIN_BUILD_LOG (
@@ -290,14 +283,12 @@ process_options (const char *options, char *modded_options, char *link_options,
               goto ERROR;
             }
         }
-      else if (memcmp (token, "-g", 2) == 0)
+      else if (strncmp (token, "-g", 2) == 0)
         {
-#ifndef LLVM_OLDER_THAN_3_8
           token = "-dwarf-column-info -debug-info-kind=limited " \
 	    "-dwarf-version=4 -debugger-tuning=gdb";
-#endif
         }
-      else if (memcmp (token, "-D", 2) == 0 || memcmp (token, "-I", 2) == 0)
+      else if (strncmp (token, "-D", 2) == 0 || strncmp (token, "-I", 2) == 0)
         {
           APPEND_TOKEN();
           /* if there is a space in between, then next token is part
@@ -310,11 +301,11 @@ process_options (const char *options, char *modded_options, char *link_options,
               continue;
             }
         }
-      else if (memcmp (token, "-x", 2) == 0 && strlen (token) == 2)
+      else if (strncmp (token, "-x", 2) == 0 && strlen (token) == 2)
         {
           /* only "-x spir" is valid for the "-x" option */
           token = strtok_r (NULL, " ", &saveptr);
-          if (!token || memcmp (token, "spir", 4) != 0)
+          if (!token || strncmp (token, "spir", 4) != 0)
             {
               APPEND_TO_MAIN_BUILD_LOG (
                   "Invalid parameter to -x build option\n");
@@ -334,7 +325,7 @@ process_options (const char *options, char *modded_options, char *link_options,
           token = strtok_r (NULL, " ", &saveptr);
           continue;
         }
-      else if (memcmp (token, "-spir-std=1.2", 13) == 0)
+      else if (strncmp (token, "-spir-std=1.2", 13) == 0)
         {
           /* "-spir-std=" flags are not valid when building from source */
           if (program->source)
@@ -349,7 +340,7 @@ process_options (const char *options, char *modded_options, char *link_options,
           token = strtok_r (NULL, " ", &saveptr);
           continue;
         }
-      else if (memcmp (token, "-create-library", 15) == 0)
+      else if (strncmp (token, "-create-library", 15) == 0)
         {
           if (!linking)
             {
@@ -362,7 +353,7 @@ process_options (const char *options, char *modded_options, char *link_options,
           token = strtok_r (NULL, " ", &saveptr);
           continue;
         }
-      else if (memcmp (token, "-enable-link-options", 20) == 0)
+      else if (strncmp (token, "-enable-link-options", 20) == 0)
         {
           if (!linking)
             {
@@ -417,15 +408,22 @@ free_meta (cl_program program)
       for (i = 0; i < program->num_kernels; i++)
         {
           pocl_kernel_metadata_t *meta = &program->kernel_meta[i];
+          POCL_MEM_FREE (meta->build_hash);
+          if (meta->builtin_kernel)
+            continue;
           POCL_MEM_FREE (meta->attributes);
           POCL_MEM_FREE (meta->name);
+          for (j = 0; j < meta->num_args; ++j)
+            {
+              POCL_MEM_FREE (meta->arg_info[j].name);
+              POCL_MEM_FREE (meta->arg_info[j].type_name);
+            }
           POCL_MEM_FREE (meta->arg_info);
           for (j = 0; j < program->num_devices; ++j)
             if (meta->data[j] != NULL)
               meta->data[j] = NULL; // TODO free data in driver callback
           POCL_MEM_FREE (meta->data);
           POCL_MEM_FREE (meta->local_sizes);
-          POCL_MEM_FREE (meta->build_hash);
         }
       POCL_MEM_FREE (program->kernel_meta);
     }
@@ -518,11 +516,14 @@ compile_and_link_program(int compile_program,
   POCL_GOTO_LABEL_ON (PFN_NOTIFY, program->kernels, CL_INVALID_OPERATION,
                       "Program already has kernels\n");
 
-  POCL_GOTO_LABEL_ON (PFN_NOTIFY,
-                      (program->source == NULL && program->binaries == NULL),
-                      CL_INVALID_PROGRAM,
-                      "Program doesn't have sources or binaries! You need "
-                      "to call clCreateProgramWith{Binary|Source} first\n");
+  POCL_GOTO_LABEL_ON (
+      PFN_NOTIFY,
+      (program->source == NULL && program->binaries == NULL
+       && program->builtin_kernel_names == NULL),
+      CL_INVALID_PROGRAM,
+      "Program doesn't have sources, binaries nor builtin-kernel names. You "
+      "need "
+      "to call clCreateProgramWith{Binary|Source|BuiltinKernels} first\n");
 
   POCL_GOTO_LABEL_ON (PFN_NOTIFY,
                       ((program->source == NULL) && (link_program == 0)),
@@ -619,8 +620,13 @@ compile_and_link_program(int compile_program,
         }
       actually_built++;
 
+      /* clCreateProgramWithBuiltinKernels */
+      /* No build step supported at the moment for built-in kernels. */
+      if (program->builtin_kernel_names)
+        continue;
+
       /* clCreateProgramWithSource */
-      if (program->source)
+      else if (program->source)
         {
 #ifdef OCS_AVAILABLE
           if (device->compiler_available == CL_TRUE)
@@ -759,10 +765,11 @@ compile_and_link_program(int compile_program,
       /* Read binaries from program.bc to memory */
       if (program->binaries[device_i] == NULL)
         {
-          errcode = pocl_read_file(program_bc_path, &binary, &fsize);
-          POCL_GOTO_ERROR_ON(errcode, CL_BUILD_ERROR,
-                             "Failed to read binaries from program.bc to "
-                             "memory: %s\n", program_bc_path);
+          errcode = pocl_read_file (program_bc_path, &binary, &fsize);
+          POCL_GOTO_ERROR_ON (errcode, CL_BUILD_ERROR,
+                              "Failed to read binaries from program.bc to "
+                              "memory: %s\n",
+                              program_bc_path);
 
           program->binary_sizes[device_i] = (size_t)fsize;
           program->binaries[device_i] = (unsigned char *)binary;
@@ -770,14 +777,14 @@ compile_and_link_program(int compile_program,
 
       if (program->llvm_irs[device_i] == NULL)
         {
-          pocl_update_program_llvm_irs(program, device_i);
+          pocl_update_program_llvm_irs (program, device_i);
         }
-      /* Maintain a 'last_accessed' file in every program's
-       * cache directory. Will be useful for cache pruning script
-       * that flushes old directories based on LRU */
-      pocl_cache_update_program_last_access(program, device_i);
 #endif
 
+     /* Maintain a 'last_accessed' file in every program's
+      * cache directory. Will be useful for cache pruning script
+      * that flushes old directories based on LRU */
+      pocl_cache_update_program_last_access (program, device_i);
     }
 
   POCL_GOTO_ERROR_ON ((actually_built < num_devices), build_error_code,
@@ -800,10 +807,35 @@ compile_and_link_program(int compile_program,
 
   assert(program->num_kernels == 0);
 
+  if (program->binary_type != CL_PROGRAM_BINARY_TYPE_EXECUTABLE)
+    {
+      errcode = CL_SUCCESS;
+      goto FINISH;
+    }
+
   /* get non-device-specific kernel metadata. We can stop after finding
    * the first method that works.*/
   for (device_i = 0; device_i < program->num_devices; device_i++)
     {
+      if (program->builtin_kernel_names)
+        {
+          program->num_kernels = program->num_builtin_kernels;
+          if (program->num_kernels)
+            {
+              cl_device_id device = program->devices[device_i];
+              program->kernel_meta = calloc (program->num_kernels,
+                                             sizeof (pocl_kernel_metadata_t));
+
+              size_t i;
+              for (i = 0; i < program->num_kernels; ++i)
+                {
+                  device->ops->get_builtin_kernel_metadata (
+                      device->data, program->builtin_kernel_names[i],
+                      &program->kernel_meta[i]);
+                }
+            }
+          break;
+        }
 #ifdef OCS_AVAILABLE
       if (program->binaries[device_i])
         {
@@ -833,7 +865,7 @@ compile_and_link_program(int compile_program,
     }
 
   POCL_GOTO_ERROR_ON ((device_i >= program->num_devices), CL_INVALID_BINARY,
-                      "Could find kernel metadata in the built program\n");
+                      "Couldn't find kernel metadata in the built program\n");
 
   /* calculate device-specific kernel hashes. */
   for (j = 0; j < program->num_kernels; ++j)
@@ -841,10 +873,11 @@ compile_and_link_program(int compile_program,
       program->kernel_meta[j].build_hash
           = calloc (program->num_devices, sizeof (pocl_kernel_hash_t));
 
-      for (device_i = 0; device_i < program->num_devices; device_i++)
-        {
-          pocl_calculate_kernel_hash (program, j, device_i);
-        }
+      if (program->builtin_kernel_names == NULL)
+        for (device_i = 0; device_i < program->num_devices; device_i++)
+          {
+            pocl_calculate_kernel_hash (program, j, device_i);
+          }
     }
 
   errcode = CL_SUCCESS;
