@@ -1,12 +1,12 @@
 /* OpenCL runtime library: caching functions
 
-   Copyright (c) 2015 pocl developers
+   Copyright (c) 2015-2019 pocl developers
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
+   of this software and associated documentation files (the "Software"), to
+   deal in the Software without restriction, including without limitation the
+   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+   sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
 
    The above copyright notice and this permission notice shall be included in
@@ -16,9 +16,9 @@
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+   IN THE SOFTWARE.
 */
 
 #include <errno.h>
@@ -45,9 +45,11 @@
 #define POCL_LAST_ACCESSED_FILENAME "/last_accessed"
 /* The filename in which the program's build log is stored */
 #define POCL_BUILDLOG_FILENAME      "/build.log"
-/* The filename in which the program source is stored in the program's temp dir. */
+/* The filename in which the program source is stored in the program's temp
+ * dir. */
 #define POCL_PROGRAM_CL_FILENAME "/program.cl"
-/* The filename in which the program LLVM bc is stored in the program's temp dir. */
+/* The filename in which the program LLVM bc is stored in the program's temp
+ * dir. */
 #define POCL_PROGRAM_BC_FILENAME "/program.bc"
 
 static char cache_topdir[POCL_FILENAME_LENGTH];
@@ -108,86 +110,103 @@ void pocl_cache_program_bc_path(char*        program_bc_path,
                        device_i, POCL_PROGRAM_BC_FILENAME);
 }
 
-void pocl_cache_kernel_cachedir_path (char* kernel_cachedir_path,
-                                             cl_program program,
-                                             unsigned device_i,
-                                             cl_kernel kernel,
-                                             char* append_str,
-                                             size_t local_x,
-                                             size_t local_y,
-                                             size_t local_z)
+/* Return the cache directory for the given work-group function.
+   If specialized = 1, specialization parameters are derived from run_cmd,
+   otherwise a generic directory name is returned.
+
+   The current specialization parameters are:
+   - local size
+   - if the global offset is zero (in all dimensions) or not
+   - if the grid size in any dimension is smaller than a device
+   specified limit ("smallgrid" specialization)
+*/
+void
+pocl_cache_kernel_cachedir_path (char *kernel_cachedir_path,
+                                 cl_program program, unsigned device_i,
+                                 cl_kernel kernel, const char *append_str,
+                                 _cl_command_node *command, int specialized)
+{
+  int bytes_written;
+  _cl_command_run *run_cmd = &command->command.run;
+  char tempstring[POCL_FILENAME_LENGTH];
+  cl_device_id dev = command->device;
+  size_t max_grid_width = pocl_cmd_max_grid_dim_width (run_cmd);
+  bytes_written = snprintf (
+      tempstring, POCL_FILENAME_LENGTH, "/%s/%zu-%zu-%zu%s%s%s", kernel->name,
+      !specialized ? 0 : run_cmd->pc.local_size[0],
+      !specialized ? 0 : run_cmd->pc.local_size[1],
+      !specialized ? 0 : run_cmd->pc.local_size[2],
+      (specialized && run_cmd->pc.global_offset[0] == 0
+       && run_cmd->pc.global_offset[1] == 0
+       && run_cmd->pc.global_offset[2] == 0)
+          ? "-goffs0"
+          : "",
+      specialized && !run_cmd->force_large_grid_wg_func
+              && max_grid_width < dev->grid_width_specialization_limit
+          ? "-smallgrid"
+          : "",
+      append_str);
+  assert (bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
+
+  program_device_dir (kernel_cachedir_path, program, device_i, tempstring);
+}
+
+void
+pocl_cache_kernel_cachedir (char *kernel_cachedir_path, cl_program program,
+                            unsigned device_i, const char *kernel_name)
 {
   int bytes_written;
   char tempstring[POCL_FILENAME_LENGTH];
-
-  if (program->devices[device_i]->spmd)
-    {
-      bytes_written = snprintf(tempstring, POCL_FILENAME_LENGTH,
-                               "/%s/SPMD%s", kernel->name, append_str);
-    }
-  else
-    {
-      bytes_written = snprintf(tempstring, POCL_FILENAME_LENGTH,
-                               "/%s/%zu-%zu-%zu%s", kernel->name,
-                               local_x, local_y, local_z, append_str);
-    }
-
-  assert(bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
-
-  program_device_dir(kernel_cachedir_path, program, device_i, tempstring);
-
+  bytes_written
+      = snprintf (tempstring, POCL_FILENAME_LENGTH, "/%s", kernel_name);
+  assert (bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
+  program_device_dir (kernel_cachedir_path, program, device_i, tempstring);
 }
 
-void pocl_cache_kernel_cachedir(char* kernel_cachedir_path, cl_program   program,
-                                unsigned device_i, cl_kernel kernel)
-{
-  int bytes_written;
-  char tempstring[POCL_FILENAME_LENGTH];
-  bytes_written = snprintf(tempstring, POCL_FILENAME_LENGTH,
-                           "/%s", kernel->name);
-  assert(bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
-  program_device_dir(kernel_cachedir_path, program, device_i, tempstring);
-}
-
-
 // required in llvm API
-void pocl_cache_work_group_function_path(char* parallel_bc_path, cl_program program,
-                               unsigned device_i, cl_kernel kernel,
-                               size_t local_x, size_t local_y,
-                               size_t local_z) {
-    assert(kernel->name);
+void
+pocl_cache_work_group_function_path (char *parallel_bc_path,
+                                     cl_program program, unsigned device_i,
+                                     cl_kernel kernel,
+                                     _cl_command_node *command, int specialize)
+{
+  assert (kernel->name);
 
-    pocl_cache_kernel_cachedir_path(parallel_bc_path, program,
-                         device_i, kernel, POCL_PARALLEL_BC_FILENAME,
-                         local_x, local_y, local_z);
+  pocl_cache_kernel_cachedir_path (parallel_bc_path, program, device_i, kernel,
+                                   POCL_PARALLEL_BC_FILENAME, command,
+                                   specialize);
 }
 
-void pocl_cache_final_binary_path(char* final_binary_path, cl_program program,
-                               unsigned device_i, cl_kernel kernel,
-                               size_t local_x, size_t local_y,
-                               size_t local_z) {
-    assert(kernel->name);
+/* Return the final binary path for the given work-group function.
+   If specialized is 1, find the WG function specialized for the
+   given command's properties, if 0, return the path to a generic version. */
+void
+pocl_cache_final_binary_path (char *final_binary_path, cl_program program,
+                              unsigned device_i, cl_kernel kernel,
+                              _cl_command_node *command, int specialized)
+{
+  assert (kernel->name);
 
+  /* TODO: This should be probably refactored to either get the binary name
+     from the device itself, or let the device ops call
+     pocl_llvm_generate_workgroup_function() on their own */
 
-    /* TODO this should be probably refactored to either
-     * get the binary name from the device itself, or
-     * let the device ops call pocl_llvm_generate_workgroup_function() on their own */
+  int bytes_written;
+  char final_binary_name[POCL_FILENAME_LENGTH];
 
-    int bytes_written;
-    char final_binary_name[POCL_FILENAME_LENGTH];
+  /* FIXME: Why different naming for SPMD and why the .brig suffix? */
+  if (kernel->program->devices[device_i]->spmd)
+    bytes_written = snprintf (final_binary_name, POCL_FILENAME_LENGTH,
+                              "%s.brig", POCL_PARALLEL_BC_FILENAME);
+  else
+    bytes_written = snprintf (final_binary_name, POCL_FILENAME_LENGTH,
+                              "/%s.so", kernel->name);
 
-    if (program->devices[device_i]->spmd)
-        bytes_written = snprintf(final_binary_name, POCL_FILENAME_LENGTH,
-                                 "%s.brig", POCL_PARALLEL_BC_FILENAME);
-    else
-        bytes_written = snprintf(final_binary_name, POCL_FILENAME_LENGTH,
-                                 "/%s.so", kernel->name);
+  assert (bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
 
-    assert(bytes_written > 0 && bytes_written < POCL_FILENAME_LENGTH);
-
-    pocl_cache_kernel_cachedir_path(final_binary_path, program,
-                         device_i, kernel, final_binary_name,
-                         local_x, local_y, local_z);
+  pocl_cache_kernel_cachedir_path (final_binary_path, program, device_i,
+                                   kernel, final_binary_name, command,
+                                   specialized);
 }
 
 /******************************************************************************/
@@ -219,7 +238,7 @@ pocl_cache_write_program_source (char *program_cl_path, cl_program program)
 int
 pocl_cache_write_kernel_objfile (char *objfile_path,
                                  const char *objfile_content,
-                                 size_t objfile_size)
+                                 uint64_t objfile_size)
 {
   return pocl_write_tempfile (objfile_path, tempfile_pattern, ".so.o",
                               objfile_content, objfile_size, NULL);
@@ -228,7 +247,7 @@ pocl_cache_write_kernel_objfile (char *objfile_path,
 int
 pocl_cache_write_spirv (char *spirv_path,
                         const char *spirv_content,
-                        size_t file_size)
+                        uint64_t file_size)
 {
   return pocl_write_tempfile (spirv_path, tempfile_pattern, ".spirv",
                               spirv_content, file_size, NULL);
@@ -321,36 +340,30 @@ int pocl_cache_append_to_buildlog(cl_program  program,
 /******************************************************************************/
 
 #ifdef OCS_AVAILABLE
-int pocl_cache_write_kernel_parallel_bc(void*        bc,
-                                        cl_program   program,
-                                        unsigned     device_i,
-                                        cl_kernel    kernel,
-                                        size_t       local_x,
-                                        size_t       local_y,
-                                        size_t       local_z) {
-    assert(bc);
+int
+pocl_cache_write_kernel_parallel_bc (void *bc, cl_program program,
+                                     int device_i, cl_kernel kernel,
+                                     _cl_command_node *command, int specialize)
+{
+  assert (bc);
+  char kernel_parallel_path[POCL_FILENAME_LENGTH];
+  pocl_cache_kernel_cachedir_path (kernel_parallel_path, program, device_i,
+                                   kernel, "", command, specialize);
+  int err = pocl_mkdir_p (kernel_parallel_path);
+  if (err)
+    return err;
 
-    char kernel_parallel_path[POCL_FILENAME_LENGTH];
-    pocl_cache_kernel_cachedir_path(kernel_parallel_path, program, device_i,
-                                    kernel, "", local_x, local_y, local_z);
-    int err = pocl_mkdir_p(kernel_parallel_path);
-    if (err)
-      return err;
-
-    assert( strlen(kernel_parallel_path) <
-            (POCL_FILENAME_LENGTH - strlen(POCL_PARALLEL_BC_FILENAME)));
-    strcat(kernel_parallel_path, POCL_PARALLEL_BC_FILENAME);
-    return pocl_write_module(bc, kernel_parallel_path, 0);
+  assert (strlen (kernel_parallel_path)
+          < (POCL_FILENAME_LENGTH - strlen (POCL_PARALLEL_BC_FILENAME)));
+  strcat (kernel_parallel_path, POCL_PARALLEL_BC_FILENAME);
+  return pocl_write_module (bc, kernel_parallel_path, 0);
 }
 
-
 /******************************************************************************/
 
 static inline void
-build_program_compute_hash(cl_program program,
-                           unsigned   device_i,
-                           const char*      preprocessed_source,
-                           size_t     source_len)
+build_program_compute_hash (cl_program program, unsigned device_i,
+                            const char *preprocessed_source, size_t source_len)
 {
     SHA1_CTX hash_ctx;
     unsigned i;
@@ -401,10 +414,6 @@ build_program_compute_hash(cl_program program,
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) wg_method, strlen(wg_method));
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) PACKAGE_VERSION,
                      strlen(PACKAGE_VERSION));
-#ifdef POCL_KCACHE_SALT
-    pocl_SHA1_Update(&hash_ctx, (uint8_t*) POCL_KCACHE_SALT,
-                     strlen(POCL_KCACHE_SALT));
-#endif
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) LLVM_VERSION,
                      strlen(LLVM_VERSION));
     pocl_SHA1_Update(&hash_ctx, (uint8_t*) POCL_BUILD_TIMESTAMP,
@@ -436,36 +445,6 @@ build_program_compute_hash(cl_program program,
 #endif
 
 
-#ifdef ANDROID
-static
-char* pocl_get_process_name()
-{
-    char tmpStr[64], cmdline[512], *processName=NULL;
-    FILE *statusFile;
-    size_t len, i, begin;
-
-    snprintf(tmpStr, 64, "/proc/%d/cmdline", getpid());
-    statusFile=fopen(tmpStr, "r");
-    if (statusFile == NULL)
-        return NULL;
-
-    if (fgets(cmdline, 511, statusFile) != NULL) {
-        len=strlen(cmdline);
-        begin=0;
-        for (i=len-1; i >= 0; i--) { /* Extract program-name after last '/' */
-            if (cmdline[i] == '/') {
-                begin=i + 1;
-                break;
-            }
-        }
-        processName=strdup(cmdline + begin);
-    }
-
-    fclose(statusFile);
-    return processName;
-}
-#endif
-
 /******************************************************************************/
 
 int
@@ -483,17 +462,10 @@ pocl_cache_init_topdir ()
   if (tmp_path)
     {
       needed = snprintf (cache_topdir, POCL_FILENAME_LENGTH, "%s", tmp_path);
-    } else     {
-#ifdef POCL_ANDROID
-        char* process_name = pocl_get_process_name();
-        needed = snprintf(cache_topdir, POCL_FILENAME_LENGTH,
-                          "/data/data/%s/cache/", process_name);
-        free(process_name);
+    } else {
+#ifdef __ANDROID__
+        POCL_ABORT ("Please set the POCL_CACHE_DIR env var to your app's cache directory (Context.getCacheDir())\n");
 
-        if (!pocl_exists(cache_topdir))
-            needed = snprintf(cache_topdir,
-                              POCL_FILENAME_LENGTH,
-                              "/sdcard/pocl/kcache");
 #elif defined(_MSC_VER) || defined(__MINGW32__)
         tmp_path = getenv("LOCALAPPDATA");
         if (!tmp_path)
