@@ -1,25 +1,25 @@
 /* pocl_cl.h - local runtime library declarations.
 
    Copyright (c) 2011 Universidad Rey Juan Carlos
-                 2011-2012 Pekka Jääskeläinen
-   
+                 2011-2019 Pekka Jääskeläinen
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
-   of this software and associated documentation files (the "Software"), to deal
-   in the Software without restriction, including without limitation the rights
-   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-   copies of the Software, and to permit persons to whom the Software is
+   of this software and associated documentation files (the "Software"), to
+   deal in the Software without restriction, including without limitation the
+   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+   sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-   THE SOFTWARE.
+   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+   IN THE SOFTWARE.
 */
 
 #ifndef POCL_CL_H
@@ -29,17 +29,19 @@
 
 #include <assert.h>
 #include <stdio.h>
-#ifndef _MSC_VER
-#  include <ltdl.h>
-#else
+
+#ifdef _MSC_VER
 #  include "vccompat.hpp"
 #endif
+/* To get adaptive mutex type */
+#ifndef __USE_GNU
+#define __USE_GNU
+#endif
 #include <pthread.h>
 #ifdef HAVE_CLOCK_GETTIME
 #include <time.h>
 #endif
 
-#define CL_USE_DEPRECATED_OPENCL_1_1_APIS
 #ifdef BUILD_ICD
 #  include "pocl_icd.h"
 #endif
@@ -58,6 +60,7 @@
 # endif
 #endif
 
+typedef struct pocl_kernel_metadata_s pocl_kernel_metadata_t;
 typedef pthread_mutex_t pocl_lock_t;
 #define POCL_LOCK_INITIALIZER PTHREAD_MUTEX_INITIALIZER
 
@@ -95,6 +98,29 @@ typedef pthread_mutex_t pocl_lock_t;
     }                                                                         \
   while (0)
 
+
+/* If available, use an Adaptive mutex for locking in the pthread driver,
+   otherwise fallback to simple mutexes */
+#define POCL_FAST_LOCK_T pthread_mutex_t
+#define POCL_FAST_LOCK(l) POCL_LOCK(l)
+#define POCL_FAST_UNLOCK(l) POCL_UNLOCK(l)
+#ifdef PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
+  #define POCL_FAST_INIT(l) \
+    do { \
+      pthread_mutexattr_t attrs; \
+      pthread_mutexattr_init (&attrs); \
+      int r = pthread_mutexattr_settype (&attrs, PTHREAD_MUTEX_ADAPTIVE_NP); \
+      assert (r == 0); \
+      pthread_mutex_init(&l, &attrs); \
+      pthread_mutexattr_destroy(&attrs);\
+    } while (0)
+#else
+  #define POCL_FAST_INIT(l) pthread_mutex_init(&l, NULL);
+#endif
+#define POCL_FAST_DESTROY(l) POCL_DESTROY_LOCK(l)
+
+
+
 #define POCL_LOCK_OBJ(__OBJ__)                                                \
   do                                                                          \
     {                                                                         \
@@ -110,14 +136,11 @@ typedef pthread_mutex_t pocl_lock_t;
     }                                                                         \
   while (0)
 
-#define POCL_RELEASE_OBJECT_UNLOCKED(__OBJ__, __NEW_REFCOUNT__)               \
-  __NEW_REFCOUNT__ = --(__OBJ__)->pocl_refcount
-
 #define POCL_RELEASE_OBJECT(__OBJ__, __NEW_REFCOUNT__)                        \
   do                                                                          \
     {                                                                         \
       POCL_LOCK_OBJ (__OBJ__);                                                \
-      POCL_RELEASE_OBJECT_UNLOCKED (__OBJ__, __NEW_REFCOUNT__);               \
+      __NEW_REFCOUNT__ = --(__OBJ__)->pocl_refcount;                          \
       POCL_UNLOCK_OBJ (__OBJ__);                                              \
     }                                                                         \
   while (0)
@@ -170,7 +193,7 @@ typedef pthread_mutex_t pocl_lock_t;
     {                                                                         \
       POCL_DESTROY_LOCK ((__OBJ__)->pocl_lock);                               \
     }                                                                         \
-  while (0);
+  while (0)
 
 /* Declares the generic pocl object attributes inside a struct. */
 #define POCL_OBJECT \
@@ -183,9 +206,6 @@ typedef pthread_mutex_t pocl_lock_t;
 #ifdef __APPLE__
 /* Note: OSX doesn't support aliases because it doesn't use ELF */
 
-#  ifdef BUILD_ICD
-#    error "ICD not supported on OSX"
-#  endif
 #  define POname(name) name
 #  define POdeclsym(name)
 #  define POsym(name)
@@ -216,6 +236,14 @@ typedef pthread_mutex_t pocl_lock_t;
 
 #endif
 
+#ifdef __cplusplus
+extern "C"
+{
+  CL_API_ENTRY cl_int CL_API_CALL POname (clReleaseEvent) (cl_event event)
+      CL_API_SUFFIX__VERSION_1_0;
+}
+#endif
+
 /* The ICD compatibility part. This must be first in the objects where
  * it is used (as the ICD loader assumes that)*/
 #ifdef BUILD_ICD
@@ -241,7 +269,10 @@ typedef pthread_mutex_t pocl_lock_t;
 
 typedef struct pocl_argument {
   uint64_t size;
+  uint64_t offset;
   void *value;
+  /* 1 if this argument has been set by clSetKernelArg */
+  int is_set;
 } pocl_argument;
 
 
@@ -257,6 +288,9 @@ typedef enum {
   POCL_ARG_TYPE_SAMPLER = 3,
 } pocl_argument_type;
 
+#define ARG_IS_LOCAL(a) (a.address_qualifier == CL_KERNEL_ARG_ADDRESS_LOCAL)
+#define ARGP_IS_LOCAL(a) (a->address_qualifier == CL_KERNEL_ARG_ADDRESS_LOCAL)
+
 typedef struct pocl_argument_info {
   char* type_name;
   char* name;
@@ -264,8 +298,6 @@ typedef struct pocl_argument_info {
   cl_kernel_arg_access_qualifier access_qualifier;
   cl_kernel_arg_type_qualifier type_qualifier;
   pocl_argument_type type;
-  char is_local;
-  char is_set;
   unsigned type_size;
 } pocl_argument_info;
 
@@ -284,7 +316,8 @@ struct pocl_device_ops {
   void (*submit) (_cl_command_node *node, cl_command_queue cq);
 
   /* join is called by clFinish and this function blocks until all the enqueued
-     commands are finished. */
+     commands are finished. Called by the user thread; see notify_cmdq_finished
+     for the driver thread counterpart. */
   void (*join) (cl_device_id device, cl_command_queue cq);
 
   /* flush is called when clFlush is called. This function ensures that
@@ -307,24 +340,39 @@ struct pocl_device_ops {
      list of events waiting on 'event'. */
   void (*broadcast) (cl_event event);
 
-  /* wait_event is blocking the execution until the waited event is complete.
-     Called (and must return) with unlocked event. */
+  /* wait_event is called by clWaitForEvents() and blocks the execution until
+   * the waited event is complete or failed. Called by user threads; see the
+   * notify_event_finished() callback for driver thread counterpart.
+   * Called (and must return) with unlocked event. */
   void (*wait_event) (cl_device_id device, cl_event event);
 
-  /* update_event is an alternative way of handling event status changes if
-     something device specific needs to be done when the status of the event
-     changes.
-     This function is called via POCL_UPDATE_EVENT_* macros if available.
-     All POCL_UPDATE_EVENT_ (except COMPLETE) must be called with LOCKED event.
-     may be NULL, no need to implement if not needed.
-     Called (and must return) with locked event. */
-  void (*update_event) (cl_device_id device, cl_event event, cl_int status);
+  /* update_event is an extra callback called during handling of event status
+   * changes, useful if something device specific needs to be done. May be
+   * NULL; no need to implement if not needed.
+   *
+   * Called via pocl_update_event_* functions in pocl_util.c
+   * All pocl_update_event_* (except COMPLETE) must be called (and return)
+   * with LOCKED event.
+   */
+  void (*update_event) (cl_device_id device, cl_event event);
 
   /* free_event_data may be called when event is freed. Event data may only be
      used by the device driver owning the corresponding command.
      No need to implement this if the device does not need any event data. */
   void (*free_event_data) (cl_event event);
 
+  /* Called from driver threads to notify every user thread waiting on
+   * command queue finish. See join() for user counterpart.
+   * Driver may chose to not implement this, which will result in
+   * undefined behaviour in multi-threaded user programs. */
+  void (*notify_cmdq_finished) (cl_command_queue cq);
+
+  /* Called from driver threads to notify every user thread waiting on
+   * a specific event. See wait_event() for user counterpart.
+   * Driver may chose to not implement this, which will result in
+   * undefined behaviour in multi-threaded user programs. */
+  void (*notify_event_finished) (cl_event event);
+
   /* /New driver api extension */
 
   /* Detects & returns the number of available devices the driver finds on the system. */
@@ -336,10 +384,10 @@ struct pocl_device_ops {
    */
   cl_int (*init) (unsigned j, cl_device_id device, const char *parameters);
   /* uninitializes the driver for a particular device. May free hardware resources. */
-  cl_int (*uninit) (cl_device_id device);
+  cl_int (*uninit) (unsigned j, cl_device_id device);
   /* reinitializes the driver for a particular device. Called after uninit;
    * the first initialization is done by 'init'. May be NULL */
-  cl_int (*reinit) (cl_device_id device);
+  cl_int (*reinit) (unsigned j, cl_device_id device);
 
   /* if the driver needs to use hardware resources for command queues, use this */
   cl_int (*init_queue) (cl_command_queue queue);
@@ -347,7 +395,6 @@ struct pocl_device_ops {
 
   /* allocate a buffer in device memory */
   cl_int (*alloc_mem_obj) (cl_device_id device, cl_mem mem_obj, void* host_ptr);
-  void *(*create_sub_buffer) (void *data, void* buffer, size_t origin, size_t size);
   /* free a device buffer */
   void (*free) (cl_device_id device, cl_mem mem_obj);
 
@@ -449,15 +496,19 @@ struct pocl_device_ops {
                        cl_mem dst_buf,
                        mem_mapping_t *map);
 
-  /* compile the fully linked LLVM IR to target-specific binaries. */
-  void (*compile_kernel) (_cl_command_node* cmd, cl_kernel kernel, cl_device_id device);
+  /* Compile the fully linked LLVM IR to target-specific binaries.
+     If specialize is set to 1, allow specializing the final result
+     according to the properites of the given cmd. Typically
+     specialization is _not_ wanted only when creating a generic
+     WG function for storing in a binary. Local size of all zeros
+     forces dynamic local size work-group even if specializing
+     according to other properties. */
+  void (*compile_kernel) (_cl_command_node *cmd, cl_kernel kernel,
+                          cl_device_id device, int specialize);
   /* clEnqueueNDRangeKernel */
-  void (*run) (void *data, _cl_command_node* cmd);
+  void (*run) (void *data, _cl_command_node *cmd);
   /* for clEnqueueNativeKernel. may be NULL */
-  void (*run_native) (void *data, _cl_command_node* cmd);
-
-  /* The current device timer value in nanoseconds. */
-  cl_ulong (*get_timer_value) (void *data);
+  void (*run_native) (void *data, _cl_command_node *cmd);
 
   /* Perform initialization steps and can return additional
      build options that are required for the device. The caller
@@ -475,11 +526,6 @@ struct pocl_device_ops {
    * IMAGE1D_BUFFER type (which is implemented as a buffer).
    * If the device does not support images, all of these may be NULL. */
 
-  /* return supported image formats */
-  cl_int (*get_supported_image_formats) (cl_mem_flags flags,
-                                         const cl_image_format **image_formats,
-                                         cl_uint *num_image_formats);
-
   /* returns a device specific pointer which may reference
    * a hardware resource. May be NULL */
   void* (*create_image) (void *data,
@@ -511,7 +557,7 @@ struct pocl_device_ops {
                              const size_t *region);
 
   /* copies a region from host OR device buffer to device image.
-   * clEnqueueCopyImageToBuffer: src_mem_id = buffer,
+   * clEnqueueCopyBufferToImage: src_mem_id = buffer,
    *     src_host_ptr = NULL, src_row_pitch = src_slice_pitch = 0
    * clEnqueueWriteImage: src_mem_id = NULL,
    *     src_host_ptr = host pointer, src_offset = 0
@@ -528,7 +574,7 @@ struct pocl_device_ops {
                                size_t src_offset);
 
   /* copies a region from device image to host or device buffer
-   * clEnqueueCopyBufferToImage: dst_mem_id = buffer,
+   * clEnqueueCopyImageToBuffer: dst_mem_id = buffer,
    *     dst_host_ptr = NULL, dst_row_pitch = dst_slice_pitch = 0
    * clEnqueueReadImage: dst_mem_id = NULL,
    *     dst_host_ptr = host pointer, dst_offset = 0
@@ -565,15 +611,27 @@ struct pocl_device_ops {
                        const void *__restrict__ fill_pixel,
                        size_t pixel_size);
 
+
+  /* custom device functionality */
+
+  /* Check if the device supports the builtin kernel with the given name. */
+  cl_int (*supports_builtin_kernel) (void *data, const char *kernel_name);
+
+  /* Loads the metadata for the kernel with the given name to the MD object
+     allocated at the given target. */
+  cl_int (*get_builtin_kernel_metadata) (void *data, const char *kernel_name,
+                                         pocl_kernel_metadata_t *target);
 };
 
 typedef struct pocl_global_mem_t {
   pocl_lock_t pocl_lock;
-  size_t max_ever_allocated;
-  size_t currently_allocated;
-  size_t total_alloc_limit;
+  cl_ulong max_ever_allocated;
+  cl_ulong currently_allocated;
+  cl_ulong total_alloc_limit;
 } pocl_global_mem_t;
 
+#define NUM_OPENCL_IMAGE_TYPES 6
+
 struct _cl_device_id {
   POCL_ICD_OBJECT
   POCL_OBJECT;
@@ -581,10 +639,12 @@ struct _cl_device_id {
   cl_device_type type;
   cl_uint vendor_id;
   cl_uint max_compute_units;
-  // for subdevices
+
+  /* for subdevice support */
   cl_device_id parent_device;
   unsigned core_start;
   unsigned core_count;
+
   cl_uint max_work_item_dimensions;
   /* when enabled, Workgroup LLVM pass will replace all printf() calls
    * with calls to __pocl_printf and recursively change functions to
@@ -648,8 +708,11 @@ struct _cl_device_id {
   cl_bool compiler_available;
   /* Is the target a Single Program Multiple Data machine? If not,
      we need to generate work-item loops to execute all the work-items
-     in the WG, otherwise the hardware spawns the WIs. */
+     in the WG. For SPMD machines, the hardware spawns the WIs. */
   cl_bool spmd;
+  /* The device uses an HSA-like kernel ABI with a single argument buffer as
+     an input. */
+  cl_bool arg_buffer_launcher;
   /* The Workgroup pass creates launcher functions and replaces work-item
      placeholder global variables (e.g. _local_size_, _global_offset_ etc) with
      loads from the context struct passed as a kernel argument. This flag
@@ -686,14 +749,51 @@ struct _cl_device_id {
   /* pointer to an accounting struct for global memory */
   pocl_global_mem_t *global_memory;
   int has_64bit_long;  /* Does the device have 64bit longs */
+  /* Does the device set the event times in update_event() callback ?
+   * if zero, the default event change handlers set the event times based on
+   * the host's system time (pocl_gettimemono_ns). */
+  int has_own_timer;
   /* Convert automatic local variables to kernel arguments? */
   int autolocals_to_args;
+  /* Allocate local buffers device side in the work-group launcher instead of
+     having a disjoint physical local memory per work-group or having the
+     runtime/driver allocate the local space. */
+  int device_alloca_locals;
+
+  /* If > 0, specialized versions of the work-group functions are generated
+     which assume each grid dimension is of at most the given width. This
+     assumption can be then taken in account in IR optimization and codegen
+     to reduce address computation overheads etc. */
+  size_t grid_width_specialization_limit;
+
+  /* Device-specific linker flags that should be appended to the clang's
+     argument list for a final linkage call when producing the final binary
+     that can be uploaded to the device using the default LLVM-based
+     codegen. The final entry in the list must be NULL.
+
+     The flags will be added after the following command line:
+     clang -o final.bin input.obj [flags]
+  */
+
+  const char **final_linkage_flags;
+
+  /* Auxiliary functions required by the device binary which should
+     be retained across the kernel compilation unused code pruning
+     process. */
+  const char **device_aux_functions;
 
   /* The target specific IDs for the different OpenCL address spaces. */
   unsigned global_as_id;
   unsigned local_as_id;
   unsigned constant_as_id;
 
+  /* The address space where the argument data is passed. */
+  unsigned args_as_id;
+
+  /* The address space where the grid context data is passed. */
+  unsigned context_as_id;
+
+
   /* True if the device supports SVM. Then it has the responsibility of
      allocating shared buffers residing in Shared Virtual Memory areas. */
   cl_bool should_allocate_svm;
@@ -708,9 +808,15 @@ struct _cl_device_id {
   cl_uint dev_queue_max_size;
   cl_command_queue_properties on_dev_queue_props;
   cl_command_queue_properties on_host_queue_props;
+  /* OpenCL 2.1 */
+  char *spirv_version;
 
-  struct pocl_device_ops *ops; /* Device operations, shared amongst same devices */
+  /* image formats supported by the device, per image type */
+  const cl_image_format *image_formats[NUM_OPENCL_IMAGE_TYPES];
+  cl_uint num_image_formats[NUM_OPENCL_IMAGE_TYPES];
 
+  /* Device operations, shared among devices of the same type */
+  struct pocl_device_ops *ops;
 };
 
 #define DEVICE_SVM_FINEGR(dev) (dev->svm_caps & (CL_DEVICE_SVM_FINE_GRAIN_BUFFER \
@@ -724,6 +830,7 @@ struct _cl_device_id {
 #define CHECK_DEVICE_AVAIL_RET(dev) if(!dev->available) { POCL_MSG_ERR("This cl_device is not available.\n"); return CL_INVALID_DEVICE; }
 #define CHECK_DEVICE_AVAIL_RETV(dev) if(!dev->available) { POCL_MSG_ERR("This cl_device is not available.\n"); return; }
 
+#define OPENCL_MAX_DIMENSION 3
 
 struct _cl_platform_id {
   POCL_ICD_OBJECT_PLATFORM_ID
@@ -744,18 +851,37 @@ struct _cl_context {
      Returns a valid = 0 context in that case.  */
   char valid;
 
-  /* The minimal value of max_mem_alloc_size of all devices in context */
-  size_t min_max_mem_alloc_size;
+  /*********************************************************************/
+  /* these values depend on which devices are in context;
+   * they're calculated by pocl_setup_context() */
+
+  /* The largest of max_mem_alloc_size of all devices in context */
+  size_t max_mem_alloc_size;
+
+  /* union of image formats supported by all of the devices in context,
+   * per image-type (there are 6 image types)
+     TODO the getSupportedImageFormats is supposed to also respect flags,
+     but for now we ignore that. */
+  cl_image_format *image_formats[NUM_OPENCL_IMAGE_TYPES];
+  cl_uint num_image_formats[NUM_OPENCL_IMAGE_TYPES];
+
   /* The device that should allocate SVM (might be == host)
    * NULL if none of devices in the context is SVM capable */
   cl_device_id svm_allocdev;
+
+  /* The minimal required buffer alignment for all devices in the context.
+   * E.g. for clCreateSubBuffer:
+   * CL_MISALIGNED_SUB_BUFFER_OFFSET is returned in errcode_ret if there are no
+   * devices in context associated with buffer for which the origin value
+   * is aligned to the CL_DEVICE_MEM_BASE_ADDR_ALIGN value.
+   */
+  size_t min_buffer_alignment;
 };
 
 typedef struct _pocl_data_sync_item pocl_data_sync_item;
 struct _pocl_data_sync_item {
-  unsigned int volatile event_id;
-  cl_event volatile event;
-  pocl_data_sync_item *volatile next;
+  cl_event event;
+  pocl_data_sync_item *next;
 };
 
 struct _cl_event;
@@ -776,6 +902,73 @@ struct _cl_command_queue {
   void *data;
 };
 
+#define POCL_ON_SUB_MISALIGN(mem, que, operation)                             \
+  do                                                                          \
+    {                                                                         \
+      if (mem->parent != NULL)  {                                             \
+        operation (                                                           \
+            (mem->origin % que->device->mem_base_addr_align != 0),            \
+            CL_MISALIGNED_SUB_BUFFER_OFFSET,                                  \
+            "SubBuffer is not "                                               \
+            "properly aligned for this device");                              \
+        }                                                                     \
+    }                                                                         \
+  while (0)
+
+#define POCL_RETURN_ON_SUB_MISALIGN(mem, que)                                 \
+  POCL_ON_SUB_MISALIGN(mem, que, POCL_RETURN_ERROR_ON)
+
+#define POCL_GOTO_ON_SUB_MISALIGN(mem, que)                                   \
+  POCL_ON_SUB_MISALIGN(mem, que, POCL_GOTO_ERROR_ON)
+
+#define POCL_CONVERT_SUBBUFFER_OFFSET(mem, offset)                            \
+  if (mem->parent != NULL)                                                    \
+    {                                                                         \
+      offset += mem->origin;                                                  \
+      mem = mem->parent;                                                      \
+    }
+
+#define DEVICE_IMAGE_SIZE_SUPPORT 1
+#define DEVICE_IMAGE_FORMAT_SUPPORT 2
+
+#define DEVICE_DOESNT_SUPPORT_IMAGE(mem, dev_i)                               \
+  (mem->device_supports_this_image[dev_i] == 0)
+
+#define POCL_ON_UNSUPPORTED_IMAGE(mem, dev, operation)                        \
+  do                                                                          \
+    {                                                                         \
+      unsigned dev_i;                                                         \
+      for (dev_i = 0; dev_i < mem->context->num_devices; ++dev_i)             \
+        if (mem->context->devices[dev_i] == dev)                              \
+          break;                                                              \
+      assert (dev_i < mem->context->num_devices);                             \
+      operation (                                                  \
+          (mem->context->devices[dev_i]->image_support == CL_FALSE),          \
+          CL_INVALID_OPERATION, "Device %s does not support images\n",        \
+          mem->context->devices[dev_i]->long_name);                           \
+      operation (                                                  \
+          ((mem->device_supports_this_image[dev_i]                            \
+            & DEVICE_IMAGE_FORMAT_SUPPORT)                                    \
+           == 0),                                                             \
+          CL_IMAGE_FORMAT_NOT_SUPPORTED,                                      \
+          "The image type is not supported by this device\n");                \
+      operation (                                                  \
+          ((mem->device_supports_this_image[dev_i]                            \
+            & DEVICE_IMAGE_SIZE_SUPPORT)                                      \
+           == 0),                                                             \
+          CL_INVALID_IMAGE_SIZE,                                              \
+          "The image size is not supported by this device\n");                \
+    }                                                                         \
+  while (0)
+
+
+#define POCL_RETURN_ON_UNSUPPORTED_IMAGE(mem, dev)                            \
+  POCL_ON_UNSUPPORTED_IMAGE(mem, dev, POCL_RETURN_ERROR_ON)
+
+#define POCL_GOTO_ON_UNSUPPORTED_IMAGE(mem, dev)                              \
+  POCL_ON_UNSUPPORTED_IMAGE(mem, dev, POCL_GOTO_ERROR_ON)
+
+
 
 typedef struct _cl_mem cl_mem_t;
 struct _cl_mem {
@@ -813,6 +1006,10 @@ struct _cl_mem {
   /* A linked list of destructor callbacks */
   mem_destructor_callback_t *destructor_callbacks;
 
+  /* for images, a flag for each device in context,
+   * whether that device supports this */
+  int *device_supports_this_image;
+
   /* Image flags */
   cl_bool                 is_image;
   cl_channel_order        image_channel_order;
@@ -837,16 +1034,34 @@ struct _cl_mem {
 
 typedef uint8_t SHA1_digest_t[SHA1_DIGEST_SIZE * 2 + 1];
 
+typedef struct pocl_kernel_metadata_s
+{
+  cl_uint num_args;
+  cl_uint num_locals;
+  size_t *local_sizes;
+  char *name;
+  char *attributes;
+  struct pocl_argument_info *arg_info;
+  cl_bitfield has_arg_metadata;
+  size_t reqd_wg_size[OPENCL_MAX_DIMENSION];
+
+  /* array[program->num_devices] */
+  pocl_kernel_hash_t *build_hash;
+
+  /* If this is a BI kernel descriptor, they are statically defined in
+     the custom device driver, thus should not be freed. */
+  cl_bitfield builtin_kernel;
+  /* device-specific data, void* array[program->num_devices] */
+  void **data;
+
+} pocl_kernel_metadata_t;
+
 struct _cl_program {
   POCL_ICD_OBJECT
   POCL_OBJECT;
   /* queries */
   cl_context context;
   cl_uint num_devices;
-  /* bool flag, set to 1 when removing/adding default kernels to a program
-   * This code needs to be eventually fixed by introducing kernel_metadata
-   * struct, see Issue #390 */
-  int operating_on_default_kernels;
   /* -cl-denorms-are-zero build option */
   unsigned flush_denorms;
   cl_device_id *devices;
@@ -859,14 +1074,18 @@ struct _cl_program {
   size_t *binary_sizes;
   unsigned char **binaries;
 
+  /* If this is a program with built-in kernels, this is the list of kernel
+     names it contains. */
+  size_t num_builtin_kernels;
+  char **builtin_kernel_names;
+
   /* Poclcc binary format.  */
   size_t *pocl_binary_sizes;
   unsigned char **pocl_binaries;
 
-  /* "Default" kernels. See: https://github.com/pocl/pocl/issues/390  */
+  /* kernel number and the metadata for each kernel */
   size_t num_kernels;
-  char **kernel_names;
-  cl_kernel *default_kernels;
+  pocl_kernel_metadata_t *kernel_meta;
 
   /* implementation */
   cl_kernel kernels;
@@ -882,28 +1101,26 @@ struct _cl_program {
   cl_build_status build_status;
   /* Use to store binary type */
   cl_program_binary_type binary_type;
+
+  /* Store SPIR-V binary from clCreateProgramWithIL() */
+  char *program_il;
+  size_t program_il_size;
 };
 
 struct _cl_kernel {
   POCL_ICD_OBJECT
   POCL_OBJECT;
-  /* queries */
-  char *name;
-  cl_uint num_args;
+  /* -------- */
   cl_context context;
   cl_program program;
-  struct pocl_argument_info *arg_info;
-  cl_bitfield has_arg_metadata;
-  char *attributes;
-  cl_uint num_locals;
-  size_t *reqd_wg_size;
+  pocl_kernel_metadata_t *meta;
+  /* just a convenience pointer to meta->name */
+  const char *name;
+
   /* The kernel arguments that are set with clSetKernelArg().
      These are copied to the command queue command at enqueue. */
   struct pocl_argument *dyn_arguments;
   struct _cl_kernel *next;
-
-  /* backend specific data */
-  void *data;
 };
 
 typedef struct event_callback_item event_callback_item;
@@ -941,7 +1158,7 @@ struct _cl_event {
 
   /* OoO doesn't use sync points -> put used buffers here */
   cl_mem *mem_objs;
-  int num_buffers;
+  size_t num_buffers;
 
   /* The execution status of the command this event is monitoring. */
   volatile cl_int status;
@@ -976,152 +1193,8 @@ struct _cl_sampler {
   void**              device_data;
 };
 
-#define POCL_UPDATE_EVENT_QUEUED(__event)                                     \
-  do                                                                          \
-    {                                                                         \
-      if ((__event) != NULL)                                                  \
-        {                                                                     \
-          cl_command_queue __cq = (__event)->queue;                           \
-          if ((__cq)->device->ops->update_event)                              \
-            (__cq)->device->ops->update_event ((__cq)->device, (__event),     \
-                                               CL_QUEUED);                    \
-          else                                                                \
-            {                                                                 \
-              (__event)->status = CL_QUEUED;                                  \
-              if (__cq && __cq->properties & CL_QUEUE_PROFILING_ENABLE)       \
-                (__event)->time_queue = __cq->device->ops->get_timer_value (  \
-                    __cq->device->data);                                      \
-            }                                                                 \
-          pocl_event_updated (__event, CL_QUEUED);                            \
-        }                                                                     \
-    }                                                                         \
-  while (0)
-
-#define POCL_UPDATE_EVENT_SUBMITTED(__event)                                  \
-  do                                                                          \
-    {                                                                         \
-      if ((__event) != NULL)                                                  \
-        {                                                                     \
-          assert ((__event)->status == CL_QUEUED);                            \
-          cl_command_queue __cq = (__event)->queue;                           \
-          if ((__cq)->device->ops->update_event)                              \
-            (__cq)->device->ops->update_event ((__cq)->device, (__event),     \
-                                               CL_SUBMITTED);                 \
-          else                                                                \
-            {                                                                 \
-              (__event)->status = CL_SUBMITTED;                               \
-              if (__cq && __cq->properties & CL_QUEUE_PROFILING_ENABLE)       \
-                (__event)->time_submit = __cq->device->ops->get_timer_value ( \
-                    __cq->device->data);                                      \
-            }                                                                 \
-          pocl_event_updated (__event, CL_SUBMITTED);                         \
-        }                                                                     \
-    }                                                                         \
-  while (0)
-
-#define POCL_UPDATE_EVENT_RUNNING_UNLOCKED(__event)                           \
-  do                                                                          \
-    {                                                                         \
-      if (__event != NULL)                                                    \
-        {                                                                     \
-          assert ((__event)->status == CL_SUBMITTED);                         \
-          cl_command_queue __cq = (__event)->queue;                           \
-          if ((__cq)->device->ops->update_event)                              \
-            (__cq)->device->ops->update_event ((__cq)->device, (__event),     \
-                                               CL_RUNNING);                   \
-          else                                                                \
-            {                                                                 \
-              (__event)->status = CL_RUNNING;                                 \
-              if (__cq && __cq->properties & CL_QUEUE_PROFILING_ENABLE)       \
-                (__event)->time_start = __cq->device->ops->get_timer_value (  \
-                    __cq->device->data);                                      \
-            }                                                                 \
-          pocl_event_updated (__event, CL_RUNNING);                           \
-        }                                                                     \
-    }                                                                         \
-  while (0)
-
-#define POCL_UPDATE_EVENT_RUNNING(__event)                                    \
-  POCL_LOCK_OBJ (__event);                                                    \
-  POCL_UPDATE_EVENT_RUNNING_UNLOCKED (__event);                               \
-  POCL_UNLOCK_OBJ (__event);
-
-#define POCL_UPDATE_EVENT_COMPLETE_INNER(__event, POST_EVENT)                 \
-  do                                                                          \
-    {                                                                         \
-      if ((__event) != NULL)                                                  \
-        {                                                                     \
-          assert ((__event)->status == CL_RUNNING);                           \
-          POCL_LOCK_OBJ (__event);                                            \
-          cl_command_queue __cq = (__event)->queue;                           \
-          if ((__cq)->device->ops->update_event)                              \
-            (__cq)->device->ops->update_event ((__cq)->device, (__event),     \
-                                               CL_COMPLETE);                  \
-          else                                                                \
-            {                                                                 \
-              pocl_mem_objs_cleanup (__event);                                \
-              (__event)->status = CL_COMPLETE;                                \
-              if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
-                {                                                             \
-                  (__event)->time_end                                         \
-                      = (__cq)->device->ops->get_timer_value (                \
-                          (__cq)->device->data);                              \
-                }                                                             \
-              POCL_UNLOCK_OBJ (__event);                                      \
-              pocl_update_command_queue (__event);                            \
-              (__cq)->device->ops->broadcast (__event);                       \
-              POCL_LOCK_OBJ (__event);                                        \
-            }                                                                 \
-          pocl_event_updated (__event, CL_COMPLETE);                          \
-          POST_EVENT;                                                         \
-          POCL_UNLOCK_OBJ (__event);                                          \
-          POname (clReleaseEvent) (__event);                                  \
-        }                                                                     \
-    }                                                                         \
-  while (0)
-
-#define POCL_UPDATE_EVENT_COMPLETE(__event)                                   \
-  POCL_UPDATE_EVENT_COMPLETE_INNER (__event, NULL)
-
-#define POCL_UPDATE_EVENT_COMPLETE_MSG(__event, msg)                          \
-  POCL_UPDATE_EVENT_COMPLETE_INNER (__event,                                  \
-                                    POCL_DEBUG_EVENT_TIME ((__event), msg))
-
 #define CL_FAILED (-1)
 
-#define POCL_UPDATE_EVENT_FAILED(__event)                                     \
-  do                                                                          \
-    {                                                                         \
-      if ((__event) != NULL)                                                  \
-        {                                                                     \
-          cl_command_queue __cq = (__event)->queue;                           \
-          if ((__cq)->device->ops->update_event)                              \
-            (__cq)->device->ops->update_event ((__cq)->device, (__event),     \
-                                               CL_FAILED);                    \
-          else                                                                \
-            {                                                                 \
-              pocl_mem_objs_cleanup (__event);                                \
-              if ((__event)->status > CL_COMPLETE)                            \
-                (__event)->status = CL_FAILED;                                \
-              if ((__cq)->properties & CL_QUEUE_PROFILING_ENABLE)             \
-                {                                                             \
-                  (__event)->time_end                                         \
-                      = (__cq)->device->ops->get_timer_value (                \
-                          (__cq)->device->data);                              \
-                }                                                             \
-              POCL_UNLOCK_OBJ (__event);                                      \
-              pocl_update_command_queue (__event);                            \
-              (__cq)->device->ops->broadcast (__event);                       \
-              POCL_LOCK_OBJ (__event);                                        \
-            }                                                                 \
-          pocl_event_updated (__event, CL_FAILED);                            \
-          POCL_UNLOCK_OBJ (__event);                                          \
-          POname (clReleaseEvent) (__event);                                  \
-          POCL_LOCK_OBJ (__event);                                            \
-        }                                                                     \
-    }                                                                         \
-  while (0)
-
 #ifndef __cplusplus
 
 #define min(a,b) (((a) < (b)) ? (a) : (b))
@@ -1141,6 +1214,28 @@ struct _cl_sampler {
   #include <sys/endian.h>
 #else
   #include <endian.h>
+  #if defined(__GLIBC__) && __GLIBC__ == 2 && \
+      defined(__GLIBC_MINOR__) && __GLIBC_MINOR__ < 9 && \
+      defined(__x86_64__)
+    #ifndef htole64
+      #define htole64(x) (x)
+    #endif
+    #ifndef htole32
+      #define htole32(x) (x)
+    #endif
+    #ifndef htole16
+      #define htole16(x) (x)
+    #endif
+    #ifndef le64toh
+      #define le64toh(x) (x)
+    #endif
+    #ifndef le32toh
+      #define le32toh(x) (x)
+    #endif
+    #ifndef le16toh
+      #define le16toh(x) (x)
+    #endif
+  #endif
 #endif
 
 #endif /* POCL_CL_H */
