@@ -2,7 +2,7 @@
    producing program.bc
 
    Copyright (c) 2013 Kalle Raiskila
-                 2013-2017 Pekka Jääskeläinen
+                 2013-2019 Pekka Jääskeläinen
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
@@ -29,15 +29,16 @@ IGNORE_COMPILER_WARNING("-Wstrict-aliasing")
 
 #include "config.h"
 
-#include "clang/CodeGen/CodeGenAction.h"
-#include "clang/Frontend/CompilerInstance.h"
-#include "clang/Frontend/CompilerInvocation.h"
-#include "clang/Frontend/FrontendActions.h"
-#include "clang/Frontend/TextDiagnosticBuffer.h"
-
-#ifndef LLVM_OLDER_THAN_4_0
+#include <clang/Basic/Diagnostic.h>
+#include <clang/Driver/Compilation.h>
+#include <clang/Driver/Driver.h>
+#include <clang/CodeGen/CodeGenAction.h>
+#include <clang/Frontend/CompilerInstance.h>
+#include <clang/Frontend/CompilerInvocation.h>
+#include <clang/Frontend/FrontendActions.h>
+#include <clang/Frontend/TextDiagnosticBuffer.h>
+#include <clang/Frontend/TextDiagnosticPrinter.h>
 #include "clang/Lex/PreprocessorOptions.h"
-#endif
 
 #include "llvm/LinkAllPasses.h"
 #include "llvm/Linker/Linker.h"
@@ -49,6 +50,15 @@ IGNORE_COMPILER_WARNING("-Wstrict-aliasing")
 
 #include "llvm/Support/MutexGuard.h"
 
+#ifdef ENABLE_RELOCATION
+
+#if defined(__APPLE__)
+#define _DARWIN_C_SOURCE
+#endif
+#include <dlfcn.h>
+
+#endif
+
 #include <iostream>
 #include <sstream>
 
@@ -95,11 +105,7 @@ load_source(FrontendOptions &fe,
                        CL_OUT_OF_HOST_MEMORY, "Could not write program source");
 
   fe.Inputs.push_back
-#if LLVM_OLDER_THAN_5_0
-      (FrontendInputFile(source_file, clang::IK_OpenCL));
-#else
       (FrontendInputFile(source_file, clang::InputKind::OpenCL));
-#endif
 
   return 0;
 }
@@ -123,7 +129,7 @@ unlink_source(FrontendOptions &fe)
 static void appendToProgramBuildLog(cl_program program, unsigned device_i,
                                     std::string &s) {
   if (!s.empty()) {
-    POCL_MSG_ERR(s.c_str());
+    POCL_MSG_ERR("%s", s.c_str());
     /* this may not actually write anything if the buildhash is invalid,
      * but program->build_log still gets written.  */
     pocl_cache_append_to_buildlog(program, device_i, s.c_str(), s.size());
@@ -145,26 +151,45 @@ static void get_build_log(cl_program program,
                          unsigned device_i,
                          std::stringstream &ss_build_log,
                          clang::TextDiagnosticBuffer *diagsBuffer,
-                         const SourceManager &sm)
+                         const SourceManager *SM)
 {
-    for (TextDiagnosticBuffer::const_iterator i = diagsBuffer->err_begin(),
+  for (TextDiagnosticBuffer::const_iterator i = diagsBuffer->err_begin(),
          e = diagsBuffer->err_end(); i != e; ++i)
-      {
-        ss_build_log << "error: " << i->first.printToString(sm)
-                     << ": " << i->second << std::endl;
-      }
-    for (TextDiagnosticBuffer::const_iterator i = diagsBuffer->warn_begin(),
+  {
+    ss_build_log << "error: "
+                 << (SM == nullptr ? "" : (i->first.printToString(*SM) + ": "))
+                 << i->second << std::endl;
+  }
+  for (TextDiagnosticBuffer::const_iterator i = diagsBuffer->warn_begin(),
          e = diagsBuffer->warn_end(); i != e; ++i)
-      {
-        ss_build_log << "warning: " << i->first.printToString(sm)
-                     << ": " << i->second << std::endl;
-      }
+  {
+    ss_build_log << "warning: "
+                 << (SM == nullptr ? "" : (i->first.printToString(*SM) + ": "))
+                 << i->second << std::endl;
+  }
 
-    std::string log = ss_build_log.str();
-    appendToProgramBuildLog(program, device_i, log);
+  std::string log = ss_build_log.str();
+  appendToProgramBuildLog(program, device_i, log);
 }
 
-static llvm::Module *kernel_library(cl_device_id device);
+static llvm::Module *getKernelLibrary(cl_device_id device);
+
+static std::string getPoclPrivateDataDir() {
+#ifdef ENABLE_RELOCATION
+    Dl_info info;
+    if (dladdr((void*)getPoclPrivateDataDir, &info)) {
+        char const * soname = info.dli_fname;
+        std::string result = std::string(soname);
+        size_t last_slash = result.rfind('/');
+        result = result.substr(0, last_slash+1);
+        if (result.size() > 0) {
+            result += POCL_INSTALL_PRIVATE_DATADIR_REL;
+            return result;
+        }
+    }
+#endif
+    return POCL_INSTALL_PRIVATE_DATADIR;
+}
 
 int pocl_llvm_build_program(cl_program program,
                             unsigned device_i,
@@ -256,19 +281,15 @@ int pocl_llvm_build_program(cl_program program,
       llvm::StringRef tok = extensions.slice(e_start, e_end);
       e_start = e_end + 1;
       ss << "-D" << tok.str() << " ";
-#ifndef LLVM_OLDER_THAN_4_0
       cl_ext += "+";
       cl_ext += tok.str();
       cl_ext += ",";
-#endif
     }
   }
-#ifndef LLVM_OLDER_THAN_4_0
   if (!cl_ext.empty()) {
     cl_ext.back() = ' '; // replace last "," with space
     ss << "-cl-ext=-all," << cl_ext;
   }
-#endif
   /* temp dir takes preference */
   if (num_input_headers > 0)
     ss << "-I" << temp_include_dir << " ";
@@ -276,6 +297,9 @@ int pocl_llvm_build_program(cl_program program,
   if (device->has_64bit_long)
     ss << "-Dcl_khr_int64 ";
 
+  ss << "-DPOCL_DEVICE_ADDRESS_BITS=" << device->address_bits << " ";
+  ss << "-D__USE_CLANG_OPENCL_C_H ";
+
   ss << "-xcl ";
   // Remove the inline keywords to force the user functions
   // to be included in the program. Otherwise they will
@@ -343,9 +367,9 @@ int pocl_llvm_build_program(cl_program program,
   }
 
   for (unsigned idx = 0; idx < itemstrs.size(); idx++) {
-      // note: if itemstrs is modified after this, itemcstrs will be full
-      // of invalid pointers! Could make copies, but would have to clean up then...
-      itemcstrs.push_back(itemstrs[idx].c_str());
+    // note: if itemstrs is modified after this, itemcstrs will be full
+    // of invalid pointers! Could make copies, but would have to clean up then...
+    itemcstrs.push_back(itemstrs[idx].c_str());
   }
 
 #ifdef DEBUG_POCL_LLVM_API
@@ -363,25 +387,17 @@ int pocl_llvm_build_program(cl_program program,
        diags)) {
     pocl_cache_create_program_cachedir(program, device_i, NULL, 0,
                                        program_bc_path);
-    get_build_log(program, device_i, ss_build_log, diagsBuffer, CI.getSourceManager());
+    get_build_log(program, device_i, ss_build_log, diagsBuffer,
+                  CI.hasSourceManager() ? &CI.getSourceManager() : nullptr);
     return CL_INVALID_BUILD_OPTIONS;
   }
 
   LangOptions *la = pocl_build.getLangOpts();
   PreprocessorOptions &po = pocl_build.getPreprocessorOpts();
 
-#ifdef LLVM_OLDER_THAN_3_9
   pocl_build.setLangDefaults
-    (*la, clang::IK_OpenCL, clang::LangStandard::lang_opencl12);
-#else
-  pocl_build.setLangDefaults
-#if LLVM_OLDER_THAN_5_0
-      (*la, clang::IK_OpenCL, triple, po, clang::LangStandard::lang_opencl12);
-#else
       (*la, clang::InputKind::OpenCL, triple, po,
        clang::LangStandard::lang_opencl12);
-#endif
-#endif
 
   // LLVM 3.3 and older do not set that char is signed which is
   // defined by the OpenCL C specs (but not by C specs).
@@ -389,11 +405,7 @@ int pocl_llvm_build_program(cl_program program,
 
   // the per-file types don't seem to override this
   la->OpenCLVersion = cl_std_i;
-#ifdef POCL_USE_FAKE_ADDR_SPACE_IDS
-  la->FakeAddressSpaceMap = true;
-#else
   la->FakeAddressSpaceMap = false;
-#endif
   la->Blocks = true; //-fblocks
   la->MathErrno = false; // -fno-math-errno
   la->NoBuiltin = true;  // -fno-builtin
@@ -401,36 +413,48 @@ int pocl_llvm_build_program(cl_program program,
 
   la->setStackProtector(LangOptions::StackProtectorMode::SSPOff);
 
-#ifdef LLVM_OLDER_THAN_3_9
-  la->PICLevel = PICLevel::Large;
-  la->PIELevel = PICLevel::Large;
-#else
   la->PICLevel = PICLevel::BigPIC;
-  la->PIE = 0;
-#endif
+  la->PIE = 1;
 
-  std::string kernelh;
+  std::string IncludeRoot;
+  std::string KernelH;
   std::string BuiltinRenamesH;
+  std::string PoclTypesH;
+  std::string ClangResourceDir;
 
 #ifdef ENABLE_POCL_BUILDING
   if (pocl_get_bool_option("POCL_BUILDING", 0)) {
-    kernelh  = SRCDIR;
+    IncludeRoot = SRCDIR;
 #else
   if (0) {
 #endif
   } else {
-    kernelh = POCL_INSTALL_PRIVATE_DATADIR;
-  }
-  BuiltinRenamesH = kernelh;
-  kernelh += "/include/_kernel.h";
-  BuiltinRenamesH += "/include/_builtin_renames.h";
-
-  po.Includes.push_back(BuiltinRenamesH);
-#ifndef LLVM_OLDER_THAN_4_0
-  // Use Clang's opencl-c.h header.
-  po.Includes.push_back(CLANG_RESOURCE_DIR "/include/opencl-c.h");
+    IncludeRoot = getPoclPrivateDataDir();
+#ifdef ENABLE_RELOCATION
+    ClangResourceDir = IncludeRoot;
 #endif
-  po.Includes.push_back(kernelh);
+  }
+  if (ClangResourceDir.empty()) {     
+#ifndef LLVM_OLDER_THAN_9_0
+    ClangResourceDir = driver::Driver::GetResourcesPath(CLANG);
+#else
+    DiagnosticsEngine Diags{new DiagnosticIDs, new DiagnosticOptions};
+    driver::Driver TheDriver(CLANG, "", Diags);
+    ClangResourceDir = TheDriver.ResourceDir;
+#endif
+  }
+  KernelH = IncludeRoot + "/include/_kernel.h";
+  BuiltinRenamesH = IncludeRoot + "/include/_builtin_renames.h";
+  PoclTypesH = IncludeRoot + "/include/pocl_types.h";
+
+  po.Includes.push_back(PoclTypesH);
+  po.Includes.push_back(BuiltinRenamesH);
+  // Use Clang's opencl-c.h header.
+#ifndef LLVM_OLDER_THAN_9_0
+  po.Includes.push_back(ClangResourceDir + "/include/opencl-c-base.h");
+#endif
+  po.Includes.push_back(ClangResourceDir + "/include/opencl-c.h");
+  po.Includes.push_back(KernelH);
   clang::TargetOptions &ta = pocl_build.getTargetOpts();
   ta.Triple = device->llvm_target_triplet;
   if (device->llvm_cpu != NULL)
@@ -456,14 +480,7 @@ int pocl_llvm_build_program(cl_program program,
   // Lets leave vectorization to later compilation phase
   cg.VectorizeLoop = false;
   cg.VectorizeSLP = false;
-  // This workarounds a Frontend codegen issues with an illegal address
-  // space cast which is later flattened (and thus implicitly fixed) in
-  // the TargetAddressSpaces. See:  https://github.com/pocl/pocl/issues/195
-#ifdef POCL_USE_FAKE_ADDR_SPACE_IDS
-  cg.VerifyModule = false;
-#else
   cg.VerifyModule = true;
-#endif
 
   PreprocessorOutputOptions &poo = pocl_build.getPreprocessorOutputOpts();
   poo.ShowCPP = 1;
@@ -496,7 +513,8 @@ int pocl_llvm_build_program(cl_program program,
   if (PreprocessedOut == nullptr) {
     pocl_cache_create_program_cachedir(program, device_i, NULL, 0,
                                        program_bc_path);
-    get_build_log(program, device_i, ss_build_log, diagsBuffer, CI.getSourceManager());
+    get_build_log(program, device_i, ss_build_log, diagsBuffer,
+                  CI.hasSourceManager() ? &CI.getSourceManager() : nullptr);
     return CL_BUILD_PROGRAM_FAILURE;
   }
 
@@ -517,7 +535,7 @@ int pocl_llvm_build_program(cl_program program,
   clang::EmitLLVMOnlyAction EmitLLVM(&c);
   success = CI.ExecuteAction(EmitLLVM);
 
-  get_build_log(program, device_i, ss_build_log, diagsBuffer, CI.getSourceManager());
+  get_build_log(program, device_i, ss_build_log, diagsBuffer, &CI.getSourceManager());
 
   if (!success)
     return CL_BUILD_PROGRAM_FAILURE;
@@ -535,20 +553,17 @@ int pocl_llvm_build_program(cl_program program,
 
   ++numberOfIRs;
 
-#ifdef LLVM_OLDER_THAN_3_9
-  (*mod)->setPICLevel(PICLevel::Large);
-#else
-  (*mod)->setPICLevel(PICLevel::BigPIC);
-  (*mod)->setPIELevel(PIELevel::Large);
-#endif
-
+  if ((*mod)->getModuleFlag("PIC Level") == nullptr)
+    (*mod)->setPICLevel(PICLevel::BigPIC);
+  if ((*mod)->getModuleFlag("PIE Level") == nullptr)
+    (*mod)->setPIELevel(PIELevel::Large);
 
   // link w kernel lib, but not if we're called from clCompileProgram()
   // Later this should be replaced with indexed linking of source code
   // and/or bitcode for each kernel.
   if (linking_program) {
     currentPoclDevice = device;
-    llvm::Module *libmodule = kernel_library(device);
+    llvm::Module *libmodule = getKernelLibrary(device);
     assert(libmodule != NULL);
     std::string log("Error(s) while linking: \n");
     if (link(*mod, libmodule, log)) {
@@ -600,7 +615,7 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
   int error;
 
   currentPoclDevice = device;
-  llvm::Module *libmodule = kernel_library(device);
+  llvm::Module *libmodule = getKernelLibrary(device);
   assert(libmodule != NULL);
 
   PoclCompilerMutexGuard lockHolder(NULL);
@@ -633,12 +648,10 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
      */
     linked_module->setTargetTriple(libmodule->getTargetTriple());
     linked_module->setDataLayout(libmodule->getDataLayout());
-#ifdef LLVM_OLDER_THAN_3_9
-    linked_module->setPICLevel(PICLevel::Large);
-#else
-    linked_module->setPICLevel(PICLevel::BigPIC);
-    linked_module->setPIELevel(PIELevel::Large);
-#endif
+    if (linked_module->getModuleFlag("PIC Level") == nullptr)
+      linked_module->setPICLevel(PICLevel::BigPIC);
+    if (linked_module->getModuleFlag("PIE Level") == nullptr)
+      linked_module->setPIELevel(PIELevel::Large);
 
 #else
     POCL_MSG_ERR("SPIR not supported\n");
@@ -646,13 +659,8 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
 #endif
   } else {
 
-#ifdef LLVM_OLDER_THAN_3_8
-  llvm::Module *mod =
-      new llvm::Module(StringRef("linked_program"), GlobalContext());
-#else
   std::unique_ptr<llvm::Module> mod(
       new llvm::Module(StringRef("linked_program"), GlobalContext()));
-#endif
 
   for (i = 0; i < num_input_programs; i++) {
     assert(cur_device_binaries[i]);
@@ -663,10 +671,7 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
     llvm::Module *p = (llvm::Module *)cur_llvm_irs[i];
     assert(p);
 
-#ifdef LLVM_OLDER_THAN_3_8
-    if (Linker::LinkModules(mod, llvm::CloneModule(p))) {
-      delete mod;
-#elif LLVM_OLDER_THAN_7_0
+#ifdef LLVM_OLDER_THAN_7_0
     if (Linker::linkModules(*mod, llvm::CloneModule(p))) {
 #else
     if (Linker::linkModules(*mod, llvm::CloneModule(*p))) {
@@ -677,11 +682,7 @@ int pocl_llvm_link_program(cl_program program, unsigned device_i,
     }
   }
 
-#ifdef LLVM_OLDER_THAN_3_8
-  linked_module = mod;
-#else
   linked_module = mod.release();
-#endif
   }
 
   if (linked_module == nullptr)
@@ -785,9 +786,7 @@ static std::map<cl_device_id, llvm::Module *> kernelLibraryMap;
  * Return the OpenCL C built-in function library bitcode
  * for the given device.
  */
-static llvm::Module*
-kernel_library
-(cl_device_id device)
+static llvm::Module* getKernelLibrary(cl_device_id device)
 {
   Triple triple(device->llvm_target_triplet);
 
@@ -796,6 +795,8 @@ kernel_library
 
   const char *subdir = "host";
   bool is_host = true;
+  // TODO: move this to the device layer, a property to ask for
+  // the kernel builtin bitcode library name, including its subdir
 #ifdef TCE_AVAILABLE
   if (triple.getArch() == Triple::tce || triple.getArch() == Triple::tcele) {
     subdir = "tce";
@@ -822,7 +823,6 @@ kernel_library
   }
 #endif
 
-  // TODO sync with Nat Ferrus' indexed linking
   std::string kernellib;
   std::string kernellib_fallback;
 #ifdef ENABLE_POCL_BUILDING
@@ -832,21 +832,19 @@ kernel_library
     kernellib += subdir;
   } else // POCL_BUILDING == 0, use install dir
 #endif
-  kernellib = POCL_INSTALL_PRIVATE_DATADIR;
+  kernellib = getPoclPrivateDataDir();
   kernellib += "/kernel-";
   kernellib += device->llvm_target_triplet;
   if (is_host) {
     kernellib += '-';
+#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
+    kernellib += getX86KernelLibName();
+#else
     kernellib_fallback = kernellib;
     kernellib_fallback += OCL_KERNEL_TARGET_CPU;
     kernellib_fallback += ".bc";
-#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
-    if (triple.getArch() == Triple::x86_64 ||
-        triple.getArch() == Triple::x86)
-      kernellib += getX86KernelLibName();
-    else
+    kernellib += device->llvm_cpu;
 #endif
-      kernellib += device->llvm_cpu;
   }
   kernellib += ".bc";
 
@@ -859,6 +857,7 @@ kernel_library
     }
   else
     {
+#ifndef KERNELLIB_HOST_DISTRO_VARIANTS
       if (is_host && pocl_exists(kernellib_fallback.c_str()))
         {
           POCL_MSG_WARN("Using fallback %s as the built-in lib.\n",
@@ -866,6 +865,7 @@ kernel_library
           lib = parseModuleIR(kernellib_fallback.c_str());
         }
       else
+#endif
         POCL_ABORT("Kernel library file %s doesn't exist.\n", kernellib.c_str());
     }
   assert (lib != NULL);
@@ -881,3 +881,45 @@ void cleanKernelLibrary() {
   }
   kernelLibraryMap.clear();
 }
+
+/**
+ * Invoke the Clang compiler through its Driver API.
+ *
+ * @param Device the device of which toolchain to use.
+ * @param Args the command line arguments that would be passed to Clang
+ *             (a NULL terminated list). Args[0] should be the path to
+ *             the Clang binary.
+ * @return 0 on success, error code otherwise.
+ */
+int pocl_invoke_clang(cl_device_id Device, const char** Args) {
+
+  // Borrowed from driver.cpp (clang driver). We do not really care about
+  // diagnostics, but just want to get the compilation command invoked with
+  // the target's toolchain as defined in Clang.
+  IntrusiveRefCntPtr<DiagnosticOptions> DiagOpts = new DiagnosticOptions;
+
+  TextDiagnosticPrinter *DiagClient =
+    new TextDiagnosticPrinter(llvm::errs(), &*DiagOpts);
+
+  IntrusiveRefCntPtr<DiagnosticIDs> DiagID(new DiagnosticIDs());
+
+  DiagnosticsEngine Diags(DiagID, &*DiagOpts, DiagClient);
+
+  driver::Driver TheDriver(CLANG, Device->llvm_target_triplet, Diags);
+
+  const char **ArgsEnd = Args;
+  while (*ArgsEnd++ != nullptr) {}
+
+  llvm::ArrayRef<const char*> ArgsArray(Args, ArgsEnd);
+
+  std::unique_ptr<driver::Compilation> C(
+    TheDriver.BuildCompilation(ArgsArray));
+
+  if (C && !C->containsError()) {
+    SmallVector<std::pair<int, const driver::Command *>, 4> FailingCommands;
+    return TheDriver.ExecuteCompilation(*C, FailingCommands);
+  } else {
+    return -1;
+  }
+
+}
