@@ -27,6 +27,9 @@
 #include "pocl_llvm_api.h"
 #include "pocl_debug.h"
 
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
 #include <llvm/ADT/StringRef.h>
 #include <llvm/ADT/StringMap.h>
 
@@ -39,30 +42,20 @@
 #include <llvm/IR/Module.h>
 #include <llvm/IR/LLVMContext.h>
 #include <llvm/IR/DiagnosticPrinter.h>
-#ifndef LLVM_OLDER_THAN_6_0
 #include <llvm/IR/DiagnosticInfo.h>
-#endif
 
 #include <llvm/Target/TargetMachine.h>
 
 #include <llvm/IRReader/IRReader.h>
 
-#ifdef LLVM_OLDER_THAN_4_0
-#include <llvm/Bitcode/ReaderWriter.h>
-#else
 #include <llvm/Bitcode/BitcodeReader.h>
 #include <llvm/Bitcode/BitcodeWriter.h>
-#endif
 
 #include <llvm/Support/raw_os_ostream.h>
 #include <llvm/PassRegistry.h>
 
-#ifdef LLVM_OLDER_THAN_3_7
-#include <llvm/PassManager.h>
-#else
 #include <llvm/IR/LegacyPassManager.h>
 #define PassManager legacy::PassManager
-#endif
 
 using namespace llvm;
 
@@ -90,17 +83,10 @@ llvm::Module *parseModuleIRMem(const char *input_stream, size_t size) {
   std::unique_ptr<MemoryBuffer> buffer =
       MemoryBuffer::getMemBufferCopy(input_stream_ref);
 
-#ifdef LLVM_OLDER_THAN_3_8
-  llvm::ErrorOr<std::unique_ptr<llvm::Module>> parsed_module =
-      parseBitcodeFile(buffer->getMemBufferRef(), GlobalContext());
-  if (std::error_code ec = parsed_module.getError())
-    return nullptr;
-#else
   auto parsed_module =
       parseBitcodeFile(buffer->getMemBufferRef(), GlobalContext());
   if (!parsed_module)
     return nullptr;
-#endif
   return parsed_module.get().release();
 }
 
@@ -110,14 +96,10 @@ int getModuleTriple(const char *input_stream, size_t size,
   std::unique_ptr<MemoryBuffer> buffer =
       MemoryBuffer::getMemBufferCopy(input_stream_ref);
 
-#ifdef LLVM_OLDER_THAN_4_0
-  triple = getBitcodeTargetTriple(buffer->getMemBufferRef(), GlobalContext());
-#else
   auto triple_e = getBitcodeTargetTriple(buffer->getMemBufferRef());
   if (!triple_e)
     return -1;
   triple = triple_e.get();
-#endif
   return 0;
 }
 
@@ -127,18 +109,13 @@ get_llvm_cpu_name ()
 
   StringRef r = llvm::sys::getHostCPUName();
 
-#ifdef LLVM_3_8
-  // https://github.com/pocl/pocl/issues/413
-  if (r.str() == "skylake") {
-    r = llvm::StringRef("haswell");
-  }
-#endif
-
+#ifndef KERNELLIB_HOST_DISTRO_VARIANTS
   if (r.str() == "generic") {
     POCL_MSG_WARN("LLVM does not recognize your cpu, trying to use "
                    OCL_KERNEL_TARGET_CPU " for -target-cpu\n");
     r = llvm::StringRef(OCL_KERNEL_TARGET_CPU);
   }
+#endif
 
   assert(r.size() > 0);
   char *cpu_name = (char *)malloc(r.size() + 1);
@@ -156,9 +133,30 @@ int bitcode_is_spir(const char *bitcode, size_t size) {
     return 0;
 }
 
-int bitcode_is_spirv(const char *bitcode, size_t size) {
-  uint32_t magic = htole32(((uint32_t *)bitcode)[0]);
-  return (size > 20) && (magic == 0x07230203U);
+#define SPIRV_MAGIC 0x07230203U
+#define OpCapab 0x00020011
+#define KernelExecModel 0x6
+
+int bitcode_is_spirv(const char *bitcode, size_t size, int *is_opencl) {
+  const uint32_t *bc32 = (const uint32_t *)bitcode;
+  unsigned loc = 0;
+  uint32_t magic = htole32(bc32[loc++]);
+
+  if ((size < 20) || (magic != SPIRV_MAGIC))
+    return 0;
+
+  // skip version, generator, bound, schema
+  loc += 4;
+  *is_opencl = 0;
+  uint32_t ins, val;
+  do {
+    ins = htole32(bc32[loc++]);
+    val = htole32(bc32[loc++]);
+    if (val == KernelExecModel)
+      *is_opencl = 1;
+  } while (ins == OpCapab);
+
+  return 1;
 }
 
 // TODO this should be fixed to not require LLVM eventually,
@@ -288,9 +286,6 @@ void InitializeLLVM() {
   initializeVectorization(Registry);
   initializeIPO(Registry);
   initializeAnalysis(Registry);
-#ifdef LLVM_OLDER_THAN_3_8
-  initializeIPA(Registry);
-#endif
   initializeTransformUtils(Registry);
   initializeInstCombine(Registry);
   initializeInstrumentation(Registry);
@@ -300,12 +295,7 @@ void InitializeLLVM() {
 // device can reset their own options. Now one cannot compile
 // with different options to different devices at one run.
 
-#ifdef LLVM_OLDER_THAN_3_7
-  StringMap<llvm::cl::Option *> opts;
-  llvm::cl::getRegisteredOptions(opts);
-#else
   StringMap<llvm::cl::Option *> &opts = llvm::cl::getRegisteredOptions();
-#endif
 
   llvm::cl::Option *O = nullptr;
 
