@@ -2,7 +2,7 @@
    optimization passes and codegen.
 
    Copyright (c) 2013 Kalle Raiskila
-                 2013-2017 Pekka Jääskeläinen
+                 2013-2019 Pekka Jääskeläinen
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,9 @@
 #include <vector>
 #include <iostream>
 
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
 #include <llvm/Support/Casting.h>
 #include <llvm/Support/MutexGuard.h>
 #include <llvm/Support/TargetRegistry.h>
@@ -55,15 +58,10 @@
 #include <llvm/PassRegistry.h>
 #include <llvm/PassInfo.h>
 
-#ifdef LLVM_OLDER_THAN_3_7
-#include <llvm/PassManager.h>
-#include <llvm/Target/TargetLibraryInfo.h>
-#else
 #include <llvm/Analysis/TargetLibraryInfo.h>
 #include <llvm/Analysis/TargetTransformInfo.h>
 #include <llvm/IR/LegacyPassManager.h>
 #define PassManager legacy::PassManager
-#endif
 
 #include "linker.h"
 
@@ -87,36 +85,10 @@ extern cl::opt<std::string> KernelName;
  * cl_program's options. */
 static llvm::TargetOptions GetTargetOptions() {
   llvm::TargetOptions Options;
-#ifdef LLVM_OLDER_THAN_3_9
-  Options.PositionIndependentExecutable = true;
-#endif
 #ifdef HOST_FLOAT_SOFT_ABI
   Options.FloatABIType = FloatABI::Soft;
 #else
   Options.FloatABIType = FloatABI::Hard;
-#endif
-#if 0
-  Options.LessPreciseFPMADOption = EnableFPMAD;
-  Options.NoFramePointerElim = DisableFPElim;
-  Options.NoFramePointerElimNonLeaf = DisableFPElimNonLeaf;
-  Options.AllowFPOpFusion = FuseFPOps;
-  Options.UnsafeFPMath = EnableUnsafeFPMath;
-  Options.NoInfsFPMath = EnableNoInfsFPMath;
-  Options.NoNaNsFPMath = EnableNoNaNsFPMath;
-  Options.HonorSignDependentRoundingFPMathOption =
-  EnableHonorSignDependentRoundingFPMath;
-  Options.UseSoftFloat = GenerateSoftFloatCalls;
-  if (FloatABIForCalls != FloatABI::Default)
-    Options.FloatABIType = FloatABIForCalls;
-  Options.NoZerosInBSS = DontPlaceZerosInBSS;
-  Options.GuaranteedTailCallOpt = EnableGuaranteedTailCallOpt;
-  Options.DisableTailCalls = DisableTailCalls;
-  Options.StackAlignmentOverride = OverrideStackAlignment;
-  Options.RealignStack = EnableRealignStack;
-  Options.TrapFuncName = TrapFuncName;
-  Options.EnableSegmentedStacks = SegmentedStacks;
-  Options.UseInitArray = UseInitArray;
-  Options.SSPBufferSize = SSPBufferSize;
 #endif
   return Options;
 }
@@ -200,36 +172,19 @@ kernel_compiler_passes(cl_device_id device, llvm::Module *input,
   Triple triple(device->llvm_target_triplet);
   TargetMachine *Machine = GetTargetMachine(device, triple);
 
-#ifdef LLVM_OLDER_THAN_3_7
-  // Add internal analysis passes from the target machine.
-  if (Machine)
-    Machine->addAnalysisPasses(*Passes);
-#else
   if (Machine)
     Passes->add(
         createTargetTransformInfoWrapperPass(Machine->getTargetIRAnalysis()));
-#endif
 
-  if (module_data_layout != "") {
-#if (defined LLVM_OLDER_THAN_3_7)
-    Passes->add(new DataLayoutPass());
-#endif
-  }
 
   /* Disables automated generation of libcalls from code patterns.
      TCE doesn't have a runtime linker which could link the libs later on.
      Also the libcalls might be harmful for WG autovectorization where we
      want to try to vectorize the code it converts to e.g. a memset or
      a memcpy */
-#ifdef LLVM_OLDER_THAN_3_7
-  TargetLibraryInfo *TLI = new TargetLibraryInfo(triple);
-  TLI->disableAllFunctions();
-  Passes->add(TLI);
-#else
   TargetLibraryInfoImpl TLII(triple);
   TLII.disableAllFunctions();
   Passes->add(new TargetLibraryInfoWrapperPass(TLII));
-#endif
 
   /* The kernel compiler passes to run, in order.
 
@@ -265,25 +220,19 @@ kernel_compiler_passes(cl_device_id device, llvm::Module *input,
   if (SPMDDevice) {
     passes.push_back("flatten-inline-all");
     passes.push_back("always-inline");
-  }  else {
+  } else {
     passes.push_back("flatten-globals");
+    passes.push_back("flatten-barrier-subs");
     passes.push_back("always-inline");
-#ifndef LLVM_3_9
     passes.push_back("inline");
-#endif
   }
 
-#ifndef LLVM_OLDER_THAN_4_0
   // It should be now safe to run -O3 over the single work-item kernel
   // as the barrier has the attributes preventing illegal motions and
   // duplication. Let's do it to clean up the code for later passes.
   // Especially the WI context structures get needlessly bloated in case there
   // is dead code lying around.
   passes.push_back("STANDARD_OPTS");
-#else
-  // Just clean up any unused globals.
-  passes.push_back("globaldce");
-#endif
 
   if (!SPMDDevice) {
     passes.push_back("simplifycfg");
@@ -305,20 +254,22 @@ kernel_compiler_passes(cl_device_id device, llvm::Module *input,
     // work-item loop control taking care of them.
     passes.push_back("remove-barriers");
   }
+
   // Add the work group launcher functions and privatize the pseudo variable
-  // (local id) accesses.
-  if (device->workgroup_pass)
+  // (local id) accesses. We have to do this late because we rely on aggressive
+  // inlining to expose the _{local,group}_id accesses which will be replaced
+  // with context struct accesses. TODO: A cleaner and a more robust way would
+  // be to add hidden context struct parameters to the builtins that need the
+  // context data and fix the calls early.
+  if (device->workgroup_pass) {
     passes.push_back("workgroup");
+    passes.push_back("always-inline");
+  }
 
   // Attempt to move all allocas to the entry block to avoid the need for
   // dynamic stack which is problematic for some architectures.
   passes.push_back("allocastoentry");
 
-#ifdef POCL_USE_FAKE_ADDR_SPACE_IDS
-  // Convert the semantical OpenCL address space IDs to the ones of the target.
-  passes.push_back("target-address-spaces");
-#endif
-
   // Later passes might get confused (and expose possible bugs in them) due to
   // UNREACHABLE blocks left by repl. So let's clean up the CFG before running
   // the standard LLVM optimizations.
@@ -336,6 +287,11 @@ kernel_compiler_passes(cl_device_id device, llvm::Module *input,
   passes.push_back("STANDARD_OPTS");
   passes.push_back("instcombine");
 
+  // Due to unfortunate phase-ordering problems with store sinking,
+  // loop deletion does not always apply when executing -O3 only
+  // once. Cherry pick the optimization to rerun here.
+  passes.push_back("loop-deletion");
+
   // Now actually add the listed passes to the PassManager.
   for (unsigned i = 0; i < passes.size(); ++i) {
     // This is (more or less) -O3.
@@ -350,10 +306,8 @@ kernel_compiler_passes(cl_device_id device, llvm::Module *input,
         Builder.LoopVectorize = true;
         Builder.SLPVectorize = true;
       }
-#ifndef POCL_USE_FAKE_ADDR_SPACE_IDS
       Builder.VerifyInput = true;
       Builder.VerifyOutput = true;
-#endif
       Builder.populateModulePassManager(*Passes);
       continue;
     }
@@ -386,117 +340,123 @@ void pocl_destroy_llvm_module(void *modp) {
   }
 }
 
-// Defined in llvmopencl/WorkitemHandler.cc
+// The global variables used to control the WG function generation's
+// specialization parameteres. Defined in lib/llvmopencl/WorkitemHandler.cc.
 namespace pocl {
 extern size_t WGLocalSizeX;
 extern size_t WGLocalSizeY;
 extern size_t WGLocalSizeZ;
 extern bool WGDynamicLocalSize;
+extern size_t WGMaxGridDimWidth;
+extern bool WGAssumeZeroGlobalOffset;
 }
 
 int pocl_update_program_llvm_irs_unlocked(cl_program program,
                                           unsigned device_i);
 
-int pocl_llvm_generate_workgroup_function_nowrite(cl_device_id device,
-  cl_kernel kernel, size_t local_x, size_t local_y, size_t local_z, void **output) {
+int pocl_llvm_generate_workgroup_function_nowrite(
+    unsigned DeviceI, cl_device_id Device, cl_kernel Kernel,
+    _cl_command_node *Command, void **Output, int Specialize) {
 
-  cl_program program = kernel->program;
-  int device_i = pocl_cl_device_to_index(program, device);
-  assert(device_i >= 0);
+  _cl_command_run *RunCommand = &Command->command.run;
+  cl_program Program = Kernel->program;
 
-  pocl::WGDynamicLocalSize = (local_x == 0 && local_y == 0 && local_z == 0);
+  currentPoclDevice = Device;
 
-  currentPoclDevice = device;
-
-  PoclCompilerMutexGuard lockHolder(NULL);
+  PoclCompilerMutexGuard LockHolder(NULL);
   InitializeLLVM();
 
-#ifdef DEBUG_POCL_LLVM_API
-  printf("### calling the kernel compiler for kernel %s local_x %zu "
-         "local_y %zu local_z %zu parallel_filename: %s\n",
-         kernel->name, local_x, local_y, local_z, parallel_bc_path);
-#endif
+  if (Program->llvm_irs[DeviceI] == NULL)
+    pocl_update_program_llvm_irs_unlocked(Program, DeviceI);
 
-  if (program->llvm_irs[device_i] == NULL)
-    pocl_update_program_llvm_irs_unlocked(program, device_i);
+  llvm::Module *ProgramBC = (llvm::Module *)Program->llvm_irs[DeviceI];
 
-  llvm::Module *program_bc = (llvm::Module *)program->llvm_irs[device_i];
-
-  /* Create an empty Module and copy
-   * only the kernel+callgraph from program.bc */
-  llvm::Module *parallel_bc =
+  // Create an empty Module and copy only the kernel+callgraph from
+  // program.bc.
+  llvm::Module *ParallelBC =
       new llvm::Module(StringRef("parallel_bc"), GlobalContext());
 
-  parallel_bc->setTargetTriple(program_bc->getTargetTriple());
-  parallel_bc->setDataLayout(program_bc->getDataLayout());
+  ParallelBC->setTargetTriple(ProgramBC->getTargetTriple());
+  ParallelBC->setDataLayout(ProgramBC->getDataLayout());
 
-  copyKernelFromBitcode(kernel->name, parallel_bc, program_bc);
+  copyKernelFromBitcode(Kernel->name, ParallelBC, ProgramBC);
 
-  /* Now finally run the set of passes assembled above
-   * TODO pass these as parameters instead, this is not thread safe! */
-  pocl::WGLocalSizeX = local_x;
-  pocl::WGLocalSizeY = local_y;
-  pocl::WGLocalSizeZ = local_z;
-  KernelName = kernel->name;
+  // Set the specialization properties.
+  if (Specialize) {
+    pocl::WGLocalSizeX = RunCommand->pc.local_size[0];
+    pocl::WGLocalSizeY = RunCommand->pc.local_size[1];
+    pocl::WGLocalSizeZ = RunCommand->pc.local_size[2];
+    pocl::WGDynamicLocalSize = pocl::WGLocalSizeX == 0 &&
+                               pocl::WGLocalSizeY == 0 &&
+                               pocl::WGLocalSizeZ == 0;
+    pocl::WGAssumeZeroGlobalOffset = RunCommand->pc.global_offset[0] == 0 &&
+                                     RunCommand->pc.global_offset[1] == 0 &&
+                                     RunCommand->pc.global_offset[2] == 0;
+    // Compile a smallgrid version or a generic one?
+    if (RunCommand->force_large_grid_wg_func ||
+        pocl_cmd_max_grid_dim_width(RunCommand) >=
+            Device->grid_width_specialization_limit) {
+      pocl::WGMaxGridDimWidth = 0; // The generic / large / unlimited size one.
+    } else {
+      // Limited grid dimension width by the device specific limit.
+      pocl::WGMaxGridDimWidth = Device->grid_width_specialization_limit;
+    }
+  } else {
+    pocl::WGDynamicLocalSize = true;
+    pocl::WGLocalSizeX = pocl::WGLocalSizeY = pocl::WGLocalSizeZ = 0;
+    pocl::WGAssumeZeroGlobalOffset = false;
+    pocl::WGMaxGridDimWidth = 0;
+  }
 
-#ifdef LLVM_OLDER_THAN_3_7
-  kernel_compiler_passes(device, parallel_bc,
-                         parallel_bc->getDataLayout()->getStringRepresentation())
-      .run(*parallel_bc);
-#else
-  kernel_compiler_passes(device, parallel_bc,
-                         parallel_bc->getDataLayout().getStringRepresentation())
-      .run(*parallel_bc);
-#endif
+  KernelName = Kernel->name;
 
-  assert(output != NULL);
-  *output = (void *)parallel_bc;
+  kernel_compiler_passes(Device, ParallelBC,
+                         ParallelBC->getDataLayout().getStringRepresentation())
+      .run(*ParallelBC);
+
+  assert(Output != NULL);
+  *Output = (void *)ParallelBC;
   ++numberOfIRs;
   return 0;
 }
 
+int pocl_llvm_generate_workgroup_function(unsigned DeviceI, cl_device_id Device,
+                                          cl_kernel Kernel,
+                                          _cl_command_node *Command,
+                                          int Specialize) {
 
-int pocl_llvm_generate_workgroup_function(cl_device_id device, cl_kernel kernel,
-                                          size_t local_x, size_t local_y,
-                                          size_t local_z) {
+  void *Module = NULL;
 
-  void *modp = NULL;
+  char ParallelBCPath[POCL_FILENAME_LENGTH];
+  pocl_cache_work_group_function_path(ParallelBCPath, Kernel->program, DeviceI,
+                                      Kernel, Command, Specialize);
 
-  int device_i = pocl_cl_device_to_index(kernel->program, device);
-  assert(device_i >= 0);
-
-  char parallel_bc_path[POCL_FILENAME_LENGTH];
-  pocl_cache_work_group_function_path(parallel_bc_path, kernel->program,
-                                      device_i, kernel, local_x, local_y,
-                                      local_z);
-
-  if (pocl_exists(parallel_bc_path))
+  if (pocl_exists(ParallelBCPath))
     return CL_SUCCESS;
 
-  char final_binary_path[POCL_FILENAME_LENGTH];
-  pocl_cache_final_binary_path(final_binary_path, kernel->program, device_i,
-                               kernel, local_x, local_y, local_z);
+  char FinalBinaryPath[POCL_FILENAME_LENGTH];
+  pocl_cache_final_binary_path(FinalBinaryPath, Kernel->program, DeviceI,
+                               Kernel, Command, Specialize);
 
-  if (pocl_exists(final_binary_path))
+  if (pocl_exists(FinalBinaryPath))
     return CL_SUCCESS;
 
-  int error = pocl_llvm_generate_workgroup_function_nowrite(
-      device, kernel, local_x, local_y, local_z, &modp);
-  if (error)
-    return error;
+  int Error = pocl_llvm_generate_workgroup_function_nowrite(
+      DeviceI, Device, Kernel, Command, &Module, Specialize);
+  if (Error)
+    return Error;
 
-  error = pocl_cache_write_kernel_parallel_bc(
-      modp, kernel->program, device_i, kernel, local_x, local_y, local_z);
+  Error = pocl_cache_write_kernel_parallel_bc(Module, Kernel->program, DeviceI,
+                                              Kernel, Command, Specialize);
 
-  if (error)
-    {
-      POCL_MSG_ERR ("pocl_cache_write_kernel_parallel_bc()"
-                    " failed with %i\n", error);
-      return error;
-    }
+  if (Error) {
+    POCL_MSG_ERR("pocl_cache_write_kernel_parallel_bc() failed with %i\n",
+                 Error);
+    return Error;
+  }
 
-  pocl_destroy_llvm_module(modp);
-  return error;
+  pocl_destroy_llvm_module(Module);
+  return Error;
 }
 
 int pocl_update_program_llvm_irs_unlocked(cl_program program,
@@ -526,7 +486,7 @@ int pocl_update_program_llvm_irs(cl_program program,
   return pocl_update_program_llvm_irs_unlocked(program, device_i);
 }
 
-void pocl_free_llvm_irs(cl_program program, int device_i) {
+void pocl_free_llvm_irs(cl_program program, unsigned device_i) {
   if (program->llvm_irs[device_i]) {
     PoclCompilerMutexGuard lockHolder(NULL);
     InitializeLLVM();
@@ -585,67 +545,109 @@ void pocl_llvm_update_binaries(cl_program program) {
   }
 }
 
+static void initPassManagerForCodeGen(PassManager& PM, cl_device_id Device) {
+
+  llvm::Triple Triple(Device->llvm_target_triplet);
+
+  llvm::TargetLibraryInfoWrapperPass *TLIPass =
+      new TargetLibraryInfoWrapperPass(Triple);
+  PM.add(TLIPass);
+}
 
 /* Run LLVM codegen on input file (parallel-optimized).
  * modp = llvm::Module* of parallel.bc
  * Output native object file (<kernel>.so.o). */
-int pocl_llvm_codegen(cl_device_id device, void *modp, char **output,
-                      size_t *output_size) {
+int pocl_llvm_codegen(cl_device_id Device, void *Modp, char **Output,
+                      uint64_t *OutputSize) {
 
-  PoclCompilerMutexGuard lockHolder(NULL);
+  PoclCompilerMutexGuard LockHolder(nullptr);
   InitializeLLVM();
 
-  llvm::Module *input = (llvm::Module *)modp;
-  assert(input);
-  *output = NULL;
+  llvm::Module *Input = (llvm::Module *)Modp;
+  assert(Input);
+  *Output = nullptr;
 
-  llvm::Triple triple(device->llvm_target_triplet);
-  llvm::TargetMachine *target = GetTargetMachine(device, triple);
+  PassManager PMObj;
+  initPassManagerForCodeGen(PMObj, Device);
 
-  PassManager PM;
-#ifdef LLVM_OLDER_THAN_3_7
-  llvm::TargetLibraryInfo *TLI = new TargetLibraryInfo(triple);
-  PM.add(TLI);
+  llvm::Triple Triple(Device->llvm_target_triplet);
+  llvm::TargetMachine *Target = GetTargetMachine(Device, Triple);
+
+  // First try direct object code generation from LLVM, if supported by the
+  // LLVM backend for the target.
+  bool LLVMGeneratesObjectFiles = true;
+
+  SmallVector<char, 4096> Data;
+  llvm::raw_svector_ostream SOS(Data);
+  bool cannotEmitFile;
+
+#ifdef LLVM_OLDER_THAN_7_0
+  cannotEmitFile = Target->addPassesToEmitFile(PMObj, SOS,
+                                  TargetMachine::CGFT_ObjectFile);
 #else
-  llvm::TargetLibraryInfoWrapperPass *TLIPass =
-      new TargetLibraryInfoWrapperPass(triple);
-  PM.add(TLIPass);
+  cannotEmitFile = Target->addPassesToEmitFile(PMObj, SOS, nullptr,
+                                  TargetMachine::CGFT_ObjectFile);
 #endif
-#ifdef LLVM_OLDER_THAN_3_7
-  if (target != NULL) {
-    target->addAnalysisPasses(PM);
+
+  LLVMGeneratesObjectFiles = !cannotEmitFile;
+
+  if (LLVMGeneratesObjectFiles) {
+    POCL_MSG_PRINT_LLVM("Generating an object file directly.\n");
+    PMObj.run(*Input);
+    std::string O = SOS.str(); // flush
+    const char *Cstr = O.c_str();
+    size_t S = O.size();
+    *Output = (char *)malloc(S);
+    *OutputSize = S;
+    memcpy(*Output, Cstr, S);
+    return 0;
+  }
+
+  PassManager PMAsm;
+  initPassManagerForCodeGen(PMAsm, Device);
+
+  POCL_MSG_PRINT_LLVM("Generating assembly text.\n");
+
+  // The LLVM target does not implement support for emitting object file directly.
+  // Have to emit the text first and then call the assembler from the command line
+  // to produce the binary.
+#ifdef LLVM_OLDER_THAN_7_0
+  if (Target->addPassesToEmitFile(PMAsm, SOS,
+                                  TargetMachine::CGFT_AssemblyFile)) {
+    POCL_ABORT("The target supports neither obj nor asm emission!");
+  }
+#else
+  if (Target->addPassesToEmitFile(PMAsm, SOS, nullptr,
+                                  TargetMachine::CGFT_AssemblyFile)) {
+    POCL_ABORT("The target supports neither obj nor asm emission!");
   }
 #endif
 
-  // TODO: get DataLayout from the 'device'
-  // TODO: better error check
-#ifdef LLVM_OLDER_THAN_3_7
-  std::string data;
-  llvm::raw_string_ostream sos(data);
-  llvm::MCContext *mcc;
-  if (target && target->addPassesToEmitMC(PM, mcc, sos))
-    return 1;
-#else
-  SmallVector<char, 4096> data;
-  llvm::raw_svector_ostream sos(data);
-  if (target &&
-#ifdef LLVM_OLDER_THAN_7_0
-      target->addPassesToEmitFile(PM, sos, TargetMachine::CGFT_ObjectFile))
-#else
-      target->addPassesToEmitFile(PM, sos, nullptr,
-                                  TargetMachine::CGFT_ObjectFile))
-#endif
-    return 1;
-#endif
+  // This produces the assembly text:
+  PMAsm.run(*Input);
 
-  PM.run(*input);
-  std::string o = sos.str(); // flush
-  const char *cstr = o.c_str();
-  size_t s = o.size();
-  *output = (char *)malloc(s);
-  *output_size = s;
-  memcpy(*output, cstr, s);
+  // Next call the target's assembler via the Toolchain API indirectly through
+  // the Driver API.
+
+  char AsmFileName[POCL_FILENAME_LENGTH];
+  char ObjFileName[POCL_FILENAME_LENGTH];
+
+  std::string AsmStr = SOS.str().str();
+  pocl_write_tempfile(AsmFileName, "/tmp/pocl-asm", ".s", AsmStr.c_str(),
+                      AsmStr.size(), nullptr);
+  pocl_mk_tempname(ObjFileName, "/tmp/pocl-obj", ".o", nullptr);
+
+  const char *Args[] = {CLANG, AsmFileName, "-c", "-o", ObjFileName, nullptr};
+  int Res = pocl_invoke_clang(Device, Args);
+
+  if (Res == 0) {
+    if (pocl_read_file(ObjFileName, Output, OutputSize))
+      POCL_ABORT("Could not read the object file.");
+  }
+
+  pocl_remove(AsmFileName);
+  pocl_remove(ObjFileName);
+  return Res;
 
-  return 0;
 }
 /* vim: set ts=4 expandtab: */
