@@ -29,7 +29,7 @@
 #include <string.h>
 #include "pocl_cl.h"
 
-#if defined(HAVE_POSIX_MEMALIGN) || defined(POCL_ANDROID) \
+#if defined(HAVE_POSIX_MEMALIGN) || defined(__ANDROID__) \
      || (defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 201112L))
 #define HAVE_ALIGNED_ALLOC
 #else
@@ -69,7 +69,8 @@ unsigned pocl_save_ftz ();
 void pocl_restore_ftz (unsigned ftz);
 
 /* Finds the next highest power of two of the given value. */
-size_t pocl_size_ceil2(size_t x);
+size_t pocl_size_ceil2 (size_t x);
+uint64_t pocl_size_ceil2_64 (uint64_t x);
 
 /* Allocates aligned blocks of memory.
  *
@@ -145,8 +146,6 @@ pocl_command_is_ready(cl_event event)
 
 typedef void (*empty_queue_callback) (cl_command_queue cq);
 
-void pocl_update_command_queue (cl_event event, empty_queue_callback p);
-
 void pocl_cl_mem_inherit_flags (cl_mem mem, cl_mem from_buffer,
                                 cl_mem_flags flags);
 
@@ -179,6 +178,25 @@ void pocl_abort_on_pthread_error (int status, unsigned line, const char *func);
 #define PTHREAD_CHECK(code)                                                   \
   pocl_abort_on_pthread_error ((code), __LINE__, __FUNCTION__);
 
+void pocl_update_event_queued (cl_event event);
+
+void pocl_update_event_submitted (cl_event event);
+
+void pocl_update_event_running_unlocked (cl_event event);
+
+void pocl_update_event_running (cl_event event);
+
+void pocl_update_event_complete_msg (const char *func, unsigned line,
+                                     cl_event event, const char *msg);
+
+#define POCL_UPDATE_EVENT_COMPLETE_MSG(__event, msg)                          \
+  pocl_update_event_complete_msg (__func__, __LINE__, (__event), msg);
+
+#define POCL_UPDATE_EVENT_COMPLETE(__event)                                   \
+  pocl_update_event_complete_msg (__func__, __LINE__, (__event), NULL);
+
+void pocl_update_event_failed (cl_event event);
+
 const char*
 pocl_status_to_str (int status);
 
@@ -279,8 +297,8 @@ float half_to_float (uint16_t value);
              ? mem->buffer                                                    \
              : mem);
 
-#define IS_IMAGE1D_BUFFER(mem)                                                \
-  (mem->is_image && (mem->type == CL_MEM_OBJECT_IMAGE1D_BUFFER))
+#define IS_IMAGE1D_BUFFER(mem)                                        \
+  (mem && mem->is_image && (mem->type == CL_MEM_OBJECT_IMAGE1D_BUFFER))
 
 #define IMAGE1D_ORIG_REG_TO_BYTES(mem, o, r)                                  \
   size_t px = (mem->image_elem_size * mem->image_channels);                   \
