@@ -45,7 +45,7 @@ int printf(const char *format, ...);
 
 #define DEFINE_PRINT_INTS(NAME, INT_TYPE, UINT_TYPE)                          \
   void __pocl_print_ints_##NAME (param_t *p, OCL_C_AS const void *vals,       \
-                                     int n, int is_unsigned)                  \
+                                 int n, int is_unsigned)                      \
   {                                                                           \
     DEBUG_PRINTF (("[printf:ints:n=%df]\n", n));                              \
     flags_t saved_user_flags = p->flags;                                      \
@@ -76,16 +76,16 @@ DEFINE_PRINT_INTS (ulong, int64_t, uint64_t)
 
 /**************************************************************************/
 
-const char *NANs[2] = { "nan", "NAN" };
-const char *INFs[2] = { "inf", "INF" };
-
 /* Note: NANs are printed always positive.
  * This is required to pass 1.2 conformance test. */
 #define DEFINE_PRINT_FLOATS(FLOAT_TYPE)                                       \
   void __pocl_print_floats_##FLOAT_TYPE (param_t *p,                          \
                                          OCL_C_AS const void *vals, int n)    \
   {                                                                           \
-    DEBUG_PRINTF (("[printf:floats:n=%dd]\n", n));                            \
+    const char *NANs[2] = { "nan", "NAN" };                                   \
+    const char *INFs[2] = { "inf", "INF" };                                   \
+                                                                              \
+    DEBUG_PRINTF (("[printf:floats:n=%d]\n", n));                             \
     flags_t saved_user_flags = p->flags;                                      \
     for (int d = 0; d < n; ++d)                                               \
       {                                                                       \
@@ -98,7 +98,10 @@ const char *INFs[2] = { "inf", "INF" };
         if (val != val)                                                       \
           other = NANs[p->flags.uc ? 1 : 0];                                  \
         if (val == (-INFINITY))                                               \
-          { val = INFINITY; p->flags.sign = 1; }                              \
+          {                                                                   \
+            val = INFINITY;                                                   \
+            p->flags.sign = 1;                                                \
+          }                                                                   \
         if (val == (INFINITY))                                                \
           other = INFs[p->flags.uc ? 1 : 0];                                  \
         if (other)                                                            \
@@ -441,7 +444,7 @@ __pocl_printf_format_full (const OCL_CONSTANT_AS char *restrict format,
                         break;
 #ifdef cl_khr_int64
                       case 8:
-                        CALL_PRINT_INTS (ulong, uint64_t);
+                        CALL_PRINT_INTS (ulong, ulong);
                         break;
 #endif
                       default:
@@ -682,8 +685,8 @@ error:;
  * after the external (buffer) variables are handled in a LLVM pass. */
 
 int
-__pocl_printf (char *restrict __buffer, size_t *__buffer_index,
-               size_t __buffer_capacity,
+__pocl_printf (char *restrict __buffer, uint32_t *__buffer_index,
+               uint32_t __buffer_capacity,
                const OCL_CONSTANT_AS char *restrict fmt, ...)
 {
   param_t p = { 0 };
@@ -705,8 +708,8 @@ __pocl_printf (char *restrict __buffer, size_t *__buffer_index,
 /**************************************************************************/
 
 extern char *_printf_buffer;
-extern size_t *_printf_buffer_position;
-extern size_t _printf_buffer_capacity;
+extern uint32_t *_printf_buffer_position;
+extern uint32_t _printf_buffer_capacity;
 
 /* This is a placeholder printf function that will be replaced by calls
  * to __pocl_printf(), after a LLVM pass handles the hidden arguments.
@@ -714,7 +717,7 @@ extern size_t _printf_buffer_capacity;
  * here, so that the kernel library linker pulls them in. */
 
 int
-__cl_printf (const OCL_CONSTANT_AS char *restrict fmt, ...)
+printf (const OCL_CONSTANT_AS char *restrict fmt, ...)
 {
   va_list va;
   va_start (va, fmt);
