@@ -1,17 +1,17 @@
 // LLVM function pass to convert all PHIs to allocas.
-// 
-// Copyright (c) 2012 Pekka Jääskeläinen / TUT
-// 
+//
+// Copyright (c) 2012-2019 Pekka Jääskeläinen
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -24,8 +24,10 @@
 
 #include "config.h"
 
-#include "llvm/IR/IRBuilder.h"
-#include "llvm/IR/TypeBuilder.h"
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
+#include <llvm/IR/IRBuilder.h>
 
 #include "PHIsToAllocas.h"
 #include "Workgroup.h"
