@@ -1,17 +1,17 @@
 // Implementation for VariableUniformityAnalysis function pass.
-// 
-// Copyright (c) 2013-2014 Pekka Jääskeläinen / Tampere University of Technology
-// 
+//
+// Copyright (c) 2013-2019 Pekka Jääskeläinen
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -69,29 +69,14 @@ VariableUniformityAnalysis::VariableUniformityAnalysis() : FunctionPass(ID) {
 
 void
 VariableUniformityAnalysis::getAnalysisUsage(llvm::AnalysisUsage &AU) const {
-#ifdef LLVM_OLDER_THAN_3_9
-  AU.addRequired<PostDominatorTree>();
-  AU.addPreserved<PostDominatorTree>();
-#else
   AU.addRequired<PostDominatorTreeWrapperPass>();
   AU.addPreserved<PostDominatorTreeWrapperPass>();
-#endif
 
-#ifdef LLVM_OLDER_THAN_3_7
-  AU.addRequired<LoopInfo>();
-  AU.addPreserved<LoopInfo>();
-#else
   AU.addRequired<LoopInfoWrapperPass>();
   AU.addPreserved<LoopInfoWrapperPass>();
-#endif
   // required by LoopInfo:
   AU.addRequired<DominatorTreeWrapperPass>();
   AU.addPreserved<DominatorTreeWrapperPass>();
-
-#ifdef LLVM_OLDER_THAN_3_7
-  AU.addRequired<DataLayoutPass>();
-  AU.addPreserved<DataLayoutPass>();
-#endif
 }
 
 // Recursively mark the canonical induction variable PHI as uniform.
@@ -129,11 +114,7 @@ VariableUniformityAnalysis::runOnFunction(Function &F) {
      divergence analysis. */
   uniformityCache_[&F].clear();
 
-#ifdef LLVM_OLDER_THAN_3_7
-  llvm::LoopInfo &LI = getAnalysis<LoopInfo>();
-#else
   llvm::LoopInfo &LI = getAnalysis<LoopInfoWrapperPass>().getLoopInfo();
-#endif
 
   for (llvm::LoopInfo::iterator i = LI.begin(), e = LI.end(); i != e; ++i) {
     llvm::Loop *L = *i;
@@ -210,10 +191,10 @@ VariableUniformityAnalysis::analyzeBBDivergence
             << std::endl;
 #endif
 
-  llvm::TerminatorInst *Term = previousUniformBB->getTerminator();
+  auto Term = previousUniformBB->getTerminator();
   if (Term == NULL) {
-    // this is most likely a function with a single basic block, the entry node, which
-    // ends with a ret
+    // this is most likely a function with a single basic block, the entry
+    // node, which ends with a ret
     return;
   }
 
@@ -245,13 +226,8 @@ VariableUniformityAnalysis::analyzeBBDivergence
 
   // Condition b)
   if (FoundUniforms.size() == 0) {
-#ifdef LLVM_OLDER_THAN_3_9
-    llvm::PostDominatorTree *PDT = &getAnalysis<PostDominatorTree>();
-    if (PDT->dominates(bb, previousUniformBB)) {
-#else
     llvm::PostDominatorTreeWrapperPass *PDT = &getAnalysis<PostDominatorTreeWrapperPass>();
     if (PDT->getPostDomTree().dominates(bb, previousUniformBB)) {
-#endif
       setUniform(f, bb, true);
       FoundUniforms.push_back(bb);
     }
@@ -264,9 +240,10 @@ VariableUniformityAnalysis::analyzeBBDivergence
   for (auto UniformBB : FoundUniforms) {
 
     // Propagate the Uniform BB data downwards.
-    llvm::TerminatorInst *NextTerm = UniformBB->getTerminator();
+    auto NextTerm = UniformBB->getTerminator();
 
-    for (unsigned suc = 0, end = NextTerm->getNumSuccessors(); suc < end; ++suc) {
+    for (unsigned suc = 0, end = NextTerm->getNumSuccessors(); suc < end;
+         ++suc) {
       llvm::BasicBlock *NextBB = NextTerm->getSuccessor(suc);
       if (!isUniformityAnalyzed(f, NextBB)) {
         analyzeBBDivergence(f, NextBB, UniformBB);
