@@ -1,18 +1,18 @@
 // LLVM function pass to select the best way to create a work group
 // function for a kernel and work group size.
-// 
-// Copyright (c) 2012 Pekka Jääskeläinen / Tampere University of Technology
-// 
+//
+// Copyright (c) 2012-2019 Pekka Jääskeläinen
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -25,6 +25,9 @@
 
 #include <iostream>
 
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
 #include "llvm/Analysis/PostDominators.h"
 #include "llvm/Analysis/LoopInfo.h"
 
@@ -39,9 +42,8 @@ using namespace llvm;
 using namespace pocl;
 
 namespace {
-  static
-  RegisterPass<WorkitemHandlerChooser> X(
-      "workitem-handler-chooser", 
+static RegisterPass<WorkitemHandlerChooser>
+    X("workitem-handler-chooser",
       "Finds the best way to handle work-items to produce a multi-WG function.",
       false, false);
 }
@@ -100,15 +102,13 @@ WorkitemHandlerChooser::runOnFunction(Function &F)
       {
         ReplThreshold = atoi(getenv("POCL_FULL_REPLICATION_THRESHOLD"));
       }
-      
-      if (WGLocalSizeX*WGLocalSizeY*WGLocalSizeZ <= ReplThreshold)
-        {
-          chosenHandler_ = POCL_WIH_FULL_REPLICATION;
-        }
-      else
-        {
-          chosenHandler_ = POCL_WIH_LOOPS;
-        }
+
+      if (!WGDynamicLocalSize &&
+          WGLocalSizeX * WGLocalSizeY * WGLocalSizeZ <= ReplThreshold) {
+        chosenHandler_ = POCL_WIH_FULL_REPLICATION;
+      } else {
+        chosenHandler_ = POCL_WIH_LOOPS;
+      }
     }
 
   return false;
