@@ -1,18 +1,18 @@
-// LLVM function pass to create loops that run all the work items 
+// LLVM function pass to create loops that run all the work items
 // in a work group while respecting barrier synchronization points.
-// 
-// Copyright (c) 2012-2013 Pekka Jääskeläinen / Tampere University of Technology
-// 
+//
+// Copyright (c) 2012-2019 Pekka Jääskeläinen
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -30,11 +30,14 @@
 
 #include "pocl.h"
 
+#include "CompilerWarnings.h"
+IGNORE_COMPILER_WARNING("-Wunused-parameter")
+
 #include "llvm/ADT/Statistic.h"
 #include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/IR/IRBuilder.h"
-#include "llvm/IR/TypeBuilder.h"
+#include "llvm/IR/MDBuilder.h"
 #include "llvm/IR/DataLayout.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Module.h"
@@ -72,17 +75,9 @@ char WorkitemLoops::ID = 0;
 void
 WorkitemLoops::getAnalysisUsage(AnalysisUsage &AU) const
 {
-#ifdef LLVM_OLDER_THAN_3_9
-  AU.addRequired<PostDominatorTree>();
-#else
   AU.addRequired<PostDominatorTreeWrapperPass>();
-#endif
 
-#ifdef LLVM_OLDER_THAN_3_7
-  AU.addRequired<LoopInfo>();
-#else
   AU.addRequired<LoopInfoWrapperPass>();
-#endif
   AU.addRequired<DominatorTreeWrapperPass>();
 
   AU.addRequired<VariableUniformityAnalysis>();
@@ -105,17 +100,9 @@ WorkitemLoops::runOnFunction(Function &F)
 
   DTP = &getAnalysis<DominatorTreeWrapperPass>();
   DT = &DTP->getDomTree();
-#ifdef LLVM_OLDER_THAN_3_7
-  LI = &getAnalysis<LoopInfo>();
-#else
   LI = &getAnalysis<LoopInfoWrapperPass>();
-#endif
 
-#ifdef LLVM_OLDER_THAN_3_9
-  PDT = &getAnalysis<PostDominatorTree>();
-#else
   PDT = &getAnalysis<PostDominatorTreeWrapperPass>();
-#endif
 
   tempInstructionIndex = 0;
 
@@ -255,8 +242,7 @@ WorkitemLoops::CreateLoopAround
 
   if (peeledFirst) {
     builder.CreateStore(builder.CreateLoad(localIdXFirstVar), localIdVar);
-    builder.CreateStore
-      (ConstantInt::get(IntegerType::get(C, size_t_width), 0), localIdXFirstVar);
+    builder.CreateStore(ConstantInt::get(SizeT, 0), localIdXFirstVar);
 
     if (WGDynamicLocalSize) {
       llvm::Value *cmpResult;
@@ -268,8 +254,7 @@ WorkitemLoops::CreateLoopAround
       builder.CreateBr(loopBodyEntryBB);
     }
   } else {
-    builder.CreateStore
-      (ConstantInt::get(IntegerType::get(C, size_t_width), 0), localIdVar);
+    builder.CreateStore(ConstantInt::get(SizeT, 0), localIdVar);
 
     builder.CreateBr(loopBodyEntryBB);
   }
@@ -286,9 +271,7 @@ WorkitemLoops::CreateLoopAround
   if (!WGDynamicLocalSize)
     cmpResult = builder.CreateICmpULT(
                   builder.CreateLoad(localIdVar),
-                    ConstantInt::get(
-                      IntegerType::get(C, size_t_width),
-                      LocalSizeForDim));
+                    ConstantInt::get(SizeT, LocalSizeForDim));
   else
     cmpResult = builder.CreateICmpULT(
                   builder.CreateLoad(localIdVar),
@@ -303,13 +286,18 @@ WorkitemLoops::CreateLoopAround
 
   /* This creation of the identifier metadata is copied from
      LLVM's MDBuilder::createAnonymousTBAARoot(). */
-#ifdef LLVM_OLDER_THAN_3_7
-  MDNode *Dummy = MDNode::getTemporary(C, ArrayRef<Metadata*>());
-#else
+
   MDNode *Dummy = MDNode::getTemporary(C, ArrayRef<Metadata*>()).release();
+#ifdef LLVM_OLDER_THAN_8_0
+  MDNode *Root = MDNode::get(C, Dummy);
+#else
+  MDNode *AccessGroupMD = MDNode::getDistinct(C, {});
+  MDNode *ParallelAccessMD = MDNode::get(
+      C, {MDString::get(C, "llvm.loop.parallel_accesses"), AccessGroupMD});
+
+  MDNode *Root = MDNode::get(C, {Dummy, ParallelAccessMD});
 #endif
 
-  MDNode *Root = MDNode::get(C, Dummy);
   // At this point we have
   //   !0 = metadata !{}            <- dummy
   //   !1 = metadata !{metadata !0} <- root
@@ -318,10 +306,13 @@ WorkitemLoops::CreateLoopAround
   MDNode::deleteTemporary(Dummy);
   // We now have
   //   !1 = metadata !{metadata !1} <- self-referential root
-
   loopBranch->setMetadata("llvm.loop", Root);
-  region.AddParallelLoopMetadata(Root);
 
+#ifdef LLVM_OLDER_THAN_8_0
+  region.AddParallelLoopMetadata(Root);
+#else
+  region.AddParallelLoopMetadata(AccessGroupMD);
+#endif
 
   builder.SetInsertPoint(loopEndBB);
   builder.CreateBr(oldExit);
@@ -376,22 +367,16 @@ WorkitemLoops::ProcessFunction(Function &F)
 
   releaseParallelRegions();
 
-#ifdef LLVM_OLDER_THAN_3_7
-  original_parallel_regions = K->getParallelRegions(LI);
-#else
   original_parallel_regions = K->getParallelRegions(&LI->getLoopInfo());
-#endif
 
 #ifdef DUMP_CFGS
   F.dump();
-  dumpCFG(F, F.getName().str() + "_before_wiloops.dot", 
+  dumpCFG(F, F.getName().str() + "_before_wiloops.dot",
           original_parallel_regions);
 #endif
 
   IRBuilder<> builder(&*(F.getEntryBlock().getFirstInsertionPt()));
-  localIdXFirstVar = 
-    builder.CreateAlloca
-    (IntegerType::get(F.getContext(), size_t_width), 0, ".pocl.local_id_x_init");
+  localIdXFirstVar = builder.CreateAlloca(SizeT, 0, ".pocl.local_id_x_init");
 
   //  F.viewCFGOnly();
 
@@ -402,7 +387,7 @@ WorkitemLoops::ProcessFunction(Function &F)
 
 #if 0
   for (ParallelRegion::ParallelRegionVector::iterator
-           i = original_parallel_regions->begin(), 
+           i = original_parallel_regions->begin(),
            e = original_parallel_regions->end();
        i != e; ++i) 
   {
@@ -516,17 +501,17 @@ WorkitemLoops::ProcessFunction(Function &F)
 
         if (unrollCount > 1) {
             ParallelRegion *prev = original;
-            llvm::BasicBlock *lastBB = 
-                AppendIncBlock(original->exitBB(), localIdX);
+            llvm::BasicBlock *lastBB =
+                AppendIncBlock(original->exitBB(), LocalIdXGlobal);
             original->AddBlockAfter(lastBB, original->exitBB());
             original->SetExitBB(lastBB);
 
             if (AddWIMetadata)
                 original->AddIDMetadata(F.getContext(), 0);
 
-            for (unsigned c = 1; c < unrollCount; ++c) 
+            for (unsigned c = 1; c < unrollCount; ++c)
             {
-                ParallelRegion *unrolled = 
+                ParallelRegion *unrolled =
                     original->replicate(reference_map, ".unrolled_wi");
                 unrolled->chainAfter(prev);
                 prev = unrolled;
@@ -544,49 +529,48 @@ WorkitemLoops::ProcessFunction(Function &F)
     if (WGDynamicLocalSize) {
       GlobalVariable *gv;
       gv = M->getGlobalVariable("_local_size_x");
-      auto *SizeT_Ty = Type::getIntNTy(M->getContext(), size_t_width);
-      if (gv == NULL) 
-        gv = new GlobalVariable(*M, SizeT_Ty, true, GlobalValue::CommonLinkage,
+      if (gv == NULL)
+        gv = new GlobalVariable(*M, SizeT, true, GlobalValue::CommonLinkage,
                                 NULL, "_local_size_x", NULL,
                                 GlobalValue::ThreadLocalMode::NotThreadLocal,
                                 0, true);
 
       l = CreateLoopAround(*original, l.first, l.second, peelFirst,
-                           localIdX, WGLocalSizeX, !unrolled, gv);
+                           LocalIdXGlobal, WGLocalSizeX, !unrolled, gv);
 
       gv = M->getGlobalVariable("_local_size_y");
-      if (gv == NULL) 
-        gv = new GlobalVariable(*M, SizeT_Ty, false, GlobalValue::CommonLinkage,
+      if (gv == NULL)
+        gv = new GlobalVariable(*M, SizeT, false, GlobalValue::CommonLinkage,
                                 NULL, "_local_size_y");
 
       l = CreateLoopAround(*original, l.first, l.second,
-                           false, localIdY, WGLocalSizeY, !unrolled, gv);
+                           false, LocalIdYGlobal, WGLocalSizeY, !unrolled, gv);
 
       gv = M->getGlobalVariable("_local_size_z");
-      if (gv == NULL) 
-        gv = new GlobalVariable(*M, SizeT_Ty, true, GlobalValue::CommonLinkage,
+      if (gv == NULL)
+        gv = new GlobalVariable(*M, SizeT, true, GlobalValue::CommonLinkage,
                                 NULL, "_local_size_z", NULL,
                                 GlobalValue::ThreadLocalMode::NotThreadLocal,
                                 0, true);
 
       l = CreateLoopAround(*original, l.first, l.second,
-                           false, localIdZ, WGLocalSizeZ, !unrolled, gv);
+                           false, LocalIdZGlobal, WGLocalSizeZ, !unrolled, gv);
 
     } else {
       if (WGLocalSizeX > 1) {
-          l = CreateLoopAround(*original, l.first, l.second, peelFirst,
-                               localIdX, WGLocalSizeX, !unrolled);
-        }
+        l = CreateLoopAround(*original, l.first, l.second, peelFirst,
+                             LocalIdXGlobal, WGLocalSizeX, !unrolled);
+      }
 
       if (WGLocalSizeY > 1) {
-          l = CreateLoopAround(*original, l.first, l.second, false,
-                               localIdY, WGLocalSizeY);
-        }
+        l = CreateLoopAround(*original, l.first, l.second, false,
+                             LocalIdYGlobal, WGLocalSizeY);
+      }
 
       if (WGLocalSizeZ > 1) {
-          l = CreateLoopAround(*original, l.first, l.second, false,
-                               localIdZ, WGLocalSizeZ);
-        }
+        l = CreateLoopAround(*original, l.first, l.second, false,
+                             LocalIdZGlobal, WGLocalSizeZ);
+      }
     }
 
     /* Loop edges coming from another region mean B-loops which means 
@@ -612,16 +596,14 @@ WorkitemLoops::ProcessFunction(Function &F)
   for (ParallelRegion::ParallelRegionVector::iterator
            i = original_parallel_regions->begin(), 
            e = original_parallel_regions->end();
-       i != e; ++i) 
+       i != e; ++i)
   {
     ParallelRegion *pr = (*i);
 
     if (!peeledRegion[pr]) continue;
     pr->insertPrologue(0, 0, 0);
     builder.SetInsertPoint(&*(pr->entryBB()->getFirstInsertionPt()));
-    builder.CreateStore
-      (ConstantInt::get(IntegerType::get(F.getContext(), size_t_width), 1), 
-       localIdXFirstVar);       
+    builder.CreateStore(ConstantInt::get(SizeT, 1), localIdXFirstVar);
   }
 
   if (!WGDynamicLocalSize)
@@ -718,11 +700,10 @@ llvm::Value *
 WorkitemLoops::GetLinearWiIndex(llvm::IRBuilder<> &builder, llvm::Module *M,
                                ParallelRegion *region)
 {
-  auto *SizeTType = Type::getIntNTy(M->getContext(), size_t_width);
   GlobalVariable *LocalSizeXPtr =
-    cast<GlobalVariable>(M->getOrInsertGlobal("_local_size_x", SizeTType));
+    cast<GlobalVariable>(M->getOrInsertGlobal("_local_size_x", SizeT));
   GlobalVariable *LocalSizeYPtr =
-    cast<GlobalVariable>(M->getOrInsertGlobal("_local_size_y", SizeTType));
+    cast<GlobalVariable>(M->getOrInsertGlobal("_local_size_y", SizeT));
 
   assert(LocalSizeXPtr != NULL && LocalSizeYPtr != NULL);
 
@@ -769,11 +750,7 @@ WorkitemLoops::AddContextSave
     }
 
   /* Save the produced variable to the array. */
-#ifdef LLVM_OLDER_THAN_3_8
-  BasicBlock::iterator definition = (dyn_cast<Instruction>(instruction));
-#else
   BasicBlock::iterator definition = (dyn_cast<Instruction>(instruction))->getIterator();
-#endif
   ++definition;
   while (isa<PHINode>(definition)) ++definition;
 
@@ -793,7 +770,7 @@ WorkitemLoops::AddContextSave
     }
   else
     {
-      gepArgs.push_back(ConstantInt::get(IntegerType::get(instruction->getContext(), size_t_width), 0));
+      gepArgs.push_back(ConstantInt::get(SizeT, 0));
       gepArgs.push_back(region->LocalIDZLoad());
       gepArgs.push_back(region->LocalIDYLoad());
       gepArgs.push_back(region->LocalIDXLoad());
@@ -803,11 +780,11 @@ WorkitemLoops::AddContextSave
                                                             gepArgs));
 }
 
-llvm::Instruction *
-WorkitemLoops::AddContextRestore
-(llvm::Value *val, llvm::Instruction *alloca, llvm::Instruction *before, 
- bool isAlloca)
-{
+llvm::Instruction *WorkitemLoops::AddContextRestore(llvm::Value *val,
+                                                    llvm::Instruction *alloca,
+                                                    bool PoclWrapperStructAdded,
+                                                    llvm::Instruction *before,
+                                                    bool isAlloca) {
   assert (val != NULL);
   assert (alloca != NULL);
   IRBuilder<> builder(alloca);
@@ -840,20 +817,23 @@ WorkitemLoops::AddContextRestore
     }
   else
     {
-      gepArgs.push_back(ConstantInt::get(IntegerType::get(val->getContext(),
-                                                          size_t_width), 0));
+      gepArgs.push_back(ConstantInt::get(SizeT, 0));
       gepArgs.push_back(region->LocalIDZLoad());
       gepArgs.push_back(region->LocalIDYLoad());
       gepArgs.push_back(region->LocalIDXLoad());
     }
 
-  llvm::Instruction *gep =
-    dyn_cast<Instruction>(builder.CreateGEP(alloca, gepArgs));
-  if (isAlloca) {
-    /* In case the context saved instruction was an alloca, we created a
-       context array with pointed-to elements, and now want to return a pointer 
-       to the elements to emulate the original alloca. */
-    return gep;
+    if (PoclWrapperStructAdded)
+      gepArgs.push_back(
+          ConstantInt::get(Type::getInt32Ty(alloca->getContext()), 0));
+
+    llvm::Instruction *gep =
+        dyn_cast<Instruction>(builder.CreateGEP(alloca, gepArgs));
+    if (isAlloca) {
+      /* In case the context saved instruction was an alloca, we created a
+         context array with pointed-to elements, and now want to return a
+         pointer to the elements to emulate the original alloca. */
+      return gep;
   }
   return builder.CreateLoad(gep);
 }
@@ -863,9 +843,9 @@ WorkitemLoops::AddContextRestore
  * found.
  */
 llvm::Instruction *
-WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
-{
-  
+WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
+                               bool &PoclWrapperStructAdded) {
+  PoclWrapperStructAdded = false;
   /*
    * Unnamed temp instructions need a generated name for the
    * context array. Create one using a running integer.
@@ -915,18 +895,73 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
       elementType = instruction->getType();
     }
 
-  llvm::AllocaInst *Alloca;
   Module* M = instruction->getParent()->getParent()->getParent();
+  const llvm::DataLayout &Layout = M->getDataLayout();
+
+  /* 3D context array. In case the elementType itself is an array or struct,
+   * we must take into account it could be alloca-ed with alignment and loads
+   * or stores might use vectorized instructions expecting proper alignment.
+   * Because of that, we cannot simply allocate x*y*z*(size), we must
+   * enlarge the type to fit the alignment. */
+  Type *AllocType = elementType;
+  AllocaInst *InstCast = dyn_cast<AllocaInst>(instruction);
+  if (InstCast) {
+    unsigned Alignment = InstCast->getAlignment();
+
+    uint64_t StoreSize =
+        Layout.getTypeStoreSize(InstCast->getAllocatedType());
+
+    if ((Alignment > 1) && (StoreSize & (Alignment - 1))) {
+      uint64_t AlignedSize = (StoreSize & (~(Alignment - 1))) + Alignment;
+#ifdef DEBUG_WORK_ITEM_LOOPS
+      std::cerr << "### unaligned type found: aligning " << StoreSize << " to "
+                << AlignedSize << "\n";
+#endif
+      assert(AlignedSize > StoreSize);
+      uint64_t RequiredExtraBytes = AlignedSize - StoreSize;
+
+      if (isa<ArrayType>(elementType)) {
+
+        ArrayType *StructPadding = ArrayType::get(
+            Type::getInt8Ty(M->getContext()), RequiredExtraBytes);
+
+        std::vector<Type *> PaddedStructElements;
+        PaddedStructElements.push_back(elementType);
+        PaddedStructElements.push_back(StructPadding);
+        const ArrayRef<Type *> NewStructElements(PaddedStructElements);
+        AllocType = StructType::get(M->getContext(), NewStructElements, true);
+        PoclWrapperStructAdded = true;
+        uint64_t NewStoreSize = Layout.getTypeStoreSize(AllocType);
+        assert(NewStoreSize == AlignedSize);
+
+      } else if (isa<StructType>(elementType)) {
+        StructType *OldStruct = dyn_cast<StructType>(elementType);
+
+        ArrayType *StructPadding =
+            ArrayType::get(Type::getInt8Ty(M->getContext()), RequiredExtraBytes);
+        std::vector<Type *> PaddedStructElements;
+        for (unsigned j = 0; j < OldStruct->getNumElements(); j++)
+          PaddedStructElements.push_back(OldStruct->getElementType(j));
+        PaddedStructElements.push_back(StructPadding);
+        const ArrayRef<Type *> NewStructElements(PaddedStructElements);
+        AllocType = StructType::get(OldStruct->getContext(), NewStructElements,
+                                    OldStruct->isPacked());
+        uint64_t NewStoreSize = Layout.getTypeStoreSize(AllocType);
+        assert(NewStoreSize == AlignedSize);
+      }
+    }
+  }
+
+  llvm::AllocaInst *Alloca = nullptr;
   if (WGDynamicLocalSize)
     {
       char GlobalName[32];
       GlobalVariable* LocalSize;
       LoadInst* LocalSizeLoad[3];
-      auto *SizeT_Ty = Type::getIntNTy(M->getContext(), size_t_width);
       for (int i = 0; i < 3; ++i) {
         snprintf(GlobalName, 32, "_local_size_%c", 'x' + i);
         LocalSize =
-          cast<GlobalVariable>(M->getOrInsertGlobal(GlobalName, SizeT_Ty));
+          cast<GlobalVariable>(M->getOrInsertGlobal(GlobalName, SizeT));
         LocalSizeLoad[i] = builder.CreateLoad(LocalSize);
       }
 
@@ -937,57 +972,16 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
         builder.CreateBinOp(Instruction::Mul, LocalXTimesY,
                             LocalSizeLoad[2], "num_wi");
 
-      Alloca = builder.CreateAlloca(elementType, NumberOfWorkItems, varName);
+      Alloca = builder.CreateAlloca(AllocType, NumberOfWorkItems, varName);
     }
   else
     {
-      /* 3D context array. In case the elementType itself is an array or struct,
-       * we must take into account it could be alloca-ed with alignment and loads
-       * or stores might use vectorized instructions expecting proper alignment.
-       * Because of that, we cannot simply allocate x*y*z*(size), we must
-       * enlarge the array type to fit the alignment. */
-      Type *allocType = elementType;
-      AllocaInst *allocaInst = dyn_cast<AllocaInst>(instruction);
-      if (allocaInst) {
-        unsigned alignment = allocaInst->getAlignment();
-
-        const DataLayout &dataLayout = M->getDataLayout();
-        uint64_t storeSize =
-          dataLayout.getTypeStoreSize(allocaInst->getAllocatedType());
-
-        if ((alignment > 1) && (storeSize & (alignment - 1))) {
-          uint64_t alignedSize = (storeSize & (~(alignment - 1))) + alignment;
-#ifdef DEBUG_WORK_ITEM_LOOPS
-        std::cerr << "### unaligned type found: aligning " << storeSize
-                  << " to " << alignedSize << "\n";
-#endif
-          if (isa<ArrayType>(elementType)) {
-            allocType =
-              ArrayType::get(elementType->getArrayElementType(), alignedSize);
-          } else if (isa<StructType>(elementType)) {
-            StructType *old_struct = dyn_cast<StructType>(elementType);
-
-            unsigned required_bytes = alignedSize - storeSize;
-            ArrayType *structPadding = ArrayType::get(
-                Type::getInt8Ty(M->getContext()), required_bytes);
-            std::vector<Type *> ary;
-            for (unsigned j = 0; j < old_struct->getNumElements(); j++)
-              ary.push_back(old_struct->getElementType(j));
-            ary.push_back(structPadding);
-            const ArrayRef<Type *> new_el(ary);
-            allocType = StructType::get(old_struct->getContext(), new_el,
-                                        old_struct->isPacked());
-            unsigned newStoreSize = dataLayout.getTypeStoreSize(allocType);
-            assert(newStoreSize == alignedSize);
-          }
-        }
-      }
       llvm::Type *contextArrayType = ArrayType::get(
-          ArrayType::get(ArrayType::get(allocType, WGLocalSizeX), WGLocalSizeY),
+          ArrayType::get(ArrayType::get(AllocType, WGLocalSizeX), WGLocalSizeY),
           WGLocalSizeZ);
 
       /* Allocate the context data array for the variable. */
-      Alloca = builder.CreateAlloca(contextArrayType, 0, varName);
+      Alloca = builder.CreateAlloca(contextArrayType, nullptr, varName);
     }
 
   /* Align the context arrays to stack to enable wide vectors
@@ -1000,7 +994,6 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
   return Alloca;
 }
 
-
 /**
  * Adds context save/restore code for the value produced by the
  * given instruction.
@@ -1021,7 +1014,9 @@ WorkitemLoops::AddContextSaveRestore
 (llvm::Instruction *instruction) {
 
   /* Allocate the context data array for the variable. */
-  llvm::Instruction *alloca = GetContextArray(instruction);
+  bool PoclWrapperStructAdded = false;
+  llvm::Instruction *alloca =
+      GetContextArray(instruction, PoclWrapperStructAdded);
   llvm::Instruction *theStore = AddContextSave(instruction, alloca);
 
   InstructionVec uses;
@@ -1096,10 +1091,11 @@ WorkitemLoops::AddContextSaveRestore
           assert (incomingBB != NULL);
           contextRestoreLocation = incomingBB->getTerminator();
         }
-      llvm::Value *loadedValue = 
-        AddContextRestore
-        (user, alloca, contextRestoreLocation, isa<AllocaInst>(instruction));
-      user->replaceUsesOfWith(instruction, loadedValue);
+        llvm::Value *loadedValue = AddContextRestore(
+            user, alloca, PoclWrapperStructAdded, contextRestoreLocation,
+            isa<AllocaInst>(instruction));
+        user->replaceUsesOfWith(instruction, loadedValue);
+
 #ifdef DEBUG_WORK_ITEM_LOOPS
       std::cerr << "### done, the user was converted to:" << std::endl;
       user->dump();
@@ -1115,41 +1111,41 @@ WorkitemLoops::ShouldNotBeContextSaved(llvm::Instruction *instr)
       problems in conditional branch case where the header node
       of the region is shared across the branches and thus the
       header node's ID loads might get context saved which leads
-      to egg-chicken problems. 
+      to egg-chicken problems.
     */
   if (isa<BranchInst>(instr)) return true;
 
     llvm::LoadInst *load = dyn_cast<llvm::LoadInst>(instr);
     if (load != NULL &&
-        (load->getPointerOperand() == localIdZ ||
-         load->getPointerOperand() == localIdY ||
-         load->getPointerOperand() == localIdX))
+        (load->getPointerOperand() == LocalIdZGlobal ||
+         load->getPointerOperand() == LocalIdYGlobal ||
+         load->getPointerOperand() == LocalIdXGlobal))
       return true;
 
-    VariableUniformityAnalysis &VUA = 
+    VariableUniformityAnalysis &VUA =
       getAnalysis<VariableUniformityAnalysis>();
 
     /* In case of uniform variables (same for all work-items),
        there is no point to create a context array slot for them,
-       but just use the original value everywhere. 
+       but just use the original value everywhere.
 
        Allocas are problematic: they include the de-phi induction
-       variables of the b-loops. In those case each work item 
+       variables of the b-loops. In those case each work item
        has a separate loop iteration variable in the LLVM IR but
        which is really a parallel region loop invariant. But
        because we cannot separate such loop invariant variables
        at this point sensibly, let's just replicate the iteration
        variable to each work item and hope the latter optimizations
        reduce them back to a single induction variable outside the
-       parallel loop.   
+       parallel loop.
     */
     if (!VUA.shouldBePrivatized(instr->getParent()->getParent(), instr)) {
 #ifdef DEBUG_WORK_ITEM_LOOPS
       std::cerr << "### based on VUA, not context saving:";
       instr->dump();
-#endif     
+#endif
       return true;
-    } 
+    }
 
     return false;
 }
@@ -1163,7 +1159,7 @@ WorkitemLoops::AppendIncBlock
   llvm::BasicBlock *oldExit = after->getTerminator()->getSuccessor(0);
   assert (oldExit != NULL);
 
-  llvm::BasicBlock *forIncBB = 
+  llvm::BasicBlock *forIncBB =
     BasicBlock::Create(C, "pregion_for_inc", after->getParent());
 
   after->getTerminator()->replaceUsesOfWith(oldExit, forIncBB);
@@ -1172,11 +1168,10 @@ WorkitemLoops::AppendIncBlock
 
   builder.SetInsertPoint(forIncBB);
   /* Create the iteration variable increment */
-  builder.CreateStore
-    (builder.CreateAdd
-     (builder.CreateLoad(localIdVar),
-      ConstantInt::get(IntegerType::get(C, size_t_width), 1)),
-     localIdVar);
+  builder.CreateStore(builder.CreateAdd(
+                        builder.CreateLoad(localIdVar),
+                        ConstantInt::get(SizeT, 1)),
+                      localIdVar);
 
   builder.CreateBr(oldExit);
 
