@@ -54,12 +54,7 @@ namespace pocl {
 
     llvm::DominatorTree *DT;
     llvm::DominatorTreeWrapperPass *DTP;
-
-#ifdef LLVM_OLDER_THAN_3_7
-    llvm::LoopInfo *LI;
-#else
     llvm::LoopInfoWrapperPass *LI;
-#endif
 
     typedef std::set<llvm::BasicBlock *> BasicBlockSet;
     typedef std::vector<llvm::BasicBlock *> BasicBlockVector;
